# @file
# 
# - Find Cube
# This module looks for Cube support (generic lookup). It will set the following variables:
#  CUBE_INCLUDE_DIR     = where Cube.h can be found
#  CUBE_LIBRARY         = the Cube library to link in
#  CUBE_LIBRARIES       = these are all the needed libraries for Cube this is identical to ${CUBE_LIBRARY}
#
# Note: If CUBE_HOME is set by the user this is the place where the FIND_* macros
#       will look first
#
# @author Blasius Czink
#

include (MarmotHelperMacros)

# show an option for CUBE_HOME
IF (CUBE_HOME)
  # user selected something
  SET(CUBE_HOME ${CUBE_HOME} CACHE PATH "CUBE_HOME")
  # set variables to ...-NOTFOUND to reset (Is there a more elegant way to do that?)
  BC_RESET(CUBE_INCLUDE_DIR)
  BC_RESET(CUBE_LIBRARY)
  BC_RESET(CUBE_LIBRARIES)
  BC_RESET(CUBE_LIBRARY_RELEASE)
  BC_RESET(CUBE_LIBRARY_DEBUG)
ELSE (CUBE_HOME)
  # nothing, so just create an empty field
  SET(CUBE_HOME "" CACHE PATH "CUBE_HOME")
ENDIF (CUBE_HOME)

IF (CUBE_VERSION EQUAL "2")
	SET (CUBE_LIB_NAME "cube")
  # only need libxml2 for cube version 2
  FIND_PACKAGE (LibXml2)
ELSE (CUBE_VERSION EQUAL "2")
	SET (CUBE_LIB_NAME "cube3")
ENDIF (CUBE_VERSION EQUAL "2")

FIND_PATH (CUBE_INCLUDE_DIR Cube.h
          PATHS
          "${CUBE_HOME}/include"
          /usr/include 
          /usr/local/include 
          "$ENV{EXTERNLIBS}/cube/include"
          "$ENV{ProgramFiles}/cube/include"
          DOC "The Cube Header path"
          NO_DEFAULT_PATH
)

FIND_LIBRARY (CUBE_LIBRARY_RELEASE
          NAMES ${CUBE_LIB_NAME}
          PATHS
          ${CUBE_HOME}/lib
          /usr/lib
          /usr/local/lib
          "$ENV{EXTERNLIBS}/cube/lib"
          "$ENV{ProgramFiles}/cube/lib"
)
FIND_LIBRARY (CUBE_LIBRARY_DEBUG
          NAMES "${CUBE_LIB_NAME}d"
          PATHS
          ${CUBE_HOME}/lib
          /usr/lib
          /usr/local/lib
          "$ENV{EXTERNLIBS}/cube/lib"
          "$ENV{ProgramFiles}/cube/lib"
)

SET (CUBE_FOUND "NO")
# was include path found at least?
IF (CUBE_INCLUDE_DIR)
  # adjust libvars & check
  BC_ADJUST_LIB_VARS(CUBE)
  # check
  IF (CUBE_LIBRARY)
    SET (CUBE_FOUND YES)
    ADD_DEFINITIONS (${LIBXML2_DEFINITIONS})
    SET (CUBE_LIBRARIES ${CUBE_LIBRARY} ${LIBXML2_LIBRARIES} CACHE STRING "The CUBE_LIBRARIES" FORCE)
  ENDIF (CUBE_LIBRARY)
ENDIF (CUBE_INCLUDE_DIR)

IF (CUBE_FOUND)
   IF (NOT CUBE_FIND_QUIETLY)
      MESSAGE(STATUS "Found Cube")
   ENDIF (NOT CUBE_FIND_QUIETLY)
ELSE (CUBE_FOUND)
   IF (CUBE_FIND_REQUIRED)
      MESSAGE(FATAL_ERROR "Could not find Cube")
   ENDIF (CUBE_FIND_REQUIRED)
ENDIF (CUBE_FOUND)
