/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**********************************************************************
 * @file   NetUtil.h
 * @brief  Network helper functions and macros
 * 
 * @author Blasius Czink
 * @date   02.01.2003
 **********************************************************************
 * Version History:
 * V 0.1  02.01.2003  : First Revision
 * V 0.2  07.02.2008  : Some small adjustments for MacOS
 **********************************************************************/
#ifndef NET_UTIL_H
#define NET_UTIL_H

#ifdef _WIN32
// windows specific stuff
#include <winsock2.h>
#include <ws2tcpip.h>
#include <windows.h>
// useful defines
typedef SOCKET               net_socket_t;
typedef int                  net_socklen_t;
#define net_closesocket(s)   closesocket(s)

#define NET_INVALID_SOCKET   INVALID_SOCKET
#define NET_SOCKET_ERROR     SOCKET_ERROR
#define NET_EINTR            WSAEINTR
#define NET_EACCES           WSAEACCES
#define NET_EFAULT           WSAEFAULT
#define NET_EINVAL           WSAEINVAL
#define NET_EMFILE           WSAEMFILE
#define NET_EAGAIN           WSAEWOULDBLOCK
#define NET_EWOULDBLOCK      WSAEWOULDBLOCK
#define NET_EINPROGRESS      WSAEINPROGRESS
#define NET_EALREADY         WSAEALREADY
#define NET_ENOTSOCK         WSAENOTSOCK
#define NET_EDESTADDRREQ     WSAEDESTADDRREQ
#define NET_EMSGSIZE         WSAEMSGSIZE
#define NET_EPROTOTYPE       WSAEPROTOTYPE
#define NET_ENOPROTOOPT      WSAENOPROTOOPT
#define NET_EPROTONOSUPPORT  WSAEPROTONOSUPPORT
#define NET_ESOCKTNOSUPPORT  WSAESOCKTNOSUPPORT
#define NET_ENOTSUP          WSAEOPNOTSUPP
#define NET_EPFNOSUPPORT     WSAEPFNOSUPPORT
#define NET_EAFNOSUPPORT     WSAEAFNOSUPPORT
#define NET_EADDRINUSE       WSAEADDRINUSE
#define NET_EADDRNOTAVAIL    WSAEADDRNOTAVAIL
#define NET_ENETDOWN         WSAENETDOWN
#define NET_ENETUNREACH      WSAENETUNREACH
#define NET_ENETRESET        WSAENETRESET
#define NET_ECONNABORTED     WSAECONNABORTED
#define NET_ECONNRESET       WSAECONNRESET
#define NET_ENOBUFS          WSAENOBUFS
#define NET_EISCONN          WSAEISCONN
#define NET_ENOTCONN         WSAENOTCONN
#define NET_ESHUTDOWN        WSAESHUTDOWN
#define NET_ETIMEDOUT        WSAETIMEDOUT
#define NET_ECONNREFUSED     WSAECONNREFUSED
#define NET_EHOSTDOWN        WSAEHOSTDOWN
#define NET_EHOSTUNREACH     WSAEHOSTUNREACH
#define NET_ESYSNOTREADY     WSASYSNOTREADY
#define NET_ENOTINIT         WSANOTINITIALISED
#define NET_HOST_NOT_FOUND   WSAHOST_NOT_FOUND
#define NET_TRY_AGAIN        WSATRY_AGAIN
#define NET_NO_RECOVERY      WSANO_RECOVERY
#define NET_NO_DATA          WSANO_DATA

#else
// rest of the world
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <netdb.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <arpa/inet.h>
#if defined(sun) || defined(__APPLE__)
#include <sys/sockio.h>
#include <sys/filio.h>
#endif

//#define net_socket_t         int
//#define net_socklen_t        socklen_t
typedef int                  net_socket_t;
#ifdef _SX
typedef int                  net_socklen_t;
#define inet_addr            NetUtil::sx_workaround_inetaddr
#else
typedef socklen_t            net_socklen_t;
#endif

#define net_closesocket(s)   ::close(s)
#define NET_INVALID_SOCKET   -1
#define NET_SOCKET_ERROR     -1
//#define SOCKET             long

#define NET_EINTR            EINTR
#define NET_EACCES           EACCES
#define NET_EFAULT           EFAULT
#define NET_EINVAL           EINVAL
#define NET_EMFILE           EMFILE
#define NET_EAGAIN           EAGAIN
#define NET_EWOULDBLOCK      EWOULDBLOCK
#define NET_EINPROGRESS      EINPROGRESS
#define NET_EALREADY         EALREADY
#define NET_ENOTSOCK         ENOTSOCK
#define NET_EDESTADDRREQ     EDESTADDRREQ
#define NET_EMSGSIZE         EMSGSIZE
#define NET_EPROTOTYPE       EPROTOTYPE
#define NET_ENOPROTOOPT      ENOPROTOOPT
#define NET_EPROTONOSUPPORT  EPROTONOSUPPORT
#if defined(ESOCKTNOSUPPORT)
#define NET_ESOCKTNOSUPPORT  ESOCKTNOSUPPORT
#else
#define NET_ESOCKTNOSUPPORT  -1
#endif
#define NET_ENOTSUP          ENOTSUP
#define NET_EPFNOSUPPORT     EPFNOSUPPORT
#define NET_EAFNOSUPPORT     EAFNOSUPPORT
#define NET_EADDRINUSE       EADDRINUSE
#define NET_EADDRNOTAVAIL    EADDRNOTAVAIL
#define NET_ENETDOWN         ENETDOWN
#define NET_ENETUNREACH      ENETUNREACH
#define NET_ENETRESET        ENETRESET
#define NET_ECONNABORTED     ECONNABORTED
#define NET_ECONNRESET       ECONNRESET
#define NET_ENOBUFS          ENOBUFS
#define NET_EISCONN          EISCONN
#define NET_ENOTCONN         ENOTCONN
#if defined(ESHUTDOWN)
#define NET_ESHUTDOWN        ESHUTDOWN
#else
#define NET_ESHUTDOWN        -2
#endif
#define NET_ETIMEDOUT        ETIMEDOUT
#define NET_ECONNREFUSED     ECONNREFUSED
#if defined(EHOSTDOWN)
#define NET_EHOSTDOWN        EHOSTDOWN
#else
#define NET_EHOSTDOWN        -3
#endif
#define NET_EHOSTUNREACH     EHOSTUNREACH
#define NET_ESYSNOTREADY     -4
#define NET_ENOTINIT         -5
#define NET_HOST_NOT_FOUND   HOST_NOT_FOUND
#define NET_TRY_AGAIN        TRY_AGAIN
#define NET_NO_RECOVERY      NO_RECOVERY
#define NET_NO_DATA          NO_DATA

#define SOCKADDR_IN          sockaddr_in
typedef struct sockaddr      SOCKADDR;
typedef struct hostent*      LPHOSTENT;

#endif

#ifdef __APPLE__
#define NET_HAVE_SALEN 1
#endif

#ifndef INADDR_NONE
#define INADDR_NONE 0xFFFFFFFF
#endif


#include <stdio.h>
#include <time.h>
#include <ctype.h>
#include <string>
#include <vector>
#include <memory>

typedef struct hostent * net_hostent_ptr;

/**
 * @class  NetUtil
 * @brief  Singletonclass used internally by SocketLib to initialize WS2_32.DLL (windows specific).
 *         It also provides in it's namespace some useful functions for error handling, resolving etc.
 * 
 * @author  B.C.
 * @date    02.01.2003 
 * @version 0.1
 * @since   0.1
 */
class NetUtil
{
public:
  //! Get the instance of the singleton, create one if none exists yet
  static NetUtil *instance(void);
  //! Same as instance()
  static void initialize(void);

  //! Returns TRUE if the given string is a valid IPv4 in dotted decimal format
  static bool isipv4(const std::string &str);

  //! Resolves hostname to IP or IP to hostname (aliases are considered)
  //! Passing NULL for hostname will return the public IP of localhost
  static bool resolve(const char *hostname, std::vector<std::string> &output);

  //! Similar to resolve() but it just returns a pointer to the first hostent structure
  static net_hostent_ptr getHostEnt(const std::string &hostname);

  //! Like gettimeofday(p, NULL) but also works on windows
  static void getTime(struct timeval *p);

  //! Get the last error code (it returns errno on UNIX and WSAGetLastError() on windows)
  static int  getLastError(void);

  //! Get a descriptive error-string corresponding to the given error-code
  static const char *getStrError(int code);

  //! Returns a string composed of the error-code, error-string and a user specified text
  static std::string composeErrorStr(int err_code, const char *usertxt);

  //! Reverse a string (used by itoa())
  static void strreverse(char *begin, char *end);

  //! Converts an integer to a string
  static void itoa(int value, char *str, int base);

  // Helper method/attributes for DBGOUT macro
  static std::string DBG_FILE;
  static int         DBG_LINE;
  static void myHelperDebug(const char *msg, ...);

#ifdef _SX
  static unsigned long sx_workaround_inetaddr(const char *cp);
#endif

private:
  // No standard and copy-constr.
  NetUtil();
  NetUtil(const NetUtil &) {};
  // No operator=
  NetUtil& operator=(const NetUtil &) { return *this; };
#ifdef _WIN32
	WSADATA         m_WSAData;
#endif
  static NetUtil *m_instance;
};

//! DBGOUT - print a debug message but only if _DEBUG is defined!
//! Usage like printf() but be careful the macro are breakable!
#ifdef _DEBUG
#define DBGOUT   NetUtil::DBG_FILE=__FILE__; NetUtil::DBG_LINE=__LINE__; NetUtil::myHelperDebug
#else
// otherwise replace with "nothing"
inline  void __dummy_dbgout(const char *unusedparam=NULL, ... )  {}
inline  void __dummy_dbgout(std::string &unusedparam, ... )  {}
#define DBGOUT   1 ? (void)0 : __dummy_dbgout
#endif

#endif  // NET_UTIL_H
