/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**********************************************************************
 * @file   SocketAddr.h
 * @brief  Declaration of the socket-address class
 * 
 * @author Blasius Czink
 * @date   02.01.2003
 **********************************************************************
 * Version History:
 * V 0.1  02.01.2003  : First Revision
 **********************************************************************/
#ifndef SOCKET_ADDR_H
#define SOCKET_ADDR_H

#include <stdio.h>
#include <string>
#include <queue>
#include "NetUtil.h"


/**
 * @class  CSocketAddr
 * @brief  Class for socket addresses consisting of IP and port 
 * 
 * @author  B.C.
 * @date    02.01.2003 
 * @version 0.1
 * @since   0.1
 */

class CSocketAddr {
public:
  //! Standard constructor gives an "invalid" address
	CSocketAddr();
  
  //! Creates a SocketAddress from an IP address and a port number.
	//! The IP address must either be a domain name, or it must be in dotted decimal format.
  CSocketAddr(const std::string &host, unsigned short port);
  
  //! Creates a SocketAddress from an IP address or host name and a
  //! port number/service name. Host name/address and port number must
  //! be separated by a colon.
  CSocketAddr(const std::string &hostAndPort);
  
  //! Copy constructor
  CSocketAddr(const CSocketAddr &addr);
  
  //! Creates a CSocketAddr from a native socket address.
  CSocketAddr(const struct sockaddr_in* addr);
	
  //! Destructor
	~CSocketAddr();
		
  //! Assigns another CSocketAddr.
  CSocketAddr& operator=(const CSocketAddr &addr);

  //! Swaps the SocketAddress with another one.	
  void swap(CSocketAddr &addr);

  //! Returns the host IP address.
  std::string getHost() const;

  //! Returns the port number.
  unsigned short getPort() const;

  //! Returns the length of the internal native socket address.	
	net_socklen_t getLength() const;

  //! Returns a pointer to the internal native socket address.
  const struct sockaddr* getAddr() const;
  
  //! Returns the address family (AF_INET) of the address.
  int getAF() const;

  //! Returns a string representation of the address.
  std::string toString() const;

  //! Returns whether socket address is valid
  bool isValid() const;

  //! Maximum length in bytes of a socket address.
  enum { MAX_ADDR_LENGTH = sizeof(struct sockaddr_in)  };

private:
  struct sockaddr_in m_addr;
  bool   m_valid;
};


#endif // SOCKET_ADDR_H
