/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Macros for enhancing MPI-Calls with linenumber and fileinformation
 *
 *  @author Tobias Hilbrich
 *
 *  $Id: enhancempicalls.h 927 2009-02-18 09:24:51Z tobias $
 */

#define MPI_Abort(comm,errorcode) MPI_SOURCEINFO_Abort(comm,errorcode,__LINE__,__FILE__)
#define MPI_Address(location, address) MPI_SOURCEINFO_Address(location, address,__LINE__,__FILE__)
#define MPI_Allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm) MPI_SOURCEINFO_Allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm,__LINE__,__FILE__)
#define MPI_Allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm) MPI_SOURCEINFO_Allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm,__LINE__,__FILE__)
#define MPI_Allreduce(sendbuf, recvbuf, count, datatype, op, comm) MPI_SOURCEINFO_Allreduce(sendbuf, recvbuf, count, datatype, op, comm,__LINE__,__FILE__)
#define MPI_Alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm) MPI_SOURCEINFO_Alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm,__LINE__,__FILE__)
#define MPI_Alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm) MPI_SOURCEINFO_Alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm,__LINE__,__FILE__)
#define MPI_Attr_delete(comm, keyval) MPI_SOURCEINFO_Attr_delete(comm, keyval,__LINE__,__FILE__)
#define MPI_Attr_get(comm, keyval, attribute_val, flag) MPI_SOURCEINFO_Attr_get(comm, keyval, attribute_val, flag,__LINE__,__FILE__)
#define MPI_Attr_put(comm, keyval, attribute_val) MPI_SOURCEINFO_Attr_put(comm, keyval, attribute_val,__LINE__,__FILE__)
#define MPI_Barrier(comm ) MPI_SOURCEINFO_Barrier(comm ,__LINE__,__FILE__)
#define MPI_Bcast(buffer, count, datatype, root, comm ) MPI_SOURCEINFO_Bcast(buffer, count, datatype, root, comm ,__LINE__,__FILE__)
#define MPI_Bsend(buf, count, datatype, dest, tag, comm) MPI_SOURCEINFO_Bsend(buf, count, datatype, dest, tag, comm,__LINE__,__FILE__)
#define MPI_Bsend_init(buf, count, datatype, dest, tag, comm, request) MPI_SOURCEINFO_Bsend_init(buf, count, datatype, dest, tag, comm, request,__LINE__,__FILE__)
#define MPI_Buffer_attach( buffer, size) MPI_SOURCEINFO_Buffer_attach( buffer, size,__LINE__,__FILE__)
#define MPI_Buffer_detach( buffer, size) MPI_SOURCEINFO_Buffer_detach( buffer, size,__LINE__,__FILE__)
#define MPI_Cancel(request) MPI_SOURCEINFO_Cancel(request,__LINE__,__FILE__)
#define MPI_Cart_coords(comm, rank, maxdims, coords) MPI_SOURCEINFO_Cart_coords(comm, rank, maxdims, coords,__LINE__,__FILE__)
#define MPI_Cart_create(comm_old, ndims, dims, periods, reorder, comm_cart) MPI_SOURCEINFO_Cart_create(comm_old, ndims, dims, periods, reorder, comm_cart,__LINE__,__FILE__)
#define MPI_Cart_get(comm, maxdims, dims, periods, coords) MPI_SOURCEINFO_Cart_get(comm, maxdims, dims, periods, coords,__LINE__,__FILE__)
#define MPI_Cart_map(comm, ndims, dims, periods, newrank) MPI_SOURCEINFO_Cart_map(comm, ndims, dims, periods, newrank,__LINE__,__FILE__)
#define MPI_Cart_rank(comm, coords, rank) MPI_SOURCEINFO_Cart_rank(comm, coords, rank,__LINE__,__FILE__)
#define MPI_Cart_shift(comm, direction, disp, rank_source, rank_dest) MPI_SOURCEINFO_Cart_shift(comm, direction, disp, rank_source, rank_dest,__LINE__,__FILE__)
#define MPI_Cart_sub(comm, remain_dims, newcomm) MPI_SOURCEINFO_Cart_sub(comm, remain_dims, newcomm,__LINE__,__FILE__)
#define MPI_Cartdim_get(comm, ndims) MPI_SOURCEINFO_Cartdim_get(comm, ndims,__LINE__,__FILE__)
#define MPI_Comm_compare(comm1,comm2,result) MPI_SOURCEINFO_Comm_compare(comm1,comm2,result,__LINE__,__FILE__)
#define MPI_Comm_create(comm,group,newcomm) MPI_SOURCEINFO_Comm_create(comm,group,newcomm,__LINE__,__FILE__)
#define MPI_Comm_dup(comm,newcomm) MPI_SOURCEINFO_Comm_dup(comm,newcomm,__LINE__,__FILE__)
#define MPI_Comm_free(comm) MPI_SOURCEINFO_Comm_free(comm,__LINE__,__FILE__)
#define MPI_Comm_group(comm,group) MPI_SOURCEINFO_Comm_group(comm,group,__LINE__,__FILE__)
#define MPI_Comm_rank(comm, rank) MPI_SOURCEINFO_Comm_rank(comm, rank,__LINE__,__FILE__)
#define MPI_Comm_remote_group(comm, group) MPI_SOURCEINFO_Comm_remote_group(comm, group,__LINE__,__FILE__)
#define MPI_Comm_remote_size(comm, size) MPI_SOURCEINFO_Comm_remote_size(comm, size,__LINE__,__FILE__)
#define MPI_Comm_size(comm, size) MPI_SOURCEINFO_Comm_size(comm, size,__LINE__,__FILE__)
#define MPI_Comm_split(comm, color, key, newcomm) MPI_SOURCEINFO_Comm_split(comm, color, key, newcomm,__LINE__,__FILE__)
#define MPI_Comm_test_inter(comm, flag) MPI_SOURCEINFO_Comm_test_inter(comm, flag,__LINE__,__FILE__)
#define MPI_Dims_create(nnodes,ndims,dims) MPI_SOURCEINFO_Dims_create(nnodes,ndims,dims,__LINE__,__FILE__)
#define MPI_Errhandler_create(function,errhandler) MPI_SOURCEINFO_Errhandler_create(function,errhandler,__LINE__,__FILE__)
#define MPI_Errhandler_free(errhandler) MPI_SOURCEINFO_Errhandler_free(errhandler,__LINE__,__FILE__)
#define MPI_Errhandler_get(comm,errhandler) MPI_SOURCEINFO_Errhandler_get(comm,errhandler,__LINE__,__FILE__)
#define MPI_Errhandler_set(comm,errhandler) MPI_SOURCEINFO_Errhandler_set(comm,errhandler,__LINE__,__FILE__)
#define MPI_Error_class(errorcode,errorclass) MPI_SOURCEINFO_Error_class(errorcode,errorclass,__LINE__,__FILE__)
#define MPI_Error_string(errorcode,string,resultlen) MPI_SOURCEINFO_Error_string(errorcode,string,resultlen,__LINE__,__FILE__)
#define MPI_Finalize() MPI_SOURCEINFO_Finalize(__LINE__,__FILE__)
#define MPI_Gather(sendbuf,sendcount,sendtype,recvbuf,recvcount,recvtype,root,comm) MPI_SOURCEINFO_Gather(sendbuf,sendcount,sendtype,recvbuf,recvcount,recvtype,root,comm,__LINE__,__FILE__)
#define MPI_Gatherv(sendbuf,sendcount,sendtype,recvbuf,recvcounts,displs,recvtype,root,comm) MPI_SOURCEINFO_Gatherv(sendbuf,sendcount,sendtype,recvbuf,recvcounts,displs,recvtype,root,comm,__LINE__,__FILE__)
#define MPI_Get_count(status,datatype,count) MPI_SOURCEINFO_Get_count(status,datatype,count,__LINE__,__FILE__)
#define MPI_Get_elements(status,datatype,count) MPI_SOURCEINFO_Get_elements(status,datatype,count,__LINE__,__FILE__)
#define MPI_Get_processor_name(name,resultlen) MPI_SOURCEINFO_Get_processor_name(name,resultlen,__LINE__,__FILE__)
#define MPI_Get_version(version,subversion) MPI_SOURCEINFO_Get_version(version,subversion,__LINE__,__FILE__)
#define MPI_Graph_create(comm_old,nnodes,index,edges,reorder,comm_graph) MPI_SOURCEINFO_Graph_create(comm_old,nnodes,index,edges,reorder,comm_graph,__LINE__,__FILE__)
#define MPI_Graph_get(comm,maxindex,maxedges,index,edges) MPI_SOURCEINFO_Graph_get(comm,maxindex,maxedges,index,edges,__LINE__,__FILE__)
#define MPI_Graph_map(comm,nnodes,index,edges,newrank) MPI_SOURCEINFO_Graph_map(comm,nnodes,index,edges,newrank,__LINE__,__FILE__)
#define MPI_Graph_neighbors(comm,rank,maxneighbors,neighbors) MPI_SOURCEINFO_Graph_neighbors(comm,rank,maxneighbors,neighbors,__LINE__,__FILE__)
#define MPI_Graph_neighbors_count(comm,rank,nneighbors) MPI_SOURCEINFO_Graph_neighbors_count(comm,rank,nneighbors,__LINE__,__FILE__)
#define MPI_Graphdims_get(comm,nnodes,nedges) MPI_SOURCEINFO_Graphdims_get(comm,nnodes,nedges,__LINE__,__FILE__)
#define MPI_Group_compare(group1,group2,result) MPI_SOURCEINFO_Group_compare(group1,group2,result,__LINE__,__FILE__)
#define MPI_Group_difference(group1,group2,newgroup) MPI_SOURCEINFO_Group_difference(group1,group2,newgroup,__LINE__,__FILE__)
#define MPI_Group_excl(group,n,ranks,newgroup) MPI_SOURCEINFO_Group_excl(group,n,ranks,newgroup,__LINE__,__FILE__)
#define MPI_Group_free(group) MPI_SOURCEINFO_Group_free(group,__LINE__,__FILE__)
#define MPI_Group_incl(group,n,ranks,newgroup) MPI_SOURCEINFO_Group_incl(group,n,ranks,newgroup,__LINE__,__FILE__)
#define MPI_Group_intersection(group1,group2,newgroup) MPI_SOURCEINFO_Group_intersection(group1,group2,newgroup,__LINE__,__FILE__)
/*#define MPI_Group_range_excl(group,n,ranges,newgroup) MPI_SOURCEINFO_Group_range_excl(group,n,ranges,newgroup,__LINE__,__FILE__)*/
/*#define MPI_Group_range_incl(group,n,ranges,newgroup) MPI_SOURCEINFO_Group_range_incl(group,n,ranges,newgroup,__LINE__,__FILE__)*/
#define MPI_Group_rank(group,rank) MPI_SOURCEINFO_Group_rank(group,rank,__LINE__,__FILE__)
#define MPI_Group_size(group,size) MPI_SOURCEINFO_Group_size(group,size,__LINE__,__FILE__)
#define MPI_Group_translate_ranks(group1,n,ranks1,group2,ranks2) MPI_SOURCEINFO_Group_translate_ranks(group1,n,ranks1,group2,ranks2,__LINE__,__FILE__)
#define MPI_Group_union(group1,group2,newgroup) MPI_SOURCEINFO_Group_union(group1,group2,newgroup,__LINE__,__FILE__)
#define MPI_Ibsend(buf,count,datatype,dest,tag,comm,request) MPI_SOURCEINFO_Ibsend(buf,count,datatype,dest,tag,comm,request,__LINE__,__FILE__)
#define MPI_Init(argc,argv) MPI_SOURCEINFO_Init(argc,argv,__LINE__,__FILE__)
#define MPI_Init_thread(argc,argv,required,provided) MPI_SOURCEINFO_Init_thread(argc,argv,required,provided,__LINE__,__FILE__)
#define MPI_Initialized(flag) MPI_SOURCEINFO_Initialized(flag,__LINE__,__FILE__)
#define MPI_Intercomm_create(local_comm,local_leader,peer_comm,remote_leader,tag,newintercomm) MPI_SOURCEINFO_Intercomm_create(local_comm,local_leader,peer_comm,remote_leader,tag,newintercomm,__LINE__,__FILE__)
#define MPI_Intercomm_merge(intercomm,high,newintracomm) MPI_SOURCEINFO_Intercomm_merge(intercomm,high,newintracomm,__LINE__,__FILE__)
#define MPI_Iprobe(source,tag,comm,flag,status) MPI_SOURCEINFO_Iprobe(source,tag,comm,flag,status,__LINE__,__FILE__)
#define MPI_Irecv(buf,count,datatype,source,tag,comm,request) MPI_SOURCEINFO_Irecv(buf,count,datatype,source,tag,comm,request,__LINE__,__FILE__)
#define MPI_Irsend(buf, count, datatype, dest, tag, comm, request) MPI_SOURCEINFO_Irsend(buf, count, datatype, dest, tag, comm, request,__LINE__,__FILE__)
#define MPI_Isend(buf, count, datatype, dest, tag, comm, request) MPI_SOURCEINFO_Isend(buf, count, datatype, dest, tag, comm, request,__LINE__,__FILE__)
#define MPI_Issend(buf, count,  datatype, dest, tag, comm, request) MPI_SOURCEINFO_Issend(buf, count,  datatype, dest, tag, comm, request,__LINE__,__FILE__)
#define MPI_Keyval_create(copy_fn, delete_fn, keyval, extra_state) MPI_SOURCEINFO_Keyval_create(copy_fn, delete_fn, keyval, extra_state,__LINE__,__FILE__)
#define MPI_Keyval_free(keyval) MPI_SOURCEINFO_Keyval_free(keyval,__LINE__,__FILE__)
#define MPI_Op_create(function, commute, op) MPI_SOURCEINFO_Op_create(function, commute, op,__LINE__,__FILE__)
#define MPI_Op_free( op) MPI_SOURCEINFO_Op_free( op,__LINE__,__FILE__)
#define MPI_Pack(inbuf, incount, datatype, outbuf, outsize, position, comm) MPI_SOURCEINFO_Pack(inbuf, incount, datatype, outbuf, outsize, position, comm,__LINE__,__FILE__)
#define MPI_Pack_size(incount,datatype,comm,size) MPI_SOURCEINFO_Pack_size(incount,datatype,comm,size,__LINE__,__FILE__)
#define MPI_Pcontrol(level, others) MPI_SOURCEINFO_Pcontrol(level,__LINE__,__FILE__, others)
#define MPI_Probe(source,tag,comm, status) MPI_SOURCEINFO_Probe(source,tag,comm, status,__LINE__,__FILE__)
#define MPI_Recv(buf,count,datatype,source,tag,comm,status) MPI_SOURCEINFO_Recv(buf,count,datatype,source,tag,comm,status,__LINE__,__FILE__)
#define MPI_Recv_init(buf,count,datatype,source,tag, comm,request) MPI_SOURCEINFO_Recv_init(buf,count,datatype,source,tag, comm,request,__LINE__,__FILE__)
#define MPI_Reduce(sendbuf,recvbuf,count,datatype, op,root,comm) MPI_SOURCEINFO_Reduce(sendbuf,recvbuf,count,datatype, op,root,comm,__LINE__,__FILE__)
#define MPI_Reduce_scatter(sendbuf,recvbuf,recvcounts,datatype,op,comm) MPI_SOURCEINFO_Reduce_scatter(sendbuf,recvbuf,recvcounts,datatype,op,comm,__LINE__,__FILE__)
#define MPI_Request_free(request) MPI_SOURCEINFO_Request_free(request,__LINE__,__FILE__)
#define MPI_Rsend(buf,count, datatype,dest,tag,comm) MPI_SOURCEINFO_Rsend(buf,count, datatype,dest,tag,comm,__LINE__,__FILE__)
#define MPI_Rsend_init(buf,count,datatype,dest,tag, comm,request) MPI_SOURCEINFO_Rsend_init(buf,count,datatype,dest,tag, comm,request,__LINE__,__FILE__)
#define MPI_Scan(sendbuf,recvbuf,count, datatype, op, comm ) MPI_SOURCEINFO_Scan(sendbuf,recvbuf,count, datatype, op, comm ,__LINE__,__FILE__)
#define MPI_Scatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm) MPI_SOURCEINFO_Scatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm,__LINE__,__FILE__)
#define MPI_Scatterv(sendbuf,sendcounts,displs, sendtype, recvbuf, recvcount, recvtype, root, comm) MPI_SOURCEINFO_Scatterv(sendbuf,sendcounts,displs, sendtype, recvbuf, recvcount, recvtype, root, comm,__LINE__,__FILE__)
#define MPI_Send(buf, count,datatype, dest, tag, comm) MPI_SOURCEINFO_Send(buf, count,datatype, dest, tag, comm,__LINE__,__FILE__)
#define MPI_Send_init(buf, count, datatype, dest, tag, comm, request) MPI_SOURCEINFO_Send_init(buf, count, datatype, dest, tag, comm, request,__LINE__,__FILE__)
#define MPI_Sendrecv(sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status) MPI_SOURCEINFO_Sendrecv(sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status,__LINE__,__FILE__)
#define MPI_Sendrecv_replace(buf,count, datatype, dest, sendtag, source, recvtag, comm, status) MPI_SOURCEINFO_Sendrecv_replace(buf,count, datatype, dest, sendtag, source, recvtag, comm, status,__LINE__,__FILE__)
#define MPI_Ssend(buf, count, datatype, dest, tag, comm) MPI_SOURCEINFO_Ssend(buf, count, datatype, dest, tag, comm,__LINE__,__FILE__)
#define MPI_Ssend_init(buf,count,datatype,dest,tag,comm,request) MPI_SOURCEINFO_Ssend_init(buf,count,datatype,dest,tag,comm,request,__LINE__,__FILE__)
#define MPI_Start(request) MPI_SOURCEINFO_Start(request,__LINE__,__FILE__)
#define MPI_Startall(count,array_of_requests) MPI_SOURCEINFO_Startall(count,array_of_requests,__LINE__,__FILE__)
#define MPI_Test(request, flag, status) MPI_SOURCEINFO_Test(request, flag, status,__LINE__,__FILE__)
#define MPI_Test_cancelled(status, flag) MPI_SOURCEINFO_Test_cancelled(status, flag,__LINE__,__FILE__)
#define MPI_Testall(count, array_of_requests, flag, array_of_statuses) MPI_SOURCEINFO_Testall(count, array_of_requests, flag, array_of_statuses,__LINE__,__FILE__)
#define MPI_Testany(count, array_of_requests, index, flag, status) MPI_SOURCEINFO_Testany(count, array_of_requests, index, flag, status,__LINE__,__FILE__)
#define MPI_Testsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses) MPI_SOURCEINFO_Testsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses,__LINE__,__FILE__)
#define MPI_Topo_test(comm, status) MPI_SOURCEINFO_Topo_test(comm, status,__LINE__,__FILE__)
#define MPI_Type_commit(datatype) MPI_SOURCEINFO_Type_commit(datatype,__LINE__,__FILE__)
#define MPI_Type_contiguous(count,oldtype,newtype) MPI_SOURCEINFO_Type_contiguous(count,oldtype,newtype,__LINE__,__FILE__)
#define MPI_Type_extent(datatype,extent) MPI_SOURCEINFO_Type_extent(datatype,extent,__LINE__,__FILE__)
#define MPI_Type_free(datatype) MPI_SOURCEINFO_Type_free(datatype,__LINE__,__FILE__)
#define MPI_Type_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype) MPI_SOURCEINFO_Type_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype,__LINE__,__FILE__)
#define MPI_Type_hvector(count, blocklength, stride, oldtype, newtype) MPI_SOURCEINFO_Type_hvector(count, blocklength, stride, oldtype, newtype,__LINE__,__FILE__)
#define MPI_Type_indexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype) MPI_SOURCEINFO_Type_indexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype,__LINE__,__FILE__)
#define MPI_Type_lb(datatype, displacement) MPI_SOURCEINFO_Type_lb(datatype, displacement,__LINE__,__FILE__)
#define MPI_Type_size(datatype,size) MPI_SOURCEINFO_Type_size(datatype,size,__LINE__,__FILE__)
#define MPI_Type_struct(count, array_of_blocklengths,array_of_displacements, array_of_types, newtype) MPI_SOURCEINFO_Type_struct(count, array_of_blocklengths,array_of_displacements, array_of_types, newtype,__LINE__,__FILE__)
#define MPI_Type_ub(datatype,displacement) MPI_SOURCEINFO_Type_ub(datatype,displacement,__LINE__,__FILE__)
#define MPI_Type_vector(count, blocklength, stride, oldtype, newtype) MPI_SOURCEINFO_Type_vector(count, blocklength, stride, oldtype, newtype,__LINE__,__FILE__)
#define MPI_Unpack(inbuf, insize, position, outbuf, outcount, datatype,comm) MPI_SOURCEINFO_Unpack(inbuf, insize, position, outbuf, outcount, datatype,comm,__LINE__,__FILE__)
#define MPI_Wait(request, status) MPI_SOURCEINFO_Wait(request, status,__LINE__,__FILE__)
#define MPI_Waitall(count, array_of_requests, array_of_statuses) MPI_SOURCEINFO_Waitall(count, array_of_requests, array_of_statuses,__LINE__,__FILE__)
#define MPI_Waitany(count, array_of_requests, index, status) MPI_SOURCEINFO_Waitany(count, array_of_requests, index, status,__LINE__,__FILE__)
#define MPI_Waitsome(incount,array_of_requests, outcount,array_of_indices, array_of_statuses) MPI_SOURCEINFO_Waitsome(incount,array_of_requests, outcount,array_of_indices, array_of_statuses,__LINE__,__FILE__)
/*#define MPI_Wtick() MPI_SOURCEINFO_Wtick(__LINE__,__FILE__)*/
/*#define MPI_Wtime() MPI_SOURCEINFO_Wtime(__LINE__,__FILE__)*/
