/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Declaration of classes  MPO_Timer and MPO_Timestamp.
 *
 *  
 *  @see mpo-timer.cc (implementation)
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller
 *
 *  $Id: mpo-timer.h 838 2008-06-13 10:44:34Z hpchimml $
 */

#ifndef MPO_TIMER_H
#define MPO_TIMER_H

namespace MPO
{

    //*************************************************************************
    //
    //                          MPO_Timestamp
    //
    /** This class represents the timestamp. 
     *  
     *  @see mpo-timer.cc (implementation)
     */
    class MPO_Timestamp
    {
    public:
        //==========================
        // Constructors/Destructors                            
        //==========================

        /** Constructor taking proper arguments. */
        MPO_Timestamp();

        /** Destructor. */
        ~MPO_Timestamp();

        //==========================
        // Operators                            
        //==========================

        /** Increments timestamp. */
        void operator++();

        /** The conversion to long is for a simple access to comparison 
         *  functions.
         */
        operator  long () const;

    private:
        //==========================
        // Data                           
        //==========================

        long my_value;

    };                          // end class MPO_Timestamp



    //*************************************************************************
    //
    //                          MPO_Timer
    //
    /**
     *  This class provides useful timing routines.
     *  
     *  @see mpo-timer.cc (implementation)   
     */
    class MPO_Timer
    {
    public:
        //==========================
        // Constructors/Destructors                            
        //==========================

        /** The default Constructor. */
        MPO_Timer();

        /** Destructor. */
        ~MPO_Timer();



        //==========================
        // Methods                            
        //==========================

        /** Sleeps the specified number of microseconds. */
        void sleep(long microseconds);

    };                          // end  class MPO_Timer


}                               // end of namespace MPO

#endif
