/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 * @file
 * This Headerfile describes the MPO_LogASCII object.
 * This object recieves MPO_MsgObject objects and creates a logfile from their data.
 * The logfile will be a pur ASCII-text file.
 * @author Tobias Hilbrich
 * @date 29.10.2005
 * $Id: mpo_LogASCII.h 882 2008-11-04 14:04:09Z hpczink $
 */

#ifndef MPO_LOGASCII_H
#define MPO_LOGASCII_H

#include <string>
#include <iostream>
#include <fstream>
#include "MPO_LogFile.h"

using std::string;
using std::ofstream;

namespace MPO 
{
    /**
     * This Objects recieves MPO_MsgObject objects and creates an ASCII logfile from them.
     * The ASCII Log file object is the most simple logfile and should be useable on every 
     * System.
     */
    class MPO_LogASCII : public MPO_LogFile
    {
    protected:
						
        //==========================
        // Data
        //==========================
		
        ofstream   *m_pLog; /**< The ofstream used for logging. */
        bool       m_bInitialized; /**< Flag determines wheter this logfile is initialized. */
        bool       m_bHadAFlush;
			
    public:
        //==========================
        // Constructors/Destructors
        //==========================
		
        MPO_LogASCII (); /**< Constructor that sets state to invalid. */
        MPO_LogASCII (string logFileName); /**< Constructor. */
        ~MPO_LogASCII (); /**< Standard Destructor. */
		
        //==========================
        // Methods own
        //==========================
    public:	
        bool init (string logFileName);
        virtual bool getMsg (MPO_MsgObject &pMsg); /**< Gets a message object, looks for MPI-Standard reference text and calls logMsg. */
        virtual bool flushOfstream (void); /**< Flushs the ofstream in order to prevent lose of data. */
        virtual bool closeLog (void); /**< Flushs the ofstream and closes it. */

    protected:
        bool createOfstream (string logFileName); /**< Initializes the ofstream for the Logfile. */
        virtual bool printHeader (void); /**< Prints the Header of the Logfile like date and so on. */
        virtual bool printTrailer (void); /**< Prints the Trailer of the Logfile (maybe not needed). */
                		
    };//end of class MPO_LogFile

}//end of namespace MPO

#endif







