/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 * @file
 * This Headerfile describes the MPO_LogHTML object.
 * This object recieves MPO_MsgObject objects and creates a logfile 
 * from their data. 
 * The logfile will be a HTML file.
 * @author Tobias Hilbrich
 * @date 18.11.2005
 * $Id: mpo_LogHTML.h 838 2008-06-13 10:44:34Z hpchimml $
 */

#ifndef MPO_LOGHTML_H
#define MPO_LOGHTML_H

#include <string>
#include <iostream>
#include <fstream>
#include "mpo_LogASCII.h"

using std::string;
using std::ofstream;

namespace MPO 
{
    /**
     * This Objects receives MPO_MsgObject objects and creates an HTML logfile from them.
     * The HTML Log file will also be used for Cube logging in order to create info sites.
     * As many methods are equal to the MPO_LogASCII class we will inherit from MPO_LogASCII
     * instead of inheriting from MPO_LogFile.
     */
    class MPO_LogHTML : public MPO_LogASCII
    {
    private:
        //==========================
        // Data
        //==========================
        bool m_bUseStandardReference; /** Flag specifies whether MPI referenciation is allowed or not. */
        bool m_bPrintedTrailer; /** Flaf specifying whether there was a flush call in the past. */
        bool m_bLineEven;
            
    public:
        //==========================
        // Constructors/Destructors
        //==========================
                        
        MPO_LogHTML (string logFileName,bool bUseStandardReference); /**< Constructor. */
                        
        ~MPO_LogHTML (); /**< Standard Destructor. */
                        
        //==========================
        // Methods own
        //==========================
    public:	
        bool init (string logFileName,bool bUseStandardReference);
        bool getMsg (MPO_MsgObject &pMsg); /**< Gets a message object. */
        bool flushOfstream (void); /**< Flushs the ofstream in order to prevent lose of data. */
        
    protected:
        bool printHeader (void); /**< Prints the Header of the Logfile like date and so on. */
        bool printTrailer (void); /**< Prints the Trailer of the Logfile (maybe not needed). */
        bool unflushOfstream (void); /**< Used before printing a message when the ofstream was already flushed. */
                                        
    };//end of class MPO_LogFile

}//end of namespace MPO

#endif







