/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 * @file
 * This Headerfile describes the MPO_LogToServer object.
 * This object recieves MPO_MsgObject objects and sends them to the DebugServer where they
 * should be logged.
 * @author Tobias Hilbrich
 * @date 05.11.2005
 * $Id: mpo_LogToServer.h 838 2008-06-13 10:44:34Z hpchimml $
 */

#ifndef MPO_LOGTOSERVER_H
#define MPO_LOGTOSERVER_H

#include "MPO_LogFile.h"
#include <string>
#include <iostream>
#include <fstream>

using std::string;
using std::ofstream;

namespace MPO 
{
    /**
     * This object recieves and processes MPO_MsgObject objects.
     * This special Logfile implementation just sends the messages it receives to the 
     * DebugServer where they can be logged. This implementation should be used for creating 
     * a single logfile as it will be necessary for ASCII and CUBE logging.
     */
    class MPO_LogToServer : public MPO_LogFile, public MPO_Space
    {
    private:
      		
    public:
        MPO_LogToServer (); /**< Constructor. */
        ~MPO_LogToServer (); /**< Standard destructor. */

    public:
        bool init();
        bool getMsg (MPO_MsgObject &pMsg); /**< Gets a message object. */
        bool closeLog (void); /**< Prepares the Log for Shutdown. */
		
    };//end of class MPO_LogToServer

}//end of namespace MPO

#endif







