/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 * @file
 * This Headerfile describes the MPO_LogToServerDDT object.
 * This object recieves MPO_MsgObject objects and sends them to the DebugServer where they
 * should be logged. Additionaly it uses the mpo-breakpoints.cc and logs to std::cerr in 
 * order to enable DDT on the fly analysis of errors and warnings.
 * @author Tobias Hilbrich
 * @date 25.09.2007
 * $Id$
 */

#ifndef MPO_LOGTOSERVERDDT_H
#define MPO_LOGTOSERVERDDT_H

#include "mpo_LogToServer.h"
#include <string>
#include <iostream>
#include <fstream>

using std::string;
using std::ofstream;

namespace MPO 
{
    /**
     * This object recieves and processes MPO_MsgObject objects.
     * This special Logfile implementation just sends the messages it receives to the 
     * DebugServer where they can be logged. Additionaly it logs errors to stc::cerr and
     * uses the "mpo-breakpoints.cc" file.
     * This implementation should be used for creating a single logfile as it will be 
     * necessary for ASCII and CUBE logging and if you want to use MARMOT + DDT.  
     */
    class MPO_LogToServerDDT : public MPO_LogToServer
    {
    private:
            
    public:
    MPO_LogToServerDDT (); /**< Constructor. */
    ~MPO_LogToServerDDT (); /**< Standard destructor. */

    public: 
    bool init (); /**< Initalizes the object.*/
    bool getMsg (MPO_MsgObject &pMsg); /**< Gets a message object. */
    };//end of class MPO_LogToServerDDT

}//end of namespace MPO

#endif







