/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 * @file
 * This Headerfile describes the MPO_String object.
 * @author Tobias Hilbrich
 * @date 02.11.2005
 * $Id: mpo_String.h 921 2009-02-09 15:54:33Z tobias $
 */

#ifndef MPO_STRING_H
#define MPO_STRING_H

#include "mpo.h"
#include <string>
#include <iostream>
#include <sstream>

using std::string;

namespace MPO
{

    /**
     * This class represents a string used in MARMOT at least used for logging.
     * It provides all the std::string functionality plus the extra functionality to serialize and deserialize it.
     * Note on why such an Object:
     * by creating this object we can firstly pack the serialization and deserialisation code of strings where
     * it belongs to and we create an possibility to implement the mehtods actually provided by std::string ower own
     * this might become important in order to optimize string handling (optimizing speed).
     */
    class MPO_String : public string,public MPO_Object
    {

    public:
        //==========================
        // Constructors/Destructors
        //==========================
        MPO_String (); /**< Creates an empty String. */
        MPO_String (char *str); /**< Creates a String from a pointer to a char array. */
        MPO_String (const char *str); /**< Creates a String from a pointer to a char array. */
        MPO_String (string str); /**< Creates a String from a std::string. */

        ~MPO_String (); /**< Standard Destructor. */


    public:

        //==========================
        // Methods own
        //==========================
        /** Copys the content from the stringstream to this string. */
        void assign (std::stringstream *stream);

        //==========================
        // Methods inherited
        //==========================

        /** @return  the unique call identity set by enum MPO_CALL_ID */
        int getId() const;

        /** @return  Family ID. */
        int getFamilyId() const;

        /** @return  the name of the MPI-call (C or Fortran spelling). */
        string getCallname() const;

        /** Overloads << to print call-description. */
        std::ostream & debugout(std::ostream & output) const;

        /** Serializes an object to ostream.*/
        std::ostream & save(std::ostream & out) const;

        /** Deserializes an object from istream.*/
        std::istream & restore(std::istream & in);

    };//end of class MPO_MsgObject

}//end of namespace MPO

#endif







