/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 * @file
 * This header file describes a class used for MARMOT/TCheck combinations.
 * @author Tobias Hilbrich
 * @date 05.09.2007
 * $Id$
 */

#ifndef MPO_TCHECK_RACES_H
#define MPO_TCHECK_RACES_H

#include "mpo-enums.h"

namespace MPO 
{
    /**
     * This class contins Variables that can cause data races (read - write conflicts 
     * and the like). It is used to create data races on synchronisation Errors like 
     * Probe invalidation, thread level detection and so on. Thread Checker will hopefully
     * detect these races and report them. Thus in order to give a nice Output we will name
     * the variables like the Errors reported.
     * 
     * This class uses the GOF Singleton pattern.
     */
    class MPO_TCheck_Races
    {
        //==========================
        // Data
        //==========================
    protected:
        
        
        static MPO_TCheck_Races *our_instance; /**< The globaly unique instance of this class.*/
        
        int App_Needs_MPI_THREAD_MULTIPLE; /**< Variable used to detect whether MPI_THREAD_MULTIPLE is needed by the application. */
        int App_Performed_MPI_Finalize_to_early; /**< Variable used to detect whether MPI_Finalize is called while other threads are still doing MPI. */
        int App_Wrong_MultiThread_Request_Usage[TCHECK_NUM_REQUEST_SLOTS]; /**< Array for variables that become associated with Requests.*/
        int App_Warn_Probe_Invalidation[TCHECK_NUM_PROBE_SLOTS]; /**< Conflict variable array for Probe invalidation. */
        int App_Comm_In_Two_Collective_Calls[TCHECK_NUM_COMM_SLOTS]; /**< Array for variables that become associated with Comms.*/
    
        //==========================
        // Constructors/Destructors
        //==========================
        
    private: //important for singleton pattern !    
        MPO_TCheck_Races (); /**< Constructor. */
        
    public:
        ~MPO_TCheck_Races (); /**< Standard Destructor. */
        
        //==========================
        // Methods own
        //==========================
    public: 
        static MPO_TCheck_Races* getInstance (void);
        
        void read (MPO_TCHECK_RACES_VARS var);
        void write (MPO_TCHECK_RACES_VARS var);        
        void writeIntForRequests (int id);
        void writeIntForComm (int id);
        void writeIntForProbe (int id);
        void readIntForProbe (int id);
        
    };//end of class MPO_TCheck_Races

}//end of namespace MPO

#endif







