/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
  * \file MPO_ThreadViolationInfo.h
  * This Headerfile describes the class MPO_ThreadViolationInfo.
  * @author Tobias Hilbrich
  * @date 19.07.2007
  * $ID$ 
  */

#ifndef MPO_THREADVIOLATIONINFO_H
#define MPO_THREADVIOLATIONINFO_H

#include <list>
#include "mpo_SourceLocation.h"

namespace MPO 
{

    class MPO_ThreadViolationInfo 
    {
        protected:
            
            /**
             * Typedef for the type we use to store a list of thread ids and source locations.
             */
            typedef std::list<std::pair<int,MPO_SourceLocation> > MPO_ThreadLocationListType;
            
            /**
             * Describes the thread id and the location of all the threads
             * that participated in this violation. 
             */
            MPO_ThreadLocationListType my_participants;
                
        public:
            /**
             * Constructs an empty thread violation info without any participants.
             */
            MPO_ThreadViolationInfo (void);
            
            /**
             * Constructs a thread violation info with one initial participant.
             * You can add further participants with the addParticipant method.
             * @param threadId the id of the participating thread.
             * @param loc the source location of the application that caused the
             *           violation.
             */
            MPO_ThreadViolationInfo (int threadId,MPO_SourceLocation loc);
            
            /**
             * Adds a participating thread to this violation info.
             * @param threadId the id of the participating thread.
             * @param loc the source location of the application that caused the
             *           violation.
             */
            void addParticipant(int threadId,MPO_SourceLocation loc);
            
            /**
             * Gives a list with thread ids and locations containing  
             * for the participants of this thread violation.
             * @return the list with all the participants of this
             *            thread violation. 
             */
            std::list<std::pair<int,MPO_SourceLocation> > getParticipants (void);
            
            /**
             * Gives the amount of participants in this thread violation.
             */
            int getNumParticipants (void);
            
            /**
             * Prints the participants with their thread ids and source 
             * locations to the given ostream.
             * @param out the stream to store the participants in.
             */
            void printParticipants (std::ostream &out);
    };
}

#endif /*MPO_THREADVIOLATIONINFO_H*/

