/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  The enhanced MPI-Call prototypes.
 *
 *  @author Tobias Hilbrich
 *
 *  $Id: sourceinfompicalls.h 921 2009-02-09 15:54:33Z tobias $
 */


#if defined(__cplusplus)
extern "C" {
#endif

int MPI_SOURCEINFO_Abort(MPI_Comm comm, int errorcode,int line,const char *file);
int MPI_SOURCEINFO_Address(void* location, MPI_Aint* address,int line,const char *file);
int MPI_SOURCEINFO_Allgather(void* sendbuf, int sendcount, MPI_Datatype sendtype, void* recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm,int line,const char *file);
int MPI_SOURCEINFO_Allgatherv(void* sendbuf, int sendcount, MPI_Datatype sendtype, void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype, MPI_Comm comm,int line,const char *file);
int MPI_SOURCEINFO_Allreduce(void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm,int line,const char *file);
int MPI_SOURCEINFO_Alltoall(void* sendbuf, int sendcount, MPI_Datatype sendtype, void* recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm,int line,const char *file);
int MPI_SOURCEINFO_Alltoallv(void* sendbuf, int* sendcounts, int* sdispls, MPI_Datatype sendtype, void* recvbuf, int* recvcounts, int* rdispls, MPI_Datatype recvtype, MPI_Comm comm,int line,const char *file);
int MPI_SOURCEINFO_Attr_delete(MPI_Comm comm, int keyval,int line,const char *file);
int MPI_SOURCEINFO_Attr_get(MPI_Comm comm, int keyval, void* attribute_val, int* flag,int line,const char *file);
int MPI_SOURCEINFO_Attr_put(MPI_Comm comm, int keyval, void* attribute_val,int line,const char *file);
int MPI_SOURCEINFO_Barrier(MPI_Comm comm ,int line,const char *file);
int MPI_SOURCEINFO_Bcast(void* buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm ,int line,const char *file);
int MPI_SOURCEINFO_Bsend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm,int line,const char *file);
int MPI_SOURCEINFO_Bsend_init(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request* request,int line,const char *file);
int MPI_SOURCEINFO_Buffer_attach( void* buffer, int size,int line,const char *file);
int MPI_SOURCEINFO_Buffer_detach( void* buffer, int* size,int line,const char *file);
int MPI_SOURCEINFO_Cancel(MPI_Request* request,int line,const char *file);
int MPI_SOURCEINFO_Cart_coords(MPI_Comm comm, int rank, int maxdims, int* coords,int line,const char *file);
int MPI_SOURCEINFO_Cart_create(MPI_Comm comm_old, int ndims, int* dims, int* periods, int reorder, MPI_Comm* comm_cart,int line,const char *file);
int MPI_SOURCEINFO_Cart_get(MPI_Comm comm, int maxdims, int* dims, int* periods, int* coords,int line,const char *file);
int MPI_SOURCEINFO_Cart_map(MPI_Comm comm, int ndims, int* dims, int* periods, int* newrank,int line,const char *file);
int MPI_SOURCEINFO_Cart_rank(MPI_Comm comm, int* coords, int* rank,int line,const char *file);
int MPI_SOURCEINFO_Cart_shift(MPI_Comm comm, int direction, int disp, int* rank_source, int* rank_dest,int line,const char *file);
int MPI_SOURCEINFO_Cart_sub(MPI_Comm comm, int* remain_dims, MPI_Comm* newcomm,int line,const char *file);
int MPI_SOURCEINFO_Cartdim_get(MPI_Comm comm, int* ndims,int line,const char *file);
int MPI_SOURCEINFO_Comm_compare(MPI_Comm comm1,MPI_Comm comm2, int* result,int line,const char *file);
int MPI_SOURCEINFO_Comm_create(MPI_Comm comm, MPI_Group group, MPI_Comm* newcomm,int line,const char *file);
int MPI_SOURCEINFO_Comm_dup(MPI_Comm comm, MPI_Comm* newcomm,int line,const char *file);
int MPI_SOURCEINFO_Comm_free(MPI_Comm* comm,int line,const char *file);
int MPI_SOURCEINFO_Comm_group(MPI_Comm comm, MPI_Group* group,int line,const char *file);
int MPI_SOURCEINFO_Comm_rank(MPI_Comm comm, int* rank,int line,const char *file);
int MPI_SOURCEINFO_Comm_remote_group(MPI_Comm comm, MPI_Group* group,int line,const char *file);
int MPI_SOURCEINFO_Comm_remote_size(MPI_Comm comm, int* size,int line,const char *file);
int MPI_SOURCEINFO_Comm_size(MPI_Comm comm, int* size,int line,const char *file);
int MPI_SOURCEINFO_Comm_split(MPI_Comm comm, int color, int key, MPI_Comm* newcomm,int line,const char *file);
int MPI_SOURCEINFO_Comm_test_inter(MPI_Comm comm, int* flag,int line,const char *file);
int MPI_SOURCEINFO_Dims_create(int nnodes, int ndims, int* dims,int line,const char *file);
int MPI_SOURCEINFO_Errhandler_create(MPI_Handler_function* function, MPI_Errhandler* errhandler,int line,const char *file);
int MPI_SOURCEINFO_Errhandler_free(MPI_Errhandler* errhandler,int line,const char *file);
int MPI_SOURCEINFO_Errhandler_get(MPI_Comm comm, MPI_Errhandler* errhandler,int line,const char *file);
int MPI_SOURCEINFO_Errhandler_set(MPI_Comm comm, MPI_Errhandler errhandler,int line,const char *file);
int MPI_SOURCEINFO_Error_class(int errorcode, int* errorclass,int line,const char *file);
int MPI_SOURCEINFO_Error_string(int errorcode, char* string, int* resultlen,int line,const char *file);
int MPI_SOURCEINFO_Finalize(int line,const char *file);
int MPI_SOURCEINFO_Gather(void* sendbuf, int sendcount, MPI_Datatype sendtype, void* recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm,int line,const char *file);
int MPI_SOURCEINFO_Gatherv(void* sendbuf, int sendcount, MPI_Datatype sendtype, void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype, int root, MPI_Comm comm,int line,const char *file);
int MPI_SOURCEINFO_Get_count(MPI_Status* status, MPI_Datatype datatype, int* count,int line,const char *file);
int MPI_SOURCEINFO_Get_elements(MPI_Status* status, MPI_Datatype datatype, int* count,int line,const char *file);
int MPI_SOURCEINFO_Get_processor_name(char* name, int* resultlen,int line,const char *file);
int MPI_SOURCEINFO_Get_version(int* version, int* subversion,int line,const char *file);
int MPI_SOURCEINFO_Graph_create(MPI_Comm comm_old, int nnodes, int* index, int* edges, int reorder, MPI_Comm* comm_graph,int line,const char *file);
int MPI_SOURCEINFO_Graph_get(MPI_Comm comm, int maxindex, int maxedges, int* index, int* edges,int line,const char *file);
int MPI_SOURCEINFO_Graph_map(MPI_Comm comm, int nnodes, int* index, int* edges, int* newrank,int line,const char *file);
int MPI_SOURCEINFO_Graph_neighbors(MPI_Comm comm, int rank, int maxneighbors, int* neighbors,int line,const char *file);
int MPI_SOURCEINFO_Graph_neighbors_count(MPI_Comm comm, int rank, int* nneighbors,int line,const char *file);
int MPI_SOURCEINFO_Graphdims_get(MPI_Comm comm, int* nnodes, int* nedges,int line,const char *file);
int MPI_SOURCEINFO_Group_compare(MPI_Group group1,MPI_Group group2, int* result,int line,const char *file);
int MPI_SOURCEINFO_Group_difference(MPI_Group group1, MPI_Group group2, MPI_Group* newgroup,int line,const char *file);
int MPI_SOURCEINFO_Group_excl(MPI_Group group, int n, int* ranks, MPI_Group* newgroup,int line,const char *file);
int MPI_SOURCEINFO_Group_free(MPI_Group* group,int line,const char *file);
int MPI_SOURCEINFO_Group_incl(MPI_Group group, int n, int* ranks, MPI_Group* newgroup,int line,const char *file);
int MPI_SOURCEINFO_Group_intersection(MPI_Group group1, MPI_Group group2, MPI_Group* newgroup,int line,const char *file);
int MPI_SOURCEINFO_Group_range_excl(MPI_Group group, int n, int ranges[][3], MPI_Group* newgroup,int line,const char *file);
int MPI_SOURCEINFO_Group_range_incl(MPI_Group group, int n, int ranges[][3], MPI_Group* newgroup,int line,const char *file);
int MPI_SOURCEINFO_Group_rank(MPI_Group group, int* rank,int line,const char *file);
int MPI_SOURCEINFO_Group_size(MPI_Group group, int* size,int line,const char *file);
int MPI_SOURCEINFO_Group_translate_ranks(MPI_Group group1, int n, int* ranks1, MPI_Group group2, int* ranks2,int line,const char *file);
int MPI_SOURCEINFO_Group_union(MPI_Group group1, MPI_Group group2, MPI_Group* newgroup,int line,const char *file);
int MPI_SOURCEINFO_Ibsend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request* request,int line,const char *file);
int MPI_SOURCEINFO_Init(int* argc, char*** argv,int line,const char *file);
int MPI_SOURCEINFO_Init_thread(int* argc, char*** argv,int required,int* provided,int line,const char *file);
int MPI_SOURCEINFO_Initialized(int* flag,int line,const char *file);
int MPI_SOURCEINFO_Intercomm_create(MPI_Comm local_comm, int local_leader, MPI_Comm peer_comm, int remote_leader, int tag, MPI_Comm* newintercomm,int line,const char *file);
int MPI_SOURCEINFO_Intercomm_merge(MPI_Comm intercomm, int high, MPI_Comm* newintracomm,int line,const char *file);
int MPI_SOURCEINFO_Iprobe(int source, int tag, MPI_Comm comm, int* flag, MPI_Status* status,int line,const char *file);
int MPI_SOURCEINFO_Irecv(void* buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request* request,int line,const char *file);
int MPI_SOURCEINFO_Irsend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request* request,int line,const char *file);
int MPI_SOURCEINFO_Isend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request* request,int line,const char *file);
int MPI_SOURCEINFO_Issend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request* request,int line,const char *file);
int MPI_SOURCEINFO_Keyval_create(MPI_Copy_function* copy_fn, MPI_Delete_function* delete_fn, int* keyval, void* extra_state,int line,const char *file);
int MPI_SOURCEINFO_Keyval_free(int* keyval,int line,const char *file);
int MPI_SOURCEINFO_Op_create(MPI_User_function* function, int commute, MPI_Op* op,int line,const char *file);
int MPI_SOURCEINFO_Op_free( MPI_Op* op,int line,const char *file);
int MPI_SOURCEINFO_Pack(void* inbuf, int incount, MPI_Datatype datatype, void* outbuf, int outsize, int* position,  MPI_Comm comm,int line,const char *file);
int MPI_SOURCEINFO_Pack_size(int incount, MPI_Datatype datatype, MPI_Comm comm, int* size,int line,const char *file);
int MPI_SOURCEINFO_Pcontrol(int level,int line,char *file, ...);
int MPI_SOURCEINFO_Probe(int source, int tag, MPI_Comm comm, MPI_Status* status,int line,const char *file);
int MPI_SOURCEINFO_Recv(void* buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status* status,int line,const char *file);
int MPI_SOURCEINFO_Recv_init(void* buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request* request,int line,const char *file);
int MPI_SOURCEINFO_Reduce(void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm,int line,const char *file);
int MPI_SOURCEINFO_Reduce_scatter(void* sendbuf, void* recvbuf, int* recvcounts, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm,int line,const char *file);
int MPI_SOURCEINFO_Request_free(MPI_Request* request,int line,const char *file);
int MPI_SOURCEINFO_Rsend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm,int line,const char *file);
int MPI_SOURCEINFO_Rsend_init(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request* request,int line,const char *file);
int MPI_SOURCEINFO_Scan(void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm ,int line,const char *file);
int MPI_SOURCEINFO_Scatter(void* sendbuf, int sendcount, MPI_Datatype sendtype, void* recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm,int line,const char *file);
int MPI_SOURCEINFO_Scatterv(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype, void* recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm,int line,const char *file);
int MPI_SOURCEINFO_Send(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm,int line,const char *file);
int MPI_SOURCEINFO_Send_init(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request* request,int line,const char *file);
int MPI_SOURCEINFO_Sendrecv(void* sendbuf, int sendcount, MPI_Datatype sendtype, int dest, int sendtag, void* recvbuf, int recvcount, MPI_Datatype recvtype, int source, int recvtag, MPI_Comm comm, MPI_Status* status,int line,const char *file);
int MPI_SOURCEINFO_Sendrecv_replace(void* buf, int count, MPI_Datatype datatype, int dest, int sendtag, int source, int recvtag, MPI_Comm comm, MPI_Status* status,int line,const char *file);
int MPI_SOURCEINFO_Ssend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm,int line,const char *file);
int MPI_SOURCEINFO_Ssend_init(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request* request,int line,const char *file);
int MPI_SOURCEINFO_Start(MPI_Request* request,int line,const char *file);
int MPI_SOURCEINFO_Startall(int count, MPI_Request* array_of_requests,int line,const char *file);
int MPI_SOURCEINFO_Test(MPI_Request* request, int* flag, MPI_Status* status,int line,const char *file);
int MPI_SOURCEINFO_Test_cancelled(MPI_Status* status, int* flag,int line,const char *file);
int MPI_SOURCEINFO_Testall(int count, MPI_Request* array_of_requests, int* flag, MPI_Status* array_of_statuses,int line,const char *file);
int MPI_SOURCEINFO_Testany(int count, MPI_Request* array_of_requests, int* index, int* flag, MPI_Status* status,int line,const char *file);
int MPI_SOURCEINFO_Testsome(int incount, MPI_Request* array_of_requests, int* outcount, int* array_of_indices, MPI_Status* array_of_statuses,int line,const char *file);
int MPI_SOURCEINFO_Topo_test(MPI_Comm comm, int* status,int line,const char *file);
int MPI_SOURCEINFO_Type_commit(MPI_Datatype* datatype,int line,const char *file);
int MPI_SOURCEINFO_Type_contiguous(int count, MPI_Datatype oldtype, MPI_Datatype* newtype,int line,const char *file);
int MPI_SOURCEINFO_Type_extent(MPI_Datatype datatype, MPI_Aint* extent,int line,const char *file);
int MPI_SOURCEINFO_Type_free(MPI_Datatype* datatype,int line,const char *file);
int MPI_SOURCEINFO_Type_hindexed(int count, int* array_of_blocklengths, MPI_Aint* array_of_displacements, MPI_Datatype oldtype, MPI_Datatype* newtype,int line,const char *file);
int MPI_SOURCEINFO_Type_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype* newtype,int line,const char *file);
int MPI_SOURCEINFO_Type_indexed(int count, int* array_of_blocklengths, int* array_of_displacements, MPI_Datatype oldtype, MPI_Datatype* newtype,int line,const char *file);
int MPI_SOURCEINFO_Type_lb(MPI_Datatype datatype, MPI_Aint* displacement,int line,const char *file);
int MPI_SOURCEINFO_Type_size(MPI_Datatype datatype, int* size,int line,const char *file);
int MPI_SOURCEINFO_Type_struct(int count, int* array_of_blocklengths, MPI_Aint* array_of_displacements, MPI_Datatype* array_of_types, MPI_Datatype* newtype,int line,const char *file);
int MPI_SOURCEINFO_Type_ub(MPI_Datatype datatype, MPI_Aint* displacement,int line,const char *file);
int MPI_SOURCEINFO_Type_vector(int count, int blocklength, int stride, MPI_Datatype oldtype, MPI_Datatype* newtype,int line,const char *file);
int MPI_SOURCEINFO_Unpack(void* inbuf, int insize, int* position, void* outbuf, int outcount, MPI_Datatype datatype, MPI_Comm comm,int line,const char *file);
int MPI_SOURCEINFO_Wait(MPI_Request* request, MPI_Status* status,int line,const char *file);
int MPI_SOURCEINFO_Waitall(int count, MPI_Request* array_of_requests, MPI_Status* array_of_statuses,int line,const char *file);
int MPI_SOURCEINFO_Waitany(int count, MPI_Request* array_of_requests, int* index, MPI_Status* status,int line,const char *file);
int MPI_SOURCEINFO_Waitsome(int incount, MPI_Request* array_of_requests, int* outcount, int* array_of_indices, MPI_Status* array_of_statuses,int line,const char *file);
double MPI_SOURCEINFO_Wtick(int line,const char *file);
double MPI_SOURCEINFO_Wtime(int line,const char *file);

#if defined(__cplusplus)
}
#endif
