/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of MPO_Put class.
 *
 *  @author Bettina Krammer
 *
 *  $Id: mpo_Put.cc 838 2008-06-13 10:44:34Z hpchimml $
 */


#include "mpo-2-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Put::MPO_Put()
        : MPO_WinCommunicator(0,
                              -1,
                              MPI_DATATYPE_NULL,
                              MPI_PROC_NULL,
                              -1,
                              -1,
                              MPI_DATATYPE_NULL,
                              MPI_WIN_NULL)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Put::MPO_Put(void *origin_addr, 
                          int origin_count, 
                          MPO_Datatype origin_datatype, 
                          int target_rank, 
                          MPI_Aint target_disp,
                          int target_count, 
                          MPO_Datatype target_datatype, 
                          MPI_Win win)
        : MPO_WinCommunicator(origin_addr,
                              origin_count,
                              origin_datatype,
                              target_rank,
                              target_disp,
                              target_count,
                              target_datatype,
                              win)
    {
    }


    // destructor
    //========================================================================
    MPO_Put::~MPO_Put()
    {
    }


    // getId
    //========================================================================
    int MPO_Put::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_PUT_ID;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Put::callNativeMPI()
    {
        int rvalue = 1;

        checkPrerequisites();
        checkPrerequisites2();
        rvalue = PMPI_Put(my_origin_addr, my_origin_count,
                          my_origin_datatype, my_target_rank,
                          my_target_disp, my_target_count,
                          my_target_datatype, my_win);

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Put::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Put";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_PUT";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Put::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output 
            << getCallname()
            << "(*origin_addr"
            << ", origin_count = "
            << my_origin_count
            << ", origin_datatype = "
            << my_origin_datatype
            << ", target_rank = "
            << my_target_rank
            << ", target_disp = "
            << my_target_disp
            << ", target_count = "
            << my_target_count
            << ", target_datatype = "
            << my_target_datatype
            << ", win = "
            << my_win;

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"  
            << std::endl;

        return output;
    }

    // save
    //========================================================================
    std::ostream & MPO_Put::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        // out < my_origin_addr;
        out < my_origin_count;
        out < my_origin_datatype;
        out < my_target_rank;
        out < my_target_disp;
        out < my_target_count;
        out < my_target_datatype;
        out < my_win;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Put::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        // in > my_origin_addr;
        in > my_origin_count;
        in > my_origin_datatype;
        in > my_target_rank;
        in > my_target_disp;
        in > my_target_count;
        in > my_target_datatype;
        in > my_win;

        return in;
    }

}                               // end of namespace MPO


// mpich implementation
//========================================================================
// The following comment is copied from mpich2-1.0/src/mpi/rma/put.c

/*@
   MPI_Put - Put data into a memory window on a remote process

   Input Parameters:
+ origin_addr -initial address of origin buffer (choice) 
. origin_count -number of entries in origin buffer (nonnegative integer) 
. origin_datatype -datatype of each entry in origin buffer (handle) 
. target_rank -rank of target (nonnegative integer) 
. target_disp -displacement from start of window to target buffer (nonnegative integer) 
. target_count -number of entries in target buffer (nonnegative integer) 
. target_datatype -datatype of each entry in target buffer (handle) 

- win - window object used for communication (handle) 

.N ThreadSafe

.N Fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_WIN
.N MPI_ERR_ARG
.N MPI_ERR_TYPE
.N MPI_ERR_COUNT

int MPI_Put(void *origin_addr, int origin_count, MPI_Datatype
            origin_datatype, int target_rank, MPI_Aint target_disp,
            int target_count, MPI_Datatype target_datatype, MPI_Win
            win)
@*/
