/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of MPO_Win_fence class.
 *
 *  @author Bettina Krammer
 *
 *  $Id: mpo_Win_fence.cc 979 2009-07-17 12:54:04Z hpcdiche $
 */

//#include <bitset> does not exist for all compilers :-(

#include "mpo-2-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"



namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Win_fence::MPO_Win_fence()
        : MPO_Winhandler(MPI_WIN_NULL),
          my_assert(-1)
    {
    	int rank;
    	PMPI_Comm_rank(MPI_COMM_WORLD, &rank);
    	std::cout << rank << " enters fence constructor without arguments" << std::endl;

    }


    // proper constructor
    //========================================================================
    MPO_Win_fence::MPO_Win_fence(int assert,
                               MPI_Win win)
        : my_assert(assert)
    {
    	int rank;
    	PMPI_Comm_rank(MPI_COMM_WORLD, &rank);
    	std::cout << rank << " enters fence constructor with arguments" << std::endl;
		mpo_win_fence_Win=win;
    	if (mpo_win_fence_Win == MPI_WIN_NULL)
    		std::cout << "my_win is MPI_WIN_NULL even after MPI_Win_fence call" << std::endl;
    }



    int MPO_Win_fence::checkPost(){
    	return MARMOT_SUCCESS;
    }

    // destructor
    //========================================================================
    MPO_Win_fence::~MPO_Win_fence()
    {
    }

    // checkWin
    //========================================================================
    int MPO_Win_fence::checkWin()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES.
        int rvalue = MARMOT_SUCCESS;

        // (1) Check if window is null handle.
        if ((MPI_WIN_NULL == mpo_win_fence_Win) &&
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname()
                << ": window is MPI_WIN_NULL!"
                << std::endl;
            rvalue = MARMOT_ERR_WIN_NULL;

            MPO_String text;
            text.assign(&stream);
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation());
            our_log->getMsg(msg);
        }

        // (2) Check if window is valid, i.e. registered in our_wins.

        /*
         * Kiril: comment the next block because I am currently not able to set the window attribute in a proper
         * way before this call (e.g. in MPO_Space ?)
         *
         */
        else if ((our_wins.end() == our_wins.find(mpo_win_fence_Win)) &&
                 (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname()
                << " : Window is not valid!"
                << std::endl;
            rvalue = MARMOT_ERR_WIN_NOT_VALID;

            MPO_String text;
            text.assign(&stream);
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation());
            our_log->getMsg(msg);
        }


        return rvalue;
    }
    // checkAssert
    //========================================================================
    int MPO_Win_fence::checkAssert()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES.
        int rvalue = MARMOT_SUCCESS;

//         int my_size = sizeof(int);
//         bitset<my_size> my_assert_bitset = my_assert;

//         // Check if assert is valid.
//         if (my_assert_bitset.any())
//         {
//             for (int mode = 0; mode < my_size; mode++)
//             {
//                 //
//                 if ( (0 != my_assert_bitset[mode]) &&
//                      (MPI_MODE_NOSTORE != my_assert_bitset[mode]) &&
//                      (MPI_MODE_NOPUT != my_assert_bitset[mode]) &&
//                      (MPI_MODE_NOPRECEDE != my_assert_bitset[mode]) &&
//                      (MPI_MODE_NOSUCCEED != my_assert_bitset[mode]) &&
//                      (MSG_ERR <= DEBUG_MODE) )
//                 {
//                     std::stringstream stream;
//                     stream
//                         << "ERROR: "
//                         << getCallname()
//                         << ": assert is not valid!"
//                         << " Legal values are bit vectors OR of 0 or more of:"
//                         << "MPI_MODE_NOSTORE, "
//                         << "MPI_MODE_NOPUT, "
//                         << "MPI_MODE_NOPROCEDE, "
//                         << "MPI_MODE_NOSUCCEED."
//                         << std::endl;
//                     rvalue = MARMOT_ERR_ASSERT_NOT_VALID;

//                     MPO_String text;
//                     text.assign(&stream);
//                     MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation());
//                     our_log->getMsg(msg);
//                 }
//             }
//         }

        // If MPI_MODE_NOPRECEDE or MPI_MODE_NOSUCCEED are specified
        // by any process of the group, it has to be specified
        // by ALL processes in the group.
        // This is a global check!


        /*
         * If I leave a [MPI_Win_fence-MPI_Win_fence] block, I am allowed to free the window. If I enter the block,
         * I am not allowed to free the window before leaving the block
         * */



        return rvalue;
    }                           // end checkAssert


    // getId
    //========================================================================
    int MPO_Win_fence::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_WIN_FENCE_ID;
    }

    // callNativeMPI
    //========================================================================
    int MPO_Win_fence::callNativeMPI()
    {
        int rvalue = 1;

        checkPrerequisites();
        checkAssert();
        rvalue = PMPI_Win_fence(my_assert, mpo_win_fence_Win );

        std::map < MPI_Win, mpi_win_marmot_states_t>::iterator it =	MPO::MPO_Space::our_wins.find(mpo_win_fence_Win);
    	if (it->second == MPI_WIN_FREE_ALLOWED){
    		it->second = MPI_WIN_FREE_NOT_ALLOWED;
    	}
    	else if (it->second == MPI_WIN_FREE_NOT_ALLOWED){
    		it->second = MPI_WIN_FREE_ALLOWED;
    	}

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Win_fence::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Win_fence";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_WIN_FENCE";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Win_fence::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output
            << getCallname()
            << "(assert = "
            << my_assert
            << ", win = "
            << mpo_win_fence_Win;

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Win_fence::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        //out < my_assert;
        //out < *my_win;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Win_fence::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        //in > my_assert;
        //in > my_win;

        return in;
    }

}                               // end of namespace MPO


// mpich implementation
//========================================================================
// The following comment is copied from mpich2-1.0/src/mpi/rma/win_fence.c

/*@
   MPI_Win_fence - Perform an MPI fence synchronization on a MPI window

   Input Parameters:
+ assert - program assertion (integer)
- win - window object (handle)

   Notes:
   The 'assert' argument is used to indicate special conditions for the
   fence that an implementation may use to optimize the 'MPI_Win_fence'
   operation.  The value zero is always correct.  Other assertion values
   may be or''ed together.  Assertions that are valid for 'MPI_Win_fence' are\:

+ MPI_MODE_NOSTORE - the local window was not updated by local stores
  (or local get or receive calls) since last synchronization.
. MPI_MODE_NOPUT - the local window will not be updated by put or accumulate
  calls after the fence call, until the ensuing (fence) synchronization.
. MPI_MODE_NOPRECEDE - the fence does not complete any sequence of locally
  issued RMA calls. If this assertion is given by any process in the window
  group, then it must be given by all processes in the group.
- MPI_MODE_NOSUCCEED - the fence does not start any sequence of locally
  issued RMA calls. If the assertion is given by any process in the window
  group, then it must be given by all processes in the group.

.N ThreadSafe

.N Fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_OTHER
.N MPI_ERR_WIN

int MPI_Win_fence(int assert, MPI_Win win)
@*/
