/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of MPO_Win_test class.
 *
 *  @author Bettina Krammer
 *
 *  $Id: mpo_Win_test.cc 838 2008-06-13 10:44:34Z hpchimml $
 */


#include "mpo-2-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Win_test::MPO_Win_test()
        : MPO_Winhandler(MPI_WIN_NULL),
          my_flag(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Win_test::MPO_Win_test(MPO_Win win, 
                               int* flag)
        : MPO_Winhandler(win),
          my_flag(flag)
    {
    }


    // destructor
    //========================================================================
    MPO_Win_test::~MPO_Win_test()
    {
    }


    // getId
    //========================================================================
    int MPO_Win_test::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_WIN_TEST_ID;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Win_test::callNativeMPI()
    {
        int rvalue = 1;

        checkPrerequisites();
        rvalue = PMPI_Win_test(my_win, my_flag);

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Win_test::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Win_test";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_WIN_TEST";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Win_test::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output 
            << getCallname()
            << "(win = "
            << my_win
            << " , *flag";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"  
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Win_test::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        out < my_win;
        // out < my_flag;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Win_test::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        in > my_win;
        // in > my_flag;

        return in;
    }

}                               // end of namespace MPO


// mpich implementation
//========================================================================
// The following comment is copied from mpich2-1.0/src/mpi/rma/win_test.c

/*@
   MPI_Win_test - Test whether an RMA exposure epoch has completed

   Input Parameter:
. win - window object (handle) 

   Output Parameter:
. flag - success flag (logical) 

   Notes:
   This is the nonblocking version of 'MPI_Win_wait'.

.N ThreadSafe

.N Fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_WIN
.N MPI_ERR_OTHER
.N MPI_ERR_ARG

.seealso: MPI_Win_wait, MPI_Win_post

int MPI_Win_test(MPI_Win win, int *flag)
@*/
