/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of MPO_Win_wait class.
 *
 *  @author Bettina Krammer
 *
 *  $Id: mpo_Win_wait.cc 838 2008-06-13 10:44:34Z hpchimml $
 */


#include "mpo-2-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Win_wait::MPO_Win_wait()
        : MPO_Winhandler(MPI_WIN_NULL)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Win_wait::MPO_Win_wait(MPO_Win win)
        : MPO_Winhandler(win)
    {
    }


    // destructor
    //========================================================================
    MPO_Win_wait::~MPO_Win_wait()
    {
    }


    // getId
    //========================================================================
    int MPO_Win_wait::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_WIN_WAIT_ID;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Win_wait::callNativeMPI()
    {
        int rvalue = 1;

        checkPrerequisites();
        rvalue = PMPI_Win_wait(my_win);

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Win_wait::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Win_wait";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_WIN_WAIT";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Win_wait::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output 
            << getCallname()
            << "(win = "
            << my_win;

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"  
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Win_wait::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        out < my_win;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Win_wait::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        in > my_win;

        return in;
    }

}                               // end of namespace MPO


// mpich implementation
//========================================================================
// The following comment is copied from mpich2-1.0/src/mpi/rma/win_wait.c

/*@
   MPI_Win_wait - Completes an RMA exposure epoch begun with MPI_Win_post

   Input Parameter:
. win - window object (handle) 

.N ThreadSafe

.N Fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_WIN
.N MPI_ERR_OTHER

int MPI_Win_wait(MPI_Win win)
@*/
