/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of the base class MPO_Communicatorhandler.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: MPO_CommunicatorHandler.cc 838 2008-06-13 10:44:34Z hpchimml $
 */

#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"

namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Communicatorhandler::MPO_Communicatorhandler()
        : my_comm(MPI_COMM_NULL)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Communicatorhandler::MPO_Communicatorhandler(MPO_Comm comm)
        : my_comm(comm)
    {
    }


    // destructor
    //========================================================================
    MPO_Communicatorhandler::~MPO_Communicatorhandler()
    {
    }


    /** There are some pure virtual methods, see mpo.h.
     */


    // getFamilyId
    //========================================================================
    int MPO_Communicatorhandler::getFamilyId() const
    {
        // returns the unique family identity set by enum MPO_FAMILY_ID
        return MPO_CALLOBJECT_ID;
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Communicatorhandler::checkPrerequisites()
    {
        // check correctness of parameters
        checkComm();

        return 0;
    }


    // checkComm
    //========================================================================
    int MPO_Communicatorhandler::checkComm()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        // (1) Check if communicator is MPI_COMM_NULL.
        if ((MPI_COMM_NULL == my_comm) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": Communicator is MPI_COMM_NULL!" 
                << std::endl;
            rvalue = MARMOT_ERR_COMM_NULL;
            
            our_comm_traces->findAndPrintTraceNamed(my_comm,stream,"comm");
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        // (2) Check if communicator is valid.
        else if ((our_comms.end() == our_comms.find(my_comm)) &&
                 (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << " :Communicator is not valid!" 
                << std::endl;
            rvalue = MARMOT_ERR_COMM_NOT_VALID;
            
            our_comm_traces->findAndPrintTraceNamed(my_comm,stream,"comm");
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        return rvalue;
    }                           // end checkComm


}                               // end of namespace MPO
