/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Cart_shift.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Cart_shift.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Cart_shift::MPO_Cart_shift()
        : MPO_Cartesian(MPI_COMM_NULL),
          my_direction(-1),
          my_disp(0),
          my_rank_source(0),
          my_rank_dest(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Cart_shift::MPO_Cart_shift(MPO_Comm communicator,
                                   int direction,
                                   int disp,
                                   int *rank_source,
                                   int *rank_dest)
        : MPO_Cartesian(communicator),
          my_direction(direction),
          my_disp(disp),
          my_rank_source(rank_source),
          my_rank_dest(rank_dest)
    {
    }


    // destructor
    //========================================================================
    MPO_Cart_shift::~MPO_Cart_shift()
    {
    }


    // getId
    //========================================================================
    int MPO_Cart_shift::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_CART_SHIFT_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Cart_shift::checkPost(void)
    {

        return 0;
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Cart_shift::checkPrerequisites(void)
    {


        MPO_Cartesian::checkPrerequisites();

        our_comm_traces->findAndAddUsage(my_comm,getSourceLocation());


        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Cart_shift::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Cart_shift(my_comm, my_direction, my_disp, my_rank_source,
                               my_rank_dest);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Cart_shift::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Cart_shift";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_CART_SHIFT";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Cart_shift::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output
            << getCallname()
            << "(comm = "
            << my_comm
            << ", direction = "
            << my_direction
            << ", disp = "
            << my_disp
            << ", *rank_source, "
            << "*rank_dest";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Cart_shift::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        out < my_comm;
        out < my_direction;
        out < my_disp;
        // out < my_rank_source;
        // out < my_rank_dest;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Cart_shift::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        in > my_comm;
        in > my_direction;
        in > my_disp;
        // in > my_rank_source;
        // in > my_rank_dest;

        return in;
    }


}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied from mpich-1.2.4/src/topol/cart_shift.c.
/*@

MPI_Cart_shift - Returns the shifted source and destination ranks, given a
shift direction and amount

Input Parameters:
+ comm - communicator with cartesian structure (handle)
. direction - coordinate dimension of shift (integer)
- disp - displacement (> 0: upwards shift, < 0: downwards shift) (integer)

Output Parameters:
+ rank_source - rank of source process (integer)
- rank_dest - rank of destination process (integer)

Notes:
The 'direction' argument is in the range '[0,n-1]' for an n-dimensional
Cartesian mesh.

.N fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_TOPOLOGY
.N MPI_ERR_COMM
.N MPI_ERR_ARG

int MPI_Cart_shift (
MPI_Comm comm,
int direction,
int displ,
int *source,
int *dest )
@ */

