/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Comm representing MPI-communicators.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Comm.cc 838 2008-06-13 10:44:34Z hpchimml $
 */



#include "mpo.h"
#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "config.h"

namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Comm::MPO_Comm()
        : my_comm(MPI_COMM_NULL)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Comm::MPO_Comm(MPI_Comm communicator)
    {
        // Our own MPI_COMM_WORLD and all derived communicators 
        // have to exclude the debug server.
        if (communicator == MPI_COMM_WORLD)
        {
            my_comm = our_mpi_comm_world;
        }
        else
        {
            my_comm = communicator;
        }
    }
    

    // destructor
    //========================================================================
    MPO_Comm::~MPO_Comm()
    {
    }


    // MPI_Comm()
    //========================================================================
    MPO_Comm::operator  MPI_Comm() const
    {
        return my_comm;
    }


    // getId
    //========================================================================
    int MPO_Comm::getId() const
    {
        // returns the unique type identity set by enum MPO_TYPE_ID
        return MPO_COMM_ID;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Comm::debugout(std::ostream & output) const
    {
        if (my_comm == (MPO_Comm)MPI_COMM_NULL)
        {
            output << "MPI_COMM_NULL";
        }
        else if (my_comm == (MPO_Comm)MPI_COMM_WORLD)
        {
            output << "MPI_COMM_WORLD";
        }
        else if (my_comm == (MPO_Comm)MPI_COMM_SELF)
        {
            output << "MPI_COMM_SELF";
        }

// TODO // define our_comms_names_insert, see mpo_Init.cc
//         if (our_comms_names.find(my_comm) != our_comms_names.end())
//         {
//             output << our_comms_names.find(my_comm)->second;
//         }
        else
        {
            output << "self-defined communicator";
        }

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Comm::save(std::ostream & out) const
    {
        // we don't save and restore pointers
#ifndef MPI_COMM_NON_BASIC
        out < my_comm;
#endif

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Comm::restore(std::istream & in)
    {
        // we don't save and restore pointers
#ifndef MPI_COMM_NON_BASIC
        in > my_comm;
#endif

        return in;
    }

}                               // end of namespace MPO
