/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Comm_rank.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Comm_rank.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Comm_rank::MPO_Comm_rank()
        : MPO_Communicatorhandler(MPI_COMM_NULL),
          my_rank(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Comm_rank::MPO_Comm_rank(MPO_Comm communicator,
                                 int *rank)
        : MPO_Communicatorhandler(communicator),
          my_rank(rank)
    {
    }


    // destructor
    //========================================================================
    MPO_Comm_rank::~MPO_Comm_rank()
    {
    }


    // getId
    //========================================================================
    int MPO_Comm_rank::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_COMM_RANK_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Comm_rank::checkPost(void)
    {

        return 0;
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Comm_rank::checkPrerequisites(void)
    {


        MPO_Communicatorhandler::checkPrerequisites();

        our_comm_traces->findAndAddUsage(my_comm,getSourceLocation());


        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Comm_rank::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Comm_rank(my_comm, my_rank);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Comm_rank::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Comm_rank";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_COMM_RANK";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Comm_rank::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output
            << getCallname()
            << "(comm = "
            << my_comm
            << ", *rank";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Comm_rank::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        out < my_comm;
        // out < my_rank;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Comm_rank::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        in > my_comm;
        // in > my_rank;

        return in;
    }


}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied
// from mpich-1.2.4/src/context/comm_rank.c.
/*@

MPI_Comm_rank - Determines the rank of the calling process in the
communicator

Input Parameters:
. comm - communicator (handle)

Output Parameter:
. rank - rank of the calling process in group of  'comm'  (integer)

.N fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_COMM

int MPI_Comm_rank (
MPI_Comm comm,
int *rank )
@ */

