/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Comm_remote_size.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Comm_remote_size.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Comm_remote_size::
    MPO_Comm_remote_size()
        : MPO_Intercommunicators(MPI_COMM_NULL),
          my_size(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Comm_remote_size::MPO_Comm_remote_size(MPO_Comm communicator,
                                               int
                                               *size)
        : MPO_Intercommunicators(communicator),
          my_size(size)
    {
    }


    // destructor
    //========================================================================
    MPO_Comm_remote_size::~MPO_Comm_remote_size()
    {
    }


    // getId
    //========================================================================
    int MPO_Comm_remote_size::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_COMM_REMOTE_SIZE_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Comm_remote_size::checkPost(void)
    {

        return 0;
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Comm_remote_size::checkPrerequisites(void)
    {


        MPO_Intercommunicators::checkPrerequisites();

        our_comm_traces->findAndAddUsage(my_comm,getSourceLocation());


        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Comm_remote_size::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Comm_remote_size(my_comm, my_size);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Comm_remote_size::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Comm_remote_size";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_COMM_REMOTE_SIZE";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Comm_remote_size::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output
            << getCallname()
            << "(comm = "
            << my_comm
            << ", *size";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Comm_remote_size::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        out < my_comm;
        // out < my_size;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Comm_remote_size::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        in > my_comm;
        // in > my_size;

        return in;
    }



}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied from
// mpich-1.2.4/src/context/comm_rsize.c.
/*@

MPI_Comm_remote_size - Determines the size of the remote group
associated with an inter-communictor

Input Parameter:
. comm - communicator (handle)

Output Parameter:
. size - number of processes in the group of 'comm'  (integer)

.N fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_COMM
.N MPI_ERR_ARG

int MPI_Comm_remote_size (
MPI_Comm comm,
int *size )
@ */

