/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Datatype representing MPI-datatypes.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Datatype.cc 838 2008-06-13 10:44:34Z hpchimml $
 */



#include "mpo.h"
#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "config.h"

namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Datatype::MPO_Datatype()
        : my_datatype(MPI_DATATYPE_NULL)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Datatype::MPO_Datatype(MPI_Datatype datatype)
    {
        my_datatype = datatype;
    }


    // destructor
    //========================================================================
    MPO_Datatype::~MPO_Datatype()
    {
    }


    // MPI_Datatype
    //========================================================================
    MPO_Datatype::operator  MPI_Datatype() const
    {
        return my_datatype;
    }


    // getId
    //========================================================================
    int MPO_Datatype::getId() const
    {
        // returns the unique type identity set by enum MPO_TYPE_ID
        return MPO_DATATYPE_ID;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Datatype::debugout(std::ostream & output) const
    {
        if (our_datatype_names.find(my_datatype) != our_datatype_names.end())
        {
            output << our_datatype_names.find(my_datatype)->second;
        }
        else
        {
            output << "non-predefined datatype";
        }

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Datatype::save(std::ostream & out) const
    {
        // we don't save and restore pointers
#ifndef MPI_DATATYPE_NON_BASIC
        out < my_datatype;
#endif

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Datatype::restore(std::istream & in)
    {
        // we don't save and restore pointers
#ifndef MPI_DATATYPE_NON_BASIC
        in > my_datatype;
#endif

        return in;
    }


}                               // end of namespace MPO
