/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Group_compare.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Group_compare.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Group_compare::MPO_Group_compare()
        : MPO_Grouphandler(MPI_GROUP_NULL,
                           MPI_GROUP_NULL,
                           -1),
          my_result(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Group_compare::MPO_Group_compare(MPO_Group group,
                                         MPO_Group group2,
                                         int *result)
        : MPO_Grouphandler(group,
                           group2,
                           2),
          my_result(result)
    {
    }


    // destructor
    //========================================================================
    MPO_Group_compare::~MPO_Group_compare()
    {
    }


    // getId
    //========================================================================
    int MPO_Group_compare::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_GROUP_COMPARE_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Group_compare::checkPost(void)
    {

        return 0;
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Group_compare::checkPrerequisites(void)
    {


        MPO_Grouphandler::checkPrerequisites();

        our_group_traces->findAndAddUsage(my_group,getSourceLocation());
        our_group_traces->findAndAddUsage(my_group2,getSourceLocation());


        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Group_compare::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Group_compare(my_group, my_group2, my_result);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Group_compare::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Group_compare";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_GROUP_COMPARE";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Group_compare::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output
            << getCallname()
            << "(group1 = "
            << my_group
            << ", group2 = "
            << my_group2
            << ", *result";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Group_compare::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        out < my_group;
        out < my_group2;
        // out < my_result;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Group_compare::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        in > my_group;
        in > my_group2;
        // in > my_result;

        return in;
    }


}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied from
// mpich-1.2.4/src/context/groupcompare.c.
/*@

MPI_Group_compare - Compares two groups

Input Parameters:
+ group1 - group1 (handle)
- group2 - group2 (handle)

Output Parameter:
. result - integer which is 'MPI_IDENT' if the order and members of
the two groups are the same, 'MPI_SIMILAR' if only the members are the
same, and 'MPI_UNEQUAL' otherwise

.N fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_GROUP
.N MPI_ERR_ARG

int MPI_Group_compare (
MPI_Group group1,
MPI_Group group2,
int *result )
@ */


