/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Group_size.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Group_size.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Group_size::MPO_Group_size()
        : MPO_Grouphandler(MPI_GROUP_NULL,
                           MPI_GROUP_NULL,
                           -1),
          my_size(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Group_size::MPO_Group_size(MPO_Group group,
                                   int *size)
        : MPO_Grouphandler(group,
                           MPI_GROUP_NULL,
                           1),
          my_size(size)
    {
    }


    // destructor
    //========================================================================
    MPO_Group_size::~MPO_Group_size()
    {
    }


    // getId
    //========================================================================
    int MPO_Group_size::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_GROUP_SIZE_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Group_size::checkPost(void)
    {

        return 0;
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Group_size::checkPrerequisites(void)
    {

        MPO_Grouphandler::checkPrerequisites();

        our_group_traces->findAndAddUsage(my_group,getSourceLocation());


        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Group_size::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Group_size(my_group, my_size);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Group_size::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Group_size";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_GROUP_SIZE";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Group_size::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output
            << getCallname()
            << "(group = "
            << my_group
            << ", *size";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Group_size::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        out < my_group;
        // out < my_size;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Group_size::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        in > my_group;
        // in > my_size;

        return in;
    }


}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied from
// mpich-1.2.4/src/context/group_size.c.
/*@

MPI_Group_size - Returns the size of a group

Input Parameters:
+ group - group (handle)
Output Parameter:
- size - number of processes in the group (integer)

.N fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_GROUP
.N MPI_ERR_ARG

int MPI_Group_size (
MPI_Group group,
int *size )
@ */

