/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Initialized.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller
 *
 *  $Id: mpo_Initialized.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"


namespace MPO
{

    // MPI_Initialized can be called before MPI_Init and after MPI_Finalize.


    // default constructor
    //========================================================================
    MPO_Initialized::MPO_Initialized()
        : my_flag(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Initialized::MPO_Initialized(int *flag)
        : my_flag(flag)
    {
    }


    // destructor
    //========================================================================
    MPO_Initialized::~MPO_Initialized()
    {
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Initialized::checkPrerequisites()
    {


        // currently no checks implemented


        return 0;
    }


    // getId
    //========================================================================
    int MPO_Initialized::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_INITIALIZED_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Initialized::checkPost(void)
    {

        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Initialized::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Initialized(my_flag);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Initialized::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Initialized";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_INITIALIZED";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Initialized::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output
            << getCallname()
            << "(*flag";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Initialized::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        // out < my_flag;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Initialized::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        // in > my_flag;

        return in;
    }


}                               // end of namespace MPO




// mpich implementation
//========================================================================
// The following comment is copied from mpich-1.2.4/src/env/initialize.c.
/*@
  MPI_Initialized - Indicates whether 'MPI_Init' has been called.

  Output Parameter:
  . flag - Flag is true if 'MPI_Init' has been called and false otherwise.

  .N fortran

  int MPI_Initialized( int *flag )
  @ */

