/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Keyval_create.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Keyval_create.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Keyval_create::MPO_Keyval_create()
        : my_copy_fn(0),
          my_delete_fn(0), 
          my_keyval(0), 
          my_extra_state(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Keyval_create::MPO_Keyval_create(MPI_Copy_function * copy_fn,
                                         MPI_Delete_function * delete_fn,
                                         int *keyval,
                                         void *extra_state)
        : my_copy_fn(copy_fn),
          my_delete_fn(delete_fn), 
          my_keyval(keyval),
          my_extra_state(extra_state)
    {
    }


    // destructor
    //========================================================================
    MPO_Keyval_create::~MPO_Keyval_create()
    {
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Keyval_create::checkPrerequisites()
    {
        
        

        // currently no checks implemented    

        return 0;
    }


    // getId
    //========================================================================
    int MPO_Keyval_create::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_KEYVAL_CREATE_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Keyval_create::checkPost(void)
    {
             
        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Keyval_create::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Keyval_create(my_copy_fn, my_delete_fn, my_keyval,
                                    my_extra_state);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif                            

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Keyval_create::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Keyval_create";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_KEYVAL_CREATE";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Keyval_create::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output 
            << getCallname()
            << "(*copy_fn, "
            << "*delete_fn, "
            << "*keyval, "
            << "*extra_state";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"  
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Keyval_create::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        // out < my_copy_fn;
        // out < my_delete_fn;
        // out < my_keyval;
        // out < my_extra_state;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Keyval_create::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        // in > my_copy_fn;
        // in > my_delete_fn;
        // in > my_keyval;
        // in > my_extra_state;

        return in;
    }


}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied from 
// mpich-1.2.4/src/context/keyvalcreate.c.
/*@

MPI_Keyval_create - Generates a new attribute key

Input Parameters:
. copy_fn - Copy callback function for 'keyval' 
. delete_fn - Delete callback function for 'keyval' 
. extra_state - Extra state for callback functions 

Output Parameter:
. keyval - key value for future access (integer) 

Notes:
Key values are global (available for any and all communicators).

There are subtle differences between C and Fortran that require that the
copy_fn be written in the same language that 'MPI_Keyval_create'
is called from.
This should not be a problem for most users; only programers using both 
Fortran and C in the same program need to be sure that they follow this
rule.

.N fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_EXHAUSTED
.N MPI_ERR_ARG


int MPI_Keyval_create ( 
MPI_Copy_function *copy_fn, 
MPI_Delete_function *delete_fn, 
int *keyval, 
void *extra_state )

@ */

