/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 * @file mpo_LogQueue.cc
 * Implements the queue logger.
 *
 * @author Tobias Hilbrich
 * @date 04.12.2008
 * $ID$
 */

#include "mpo_LogQueue.h"

namespace MPO
{
    /**
     * Constructur.
     */
    MPO_LogQueue::MPO_LogQueue()
        : m_queue ()
    {
        /*Init referencing ...*/
        initMPIStandardReferencer();

        /*Init socket*/
        initSocket();
    }

    /**
     * Destructor.
     */
    MPO_LogQueue::~MPO_LogQueue ()
    {
        /*Close log, just in case*/
        closeLog ();
    }

    /**
     * Passes a message to the logger.
     * @param pMsg the message to log.
     * @return true if successful, false otherwise.
     */
    bool MPO_LogQueue::getMsg (MPO_MsgObject &pMsg)
    {
        // if there is a socket send msg
        MARMOT_ERR_CODES err = pMsg.getErrorCode ();
        const char *temp = NULL;
        if ( (err != MARMOT_SUCCESS) &&
             (err != MARMOT_ERR_UNDEFINED) &&
             (err != MARMOT_ERR_MARMOT) &&
             (err != MARMOT_LAST_ERR_CODE)    )
        {
            temp = getLocalPathToMPIStandard(err);
        }
        else
        {
            temp = "";
        }


        pMsg.writeToSocket(getLogSocket(), temp);

        // wait for ACK from log-host
        if (pMsg.getMsgType() == MSG_TYPE_ERROR)
            CSocketLogger::instance()->waitForAck();

        //Flush if necessary
        //Not possible for this logger !
        /*if ((pMsg.getMsgType() == MSG_TYPE_ERROR) || bWasTrailerPrinted || (LOGFLUSH_TYPE == LOGFILE_FLUSH_TYPE_IMMEDIATE))*/

        std::string temp_string (temp);
        m_queue.push(std::make_pair(pMsg,temp_string));
        return true;
    }

    /**
     * Closes the log.
     * @return true if successful, false otherwise.
     */
    bool MPO_LogQueue::closeLog (void)
    {
        while (m_queue.size())
            m_queue.pop ();
        return true;
    }

    /**
     * Asks the logger whether it has a message in
     * its queue.
     * @return true if message in queue, false otherwise.
     */
    bool MPO_LogQueue::hasMsg (void)
    {
      return ((m_queue.size()) ? true : false);
    }

    /**
     * Returns number of messages in queue.
     * @return number of messages in queue.
     */
    int MPO_LogQueue::numMsgs (void)
    {
        return m_queue.size();
    }

    /**
     * Returns a message from the logger queue.
     * @return first message in the queue if existent,
     *         otherwise MPO_MsgObject ();
     */
    std::pair<MPO_MsgObject,std::string> MPO_LogQueue::popMsg (void)
    {
        if (!m_queue.size())
            return std::make_pair(MPO_MsgObject(),std::string(""));

        std::pair<MPO_MsgObject,std::string> front;
        front = m_queue.front ();
        m_queue.pop ();

        return front;
    }
}

/*EOF*/
