/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 * @file
 * This Sourcefile implements the MPO_LogToServer object.
 * This object recieves MPO_MsgObject objects and sends them to the DebugServer where they
 * should be logged.
 * @author Tobias Hilbrich
 * @date 05.11.2005
 * $Id: mpo_LogToServer.cc 838 2008-06-13 10:44:34Z hpchimml $
 */

#include "mpo_LogToServer.h"

namespace MPO 
{
    /**
     * Constructor. By Tobias, Date 05.11.2005
     */
    MPO_LogToServer::MPO_LogToServer ()
    {
        init();
    }
    
    /**
     * Constructor. By Tobias, Date 05.11.2005
     */
    bool MPO_LogToServer::init ()
    {
        // nothing to be done here !
        return true;
    }
   
    /**
     * Standard destructor. By Tobias, Date 05.11.2005
     */
    MPO_LogToServer::~MPO_LogToServer ()
    {
        // nothing to be done here !
    }
   
    /**
     * Gets a message object. By Tobias, Date 05.11.2005
     */
    bool MPO_LogToServer::getMsg (MPO_MsgObject &pMsg)
    {
        
        //check whether MPI_FINALIZE was already called
        if (our_finalize_was_called == true)
        {
            //we MUST NOT do any MPI Calls to pass the message to the DebugServer
            //so we will print the error to std::cerr
            pMsg.debugout(std::cerr);
            std::cerr << std::endl;
            std::cerr.flush();
            return true;
        }
        
#ifdef MARMOT_USE_THREADSYNC
        /**
         * To speedup synchronization in the MPI_THREAD_MULTIPLE case we leave synchronization here.
         */
        if (MPO_Space::getProvidedThreadLevel() >= MPI_THREAD_MULTIPLE)
            MPO_Space::getSyncObject()->enterPMPI();
#endif
        
        //Send the message to the Server
        our_channel.sendId(pMsg.getFamilyId(),-1);
        our_channel.sendId(pMsg.getId(),-1);
        our_channel.sendObject(pMsg,-1);
      
        // wait for reply from server
        /**
         * we will only wait for error messages !!!
         */
        if (pMsg.getMsgType() == MSG_TYPE_ERROR)
        {
            int server_token = -1; 
            our_channel.receiveId(server_token,-1);
        }
        
#ifdef MARMOT_USE_THREADSYNC
        if (MPO_Space::getProvidedThreadLevel() >= MPI_THREAD_MULTIPLE)
            MPO_Space::getSyncObject()->leavePMPI();
#endif
      
        return (true);
    }
   
    /**
     * Prepares the Log for Shutdown. By Tobias, Date 05.11.2005
     * @return true
     */
    bool MPO_LogToServer::closeLog (void) 
    {
        //Nothing to be done here
        return true;
    }
		
}//end of namespace MPO







