/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Op representing MPI-operators.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Op.cc 838 2008-06-13 10:44:34Z hpchimml $
 */



#include "mpo.h"
#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "config.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Op::MPO_Op()
        : my_op(MPI_OP_NULL)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Op::MPO_Op(MPI_Op op)
    {
        my_op = op;
    }


    // destructor
    //========================================================================
    MPO_Op::~MPO_Op()
    {
    }


    // MPI_Op
    //========================================================================
    MPO_Op::operator MPI_Op()const
    {
        return my_op;
    }


    // getId
    //========================================================================
    int MPO_Op::getId() const
    {
        // returns the unique call identity set by enum MPO_TYPE_ID
        return MPO_OP_ID;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Op::debugout(std::ostream & output) const
    {
        if (our_op_names.find(my_op) != our_op_names.end())
        {
            output << our_op_names.find(my_op)->second;
        }
        else
        {
            output << "non-predefined operator";
        }

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Op::save(std::ostream & out) const
    {
        // we don't save and restore pointers
#ifndef MPI_OP_NON_BASIC
        out < my_op;
#endif

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Op::restore(std::istream & in)
    {
        // we don't save and restore pointers
#ifndef MPI_OP_NON_BASIC
        in > my_op;
#endif

        return in;
    }

}                               // end of namespace MPO
