/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 * @file
 * This Sourcefile implements the MPO_SourceLocation class.
 * A MPO_SourceLocation object describes a source-cod Location.
 * @author Tobias Hilbrich
 * @date 25.10.2005
 * $Id: mpo_SourceLocation.cc 921 2009-02-09 15:54:33Z tobias $
 */

#include "mpo_SourceLocation.h"
#include "mpo-binaryIO.h"
#include "mpo-enums.h"
#include <string.h>

namespace MPO
{
    MPO_SourceLocation::MPO_SourceLocation (char* fileName,int codeline) :
        myFileName(fileName),myLine(codeline),myFileNameMem(NULL),my_extra_id(0)
    {
    }

    MPO_SourceLocation::MPO_SourceLocation (std::string fileName,int codeline) :
        myLine(codeline),my_extra_id(0)
    {
   	    myFileNameMem = new char[fileName.length()+1];
    	myFileName   = myFileNameMem;
    	strcpy(myFileName,fileName.c_str());
    }

    MPO_SourceLocation::MPO_SourceLocation (const char* fileName,int codeline) :
        myLine(codeline),my_extra_id(0)
    {
        int len = strlen(fileName);
        myFileNameMem = new char[len+1];
        myFileName = myFileNameMem;
        strcpy(myFileNameMem,fileName);
    }

    MPO_SourceLocation::MPO_SourceLocation (const char* fileName,int codeline, unsigned long long extra_id) :
            myLine(codeline),my_extra_id(extra_id)
    {
        int len = strlen(fileName);
        myFileNameMem = new char[len+1];
        myFileName = myFileNameMem;
        strcpy(myFileNameMem,fileName);
    }

    MPO_SourceLocation::MPO_SourceLocation (char* fileName,int codeline, unsigned long long extra_id) :
        myFileName(fileName),myLine(codeline),myFileNameMem(NULL),my_extra_id(extra_id)
    {
    }

    MPO_SourceLocation::MPO_SourceLocation (std::string fileName,int codeline, unsigned long long extra_id) :
        myLine(codeline),my_extra_id(extra_id)
    {
        myFileNameMem = new char[fileName.length()+1];
        myFileName   = myFileNameMem;
        strcpy(myFileName,fileName.c_str());
    }

    MPO_SourceLocation::MPO_SourceLocation () :
        myFileName(NULL),myLine(0),myFileNameMem(NULL)
    {
    }

    MPO_SourceLocation::MPO_SourceLocation(const MPO_SourceLocation &l) :
        myFileName(NULL),myLine(0),myFileNameMem(NULL)
    {
        operator= (l);
    }

    MPO_SourceLocation::~MPO_SourceLocation ()
    {
        if (myFileNameMem)
            delete (myFileNameMem);
        myFileNameMem = NULL;

        myFileName = NULL;

        myLine = 0;
    }

    const char* MPO_SourceLocation::getFileName (void) const
    {
        if (myFileName != NULL)
            return (myFileName);

        return "";
    }

    int MPO_SourceLocation::getCodeline (void) const
    {
        return (myLine);
    }

    unsigned long long MPO_SourceLocation::getExtraID (void) const
    {
        return (my_extra_id);
    }

    bool MPO_SourceLocation::isValidLocation (void) const
    {
        if (myLine == 0)
            return (false);
        else return (true);
    }

    int MPO_SourceLocation::getId() const
    {
        return (MPO_SOURCELOCATION);
    }

    int MPO_SourceLocation::getFamilyId() const
    {
        return (MPO_MESSAGEOBJECT_ID);
    }

    string MPO_SourceLocation::getCallname() const
    {
        return ("MPO_SourceLocation");
    }

    std::ostream & MPO_SourceLocation::debugout(std::ostream & output) const
    {

        if (myLine != 0)
            output << myFileName << " line: "<<myLine;
        else output << "Unknown";

        //Additional output for memory mangling
        //output << " Mem: " << (unsigned long) myFileNameMem << " Name: " <<(unsigned long) myFileName << std::endl;

        return (output);
    }

    std::ostream & MPO_SourceLocation::save(std::ostream & out) const
    {
        if (myLine != 0)
        {
            out < myLine;
            for (int i = 0;(myFileName[i] != '\0')&&(i<FILENAME_LEN-1);i++) //<FILENAME_LEN forces max length
                out < myFileName[i];
            out < '\0';
        }
        else
            out < 0;

        out < my_extra_id;

        return(out);
    }

    std::istream & MPO_SourceLocation::restore(std::istream & in)
    {
        in > myLine;

        if (myLine != 0)
        {
            int i = 0;
            myFileNameMem = new char[FILENAME_LEN]; //using fixed length for performance purposes
            do
            {
                in > myFileNameMem[i];
            }
            while (myFileNameMem[i++] != '\0');
            myFileName = myFileNameMem;
        }

        in > my_extra_id;

        return(in);
    }

    void MPO_SourceLocation::operator = (const MPO_SourceLocation &l)
    {
        if ((myLine != 0) &&
            (myFileNameMem)  )
            delete myFileNameMem;
        myFileNameMem = myFileName = NULL;

        myLine  = l.getCodeline();

        if ((myLine > 0) && l.getFileName())
        {
            int len = strlen(l.getFileName());

            myFileNameMem = new char[len+1];
            strcpy(myFileNameMem,l.getFileName());

            myFileName    = myFileNameMem;
        }

        my_extra_id = l.getExtraID();
    }

}//end of namespace MPO







