/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 * @file
 * This sourcefile implements the MPO_String object.
 * @author Tobias Hilbrich
 * @date 02.11.2005
 * $Id: mpo_String.cc 921 2009-02-09 15:54:33Z tobias $
 */

#include "mpo_String.h"
#include "mpo-enums.h"
#include "mpo-binaryIO.h"


namespace MPO
{

    /**
     * Creates an empty String. By Tobias, Date 02.11.2005.
     */
    MPO_String::MPO_String () : string ()
    {

    }

    /**
     * Creates a String from a pointer to a char array. By Tobias, Date 02.11.2005.
     */
    MPO_String::MPO_String (char *str) : string (str)
    {

    }

    /**
     * Creates a String from a pointer to a char array. By Tobias, Date 02.11.2005.
     */
    MPO_String::MPO_String (const char *str) : string (str)
    {

    }

    /**
     * Creates a String from a std::string. By Tobias, Date 02.11.2005.
     */
    MPO_String::MPO_String (string str) : string (str)
    {

    }

    /**
     * Standard Destructor. By Tobias, Date 02.11.2005.
     */
    MPO_String::~MPO_String ()
    {

    }

    /**
     * see MPO_Object::getId. By Tobias, Date 02.11.2005.
     */
    int MPO_String::getId() const
    {
        return (MPO_STRING);
    }

    /**
     * see MPO_Object::getFamilyId. By Tobias, Date 02.11.2005.
     */
    int MPO_String::getFamilyId() const
    {
        return(MPO_MESSAGEOBJECT_ID);
    }

    /**
     * see MPO_Object::getCallname. By Tobias, Date 02.11.2005.
     */
    string MPO_String::getCallname() const
    {
        return((string)*this);
    }

    /**
     * see MPO_Object::debugout. By Tobias, Date 02.11.2005.
     */
    std::ostream & MPO_String::debugout(std::ostream & output) const
    {
        output << (string)(*this);

        return output;
    }

    /**
     * see MPO_Object::save. By Tobias, Date 02.11.2005.
     */
    std::ostream & MPO_String::save(std::ostream & out) const
    {

        for (unsigned int i = 0; i < size();i++)
        {
            out < at(i);
        }

        out < '\0';

        return out;
    }

    /**
     * see MPO_Object::restore. By Tobias, Date 02.11.2005.
     */
    std::istream & MPO_String::restore(std::istream & in)
    {

        char c;
        //(string)(*this) = "";//operator "=" is only defined for base class string, so there's no ambiguity here
     	(*this) = "";
        in > c;
        while (c != '\0')
        {
            (*this) += c;
            in > c;
        }

        return in;
    }

    /**
     * Copys the content from the stringstream to this string. By Tobias, Date 02.11.2005.
     * @param stream the stringstream whose content will be copied, only 256 characters will be copied.
     */
    void MPO_String::assign (std::stringstream *stream)
    {
        char buf[256];
        (*this) = "";
        while (stream->getline (buf,256))
        {
            (*this) += buf;
            (*this) += "\n";

        }
    }

}//end of namespace MPO







