/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 * @file
 * This source file implements a class used for MARMOT/TCheck combinations.
 * @author Tobias Hilbrich
 * @date 05.09.2007
 * $Id$
 */

#include "mpo_TCheck_Races.h"
#include <stdio.h>

//Set static variables
//========================================================================
MPO::MPO_TCheck_Races *MPO::MPO_TCheck_Races::our_instance = NULL;

namespace MPO 
{
    // MPO_TCheck_Races
    //========================================================================
    MPO_TCheck_Races::MPO_TCheck_Races ()
    {
        //set the variables to something defined here
        App_Needs_MPI_THREAD_MULTIPLE = 0;
        App_Performed_MPI_Finalize_to_early = 0;
    }
    
    // ~MPO_TCheck_Races
    //========================================================================
    MPO_TCheck_Races::~MPO_TCheck_Races ()
    {
        //nothing to be done here
    }
    
    // getInstance
    //========================================================================
    MPO_TCheck_Races* MPO_TCheck_Races::getInstance (void)
    {
        // create the global instance if necessary 
        if (our_instance == NULL)
        {
            our_instance = new MPO_TCheck_Races ();
        }
        
        //return the global instance
        return our_instance;
    }
        
    // read
    //========================================================================
    void MPO_TCheck_Races::read (MPO_TCHECK_RACES_VARS var)
    {
        int x;
        
        switch (var)
        {
        case APP_NEEDS_MPI_THREAD_MULTIPLE:
            x = App_Needs_MPI_THREAD_MULTIPLE;
            break;
        case APP_MPI_FINALIZE_TO_EARLY:
            x = App_Performed_MPI_Finalize_to_early;
            break;
        }
    }
    
    // write
    //========================================================================
    void MPO_TCheck_Races::write (MPO_TCHECK_RACES_VARS var)
    {
        switch (var)
        {
        case APP_NEEDS_MPI_THREAD_MULTIPLE:
            App_Needs_MPI_THREAD_MULTIPLE = 1;
            break;
        case APP_MPI_FINALIZE_TO_EARLY:
            App_Performed_MPI_Finalize_to_early = 1;
            break;
        }
    }
    
    // writeIntForRequests
    //========================================================================
    void MPO_TCheck_Races::writeIntForRequests (int id)
    {
        if (id >= TCHECK_NUM_REQUEST_SLOTS)
            return;
        
        //do a write
        App_Wrong_MultiThread_Request_Usage[id] = 1;
    }
    
    // writeIntForRequests
    //========================================================================
    void MPO_TCheck_Races::writeIntForComm (int id)
    {
        if (id >= TCHECK_NUM_COMM_SLOTS)
            return;
            
        //do a write
        App_Comm_In_Two_Collective_Calls[id] = 1;
    }
        
    // writeIntForProbe
    //========================================================================
    void MPO_TCheck_Races::writeIntForProbe (int id)
    {
        if (id >= TCHECK_NUM_PROBE_SLOTS)
            return;
                
        //do a write
        App_Warn_Probe_Invalidation[id] = 1;
    }
    
    // readIntForProbe
    //========================================================================
    void MPO_TCheck_Races::readIntForProbe (int id)
    {
        int x;
        
        if (id >= TCHECK_NUM_PROBE_SLOTS)
            return;
                    
        //do a read
        x = App_Warn_Probe_Invalidation[id];
    }
}//end of namespace MPO








