/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Test_cancelled.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Test_cancelled.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Test_cancelled::MPO_Test_cancelled()
        : my_status(0),
          my_flag(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Test_cancelled::MPO_Test_cancelled(MPI_Status * status,
                                           int *flag)
        : my_status(status),
          my_flag(flag)
    {
    }


    // destructor
    //========================================================================
    MPO_Test_cancelled::~MPO_Test_cancelled()
    {
    }


    // getId
    //========================================================================
    int MPO_Test_cancelled::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_TEST_CANCELLED_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Test_cancelled::checkPost(void)
    {

        return 0;
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Test_cancelled::checkPrerequisites(void)
    {



        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Test_cancelled::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Test_cancelled(my_status, my_flag);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Test_cancelled::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Test_cancelled";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_TEST_CANCELLED";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Test_cancelled::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output
            << getCallname()
            << "(*status, "
            << "*flag";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Test_cancelled::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        // out < my_status;
        // out < my_flag;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Test_cancelled::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        // in > my_status;
        // in > my_flag;

        return in;
    }


}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied from mpich-1.2.4/src/pt2pt/testcancel.c.
/*@
  MPI_Test_cancelled - Tests to see if a request was cancelled

  Input Parameter:
  . status - status object (Status)

  Output Parameter:
  . flag - (logical)

  .N fortran

  int MPI_Test_cancelled(
  MPI_Status *status,
  int        *flag)

  @ */

