/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
  * \file mpo_ThreadViolationInfo.cc
  * This Sourcefile implements the MPO_ThreadViolationInfo class.
  * @author Tobias Hilbrich
  * @date 19.07.2007
  * $Id$
  */

#include "mpo_ThreadViolationInfo.h"
#include "mpo_SourceLocation.h"

namespace MPO 
{

    // MPO_ThreadViolationInfo 1
    //========================================================================
    MPO_ThreadViolationInfo::MPO_ThreadViolationInfo (void)
    {
        //nothing to be done here
    }
    
    // MPO_ThreadViolationInfo 2
    //========================================================================
    MPO_ThreadViolationInfo::MPO_ThreadViolationInfo (int threadId,MPO_SourceLocation loc)
    {
        addParticipant(threadId,loc);
    }
    
    // addParticipant
    //========================================================================
    void MPO_ThreadViolationInfo::addParticipant(int threadId,MPO_SourceLocation loc)
    {
        my_participants.push_back(std::make_pair(threadId,loc));
    }
    
    // getParticipants
    //========================================================================
    std::list<std::pair<int,MPO_SourceLocation> > MPO_ThreadViolationInfo::getParticipants (void)
    {
        return my_participants; 
    }
    
    // getNumParticipants
    //========================================================================    
    int MPO_ThreadViolationInfo::getNumParticipants (void)
    {
        return my_participants.size();
    }
    
    // printParticipants
    //========================================================================
    void MPO_ThreadViolationInfo::printParticipants (std::ostream &out)
    {
        //loop over all the locations
        for (MPO_ThreadLocationListType::iterator i = my_participants.begin();i != my_participants.end();i++)
        {
            //print a newline between each participant
            if (i != my_participants.begin())
                out << std::endl;
            
            //give thread id of participant
            out << "Participant: ThreadID = "
                << i->first;
            
            //give location of participants when available
            if (i->second.isValidLocation())
            {
                out << ", Location: ";
                i->second.debugout(out);
            }            
        }
    }    
   
}//end of namespace MPO







