/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Topo_test.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Topo_test.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Topo_test::MPO_Topo_test()
        : my_comm(MPI_COMM_NULL), 
          my_status(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Topo_test::MPO_Topo_test(MPO_Comm comm,
                                 int *status)
        : my_comm(comm),
          my_status(status)
    {
    }


    // destructor
    //========================================================================
    MPO_Topo_test::~MPO_Topo_test()
    {
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Topo_test::checkPrerequisites()
    {
        
        our_comm_traces->findAndAddUsage(my_comm,getSourceLocation());
        
        

        // currently no checks implemented    

        return 0;
    }


    // getId
    //========================================================================
    int MPO_Topo_test::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_TOPO_TEST_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Topo_test::checkPost(void)
    {
    
        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Topo_test::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Topo_test(my_comm, my_status);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Topo_test::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Topo_test";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_TOPO_TEST";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Topo_test::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output
            << getCallname()
            << "(comm = " 
            << my_comm 
            << ", *status";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")" 
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Topo_test::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        out < my_comm;
        // out < my_status;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Topo_test::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        in > my_comm;
        // in > my_status;

        return in;
    }


}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied from mpich-1.2.4/src/topol/topo_test.c.
/*@
  MPI_Topo_test - Determines the type of topology (if any) associated with 
  a communicator

  Input Parameter:
  . comm - communicator (handle) 

  Output Parameter:
  . top_type - topology type of communicator 'comm' (choice).

  .N fortran

  .N Errors
  .N MPI_SUCCESS
  .N MPI_ERR_COMM
  .N MPI_ERR_ARG

  .seealso: MPI_Graph_create, MPI_Cart_create

  int MPI_Topo_test ( 
  MPI_Comm comm, 
  int *top_type )
  @ */


