/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Type_extent.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Type_extent.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Type_extent::MPO_Type_extent()
        : MPO_Typehandlers(MPI_DATATYPE_NULL),
          my_extent(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Type_extent::MPO_Type_extent(MPO_Datatype datatype,
                                     MPI_Aint *
                                     extent)
        : MPO_Typehandlers(datatype),
          my_extent(extent)
    {
    }


    // destructor
    //========================================================================
    MPO_Type_extent::~MPO_Type_extent()
    {
    }


    // getId
    //========================================================================
    int MPO_Type_extent::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_TYPE_EXTENT_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Type_extent::checkPost(void)
    {
    
        return 0;
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Type_extent::checkPrerequisites(void)
    {

        
        our_datatype_traces->findAndAddUsage(my_datatype,getSourceLocation());

        
        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Type_extent::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Type_extent(my_datatype, my_extent);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Type_extent::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Type_extent";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_TYPE_EXTENT";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Type_extent::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output 
            << getCallname()
            << "(datatype = " 
            << my_datatype 
            << ", *extent";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"  
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Type_extent::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        out < my_datatype;
        // out < my_extent;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Type_extent::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        in > my_datatype;
        // in > my_extent;

        return in;
    }

}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied 
// from mpich-1.2.4/src/pt2pt/type_extent.c.
/*@
  MPI_Type_extent - Returns the extent of a datatype

  Input Parameters:
  . datatype - datatype (handle) 

  Output Parameter:
  . extent - datatype extent (integer) 

  .N fortran

  .N Errors
  .N MPI_SUCCESS
  .N MPI_ERR_TYPE

  int MPI_Type_extent( 
  MPI_Datatype datatype, 
  MPI_Aint *extent )
  @ */

