/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Type_size.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Type_size.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Type_size::MPO_Type_size()
        : MPO_Typehandlers(MPI_DATATYPE_NULL),
          my_size(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Type_size::MPO_Type_size(MPO_Datatype datatype,
                                 int *size)
        : MPO_Typehandlers(datatype),
          my_size(size)
    {
    }


    // destructor
    //========================================================================
    MPO_Type_size::~MPO_Type_size()
    {
    }


    // getId
    //========================================================================
    int MPO_Type_size::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_TYPE_SIZE_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Type_size::checkPost(void)
    {
            
        return 0;
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Type_size::checkPrerequisites(void)
    {

        
        our_datatype_traces->findAndAddUsage(my_datatype,getSourceLocation());

        
        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Type_size::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Type_size(my_datatype, my_size);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Type_size::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Type_size";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_TYPE_SIZE";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Type_size::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output
            << getCallname()
            << "(datatype = " 
            << my_datatype 
            << ", *size";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")" 
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Type_size::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        out < my_datatype;
        // out < my_size;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Type_size::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        in > my_datatype;
        // in > my_size;

        return in;
    }

}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied from mpich-1.2.4/src/pt2pt/type_size.c.
/*@
  MPI_Type_size - Return the number of bytes occupied by entries
  in the datatype

  Input Parameters:
  . datatype - datatype (handle) 

  Output Parameter:
  . size - datatype size (integer) 

  .N fortran

  .N Errors
  .N MPI_SUCCESS
  .N MPI_ERR_TYPE
  .N MPI_ERR_ARG

  int MPI_Type_size ( 
  MPI_Datatype datatype, 
  int *size )
  @ */

