/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Win representing MPI-windows.
 *
 *  @author Bettina Krammer
 *
 *  $Id: mpo_Win.cc 953 2009-04-30 13:44:56Z hpczink $
 */

#include "mpo.h"
#include "mpo-objects.h"
#include "mpo-binaryIO.h"


#ifdef MARMOT_1SIDED

namespace MPO
{
    // default constructor
    //========================================================================
    MPO_Win::MPO_Win()
        : my_win(MPI_WIN_NULL)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Win::MPO_Win(MPI_Win win)
    {
        my_win = win;
    }


    // destructor
    //========================================================================
    MPO_Win::~MPO_Win()
    {
    }


    // MPI_Win
    //========================================================================
    MPO_Win::operator  MPI_Win() const
    {
        return my_win;
    }


    // getId
    //========================================================================
    int MPO_Win::getId() const
    {
        // returns the unique type identity set by enum MPO_TYPE_ID
        return MPO_WIN_ID;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Win::debugout(std::ostream & output) const
    {
        if (my_win == MPI_WIN_NULL)
        {
            output << "MPI_WIN_NULL";
        }
        else
        {
            output << "self-defined win";
        }

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Win::save(std::ostream & out) const
    {
        // we don't save and restore pointers
#if defined(USE_LAM_MPI) || defined(OPEN_MPI)
        // out < my_win;
#else
        out < my_win;
#endif

      return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Win::restore(std::istream & in)
    {
        // we don't save and restore pointers
#if defined(USE_LAM_MPI) || defined(OPEN_MPI)
        // in > my_win;
#else
        in > my_win;
#endif

        return in;
    }

}                               // end of namespace MPO

#endif
// end of MARMOT_1SIDED
