#include "StdAfx.h"
#include "DICE_DataLeaf.h"
#include <fstream>

namespace DICE_BACK
{

	CDICE_DataLeaf::CDICE_DataLeaf(void)
	{
		m_severity = 0.0;
		m_link.clear();
	}

	CDICE_DataLeaf::CDICE_DataLeaf(std::string desc, long DICE_id,double severity)
		: CDICE_DataItem (desc,DICE_id)
	{
		m_severity = severity;
		m_link.clear();
	}

	CDICE_DataLeaf::~CDICE_DataLeaf(void)
	{
	}

	double CDICE_DataLeaf::getSeverity (void)
	{
		return m_severity;
	}

	void CDICE_DataLeaf::setSeverity (double severity)
	{
		m_severity = severity;
	}

	void CDICE_DataLeaf::addSeverity (double severity)
	{
		m_severity += severity;
	}

	std::list<SDICE_LinkType> CDICE_DataLeaf::getLinks (void)
	{
		return m_link;
	}

	void CDICE_DataLeaf::addLink (long locationsID, long systemID, double severity)
	{
		//search for a similar link
		std::list<SDICE_LinkType>::iterator i;

		for (i = m_link.begin(); i != m_link.end(); i++)
		{
			if (i->locationsID != locationsID)
				continue;

			if (i->systemID != systemID)
				continue;

			SDICE_LinkType temp = *i;
			temp.severity += severity;

			m_link.erase(i);
			m_link.push_back(temp);

			return;
		}

		//no such link present yet, create one
		SDICE_LinkType temp;
		temp.locationsID = locationsID;
		temp.systemID = systemID;
		temp.severity = severity;
		m_link.push_back(temp);
	}

	double CDICE_DataLeaf::calculateSeverity (void)
	{
		return m_severity;
	}

	std::list<SDICE_LinkType> CDICE_DataLeaf::accumulateLinks (void)
	{
		return m_link;
	}

	bool CDICE_DataLeaf::isLeaf (void)
	{
		return true;
	}

	DICE_INTERFACE::CDICE_TreeItem* CDICE_DataLeaf::copyToInterfaceTree (void)
	{
		DICE_INTERFACE::CDICE_TreeItem* ret = NULL;
		ret = new DICE_INTERFACE::CDICE_TreeLeaf (NULL,m_desc,m_severity,m_DICE_id);
		return ret;
	}

	DICE_INTERFACE::CDICE_TreeItem* CDICE_DataLeaf::copyToInterfaceTreeWeighted (std::list<std::pair<long,double> > weights)
	{
		DICE_INTERFACE::CDICE_TreeItem* ret = NULL;

		double severity = 0;
		std::list<std::pair<long,double> >::iterator i;
		for (i = weights.begin();i != weights.end();i++)
		{
			if (m_DICE_id == i->first)
			{
				severity = i->second;
				break;
			}
		}	

		ret = new DICE_INTERFACE::CDICE_TreeLeaf (NULL,m_desc,severity,m_DICE_id);
		return ret;
	}

	bool CDICE_DataLeaf::deleteTree (void)
	{
		/*Destructor does everything, can't fail ...*/
		return true;
	}

	bool CDICE_DataLeaf::save (std::ofstream &file)
	{
		//save type
		file << "0" << std::endl;

		//save data
		file << m_severity << std::endl;

		file << m_link.size () << std::endl;

		std::list<SDICE_LinkType>::iterator i;
		for (i = m_link.begin();i != m_link.end();i++)
		{
			file << i->locationsID << std::endl;
			file << i->severity << std::endl;
			file << i->systemID << std::endl;
		}

		CDICE_DataItem::save(file);

		return true;
	}

	bool CDICE_DataLeaf::load (std::ifstream &file)
	{
		//type already loaded by creator of this object !

		//data
		file >> m_severity;

		int num_links, i;
		file >> num_links;

		m_link.clear();
		for (i = 0; i < num_links; i++)
		{
			SDICE_LinkType link;
			file >> link.locationsID;
			file >> link.severity;
			file >> link.systemID;

			m_link.push_back (link);
		}

		CDICE_DataItem::load(file);

		return true;
	}
}//end of namespace DICE_BACK