#include "StdAfx.h"
#include "DICE_Message.h"

namespace DICE_INTERFACE
{
	CDICE_Message::CDICE_Message(const CDICE_Message &msg)
	{
		m_timestamp = msg.getTimestamp();
		m_sourceRank = msg.getSourceRank();
		m_sourceThread = msg.getSourceThread();
		m_callName = msg.getCallName();
		m_shortDesc = msg.getShortDesc();
		m_text = msg.getText();
		m_locationFileName = msg.getLocationFileName();
		m_locationLine = msg.getLocationLine();
		m_type = msg.getType();
		m_mpiReference = msg.getMpiReference();
	}

	CDICE_Message::CDICE_Message(void)
	{
		m_timestamp = 0;
		m_sourceRank = 0;
		m_sourceThread = 0;
		m_text = "";
		m_locationFileName = "";
		m_locationLine = 0;
		m_type = MESSAGE_TYPE_NOTE;
		m_callName = "";
		m_shortDesc = "";
		m_mpiReference = "";
	}

	CDICE_Message::CDICE_Message(
		long timestamp,
		int sourceRank,
		int sourceThread,
		std::string callName,
		std::string shortDesc,
		std::string text,
		std::string locationFileName,
		long locationLine,
		EMESSAGE_TYPE type,
		std::string mpiReference)
	{
		m_timestamp = timestamp;
		m_sourceRank = sourceRank;
		m_sourceThread = sourceThread;
		m_text = text;
		m_locationFileName = locationFileName;
		m_locationLine = locationLine;
		m_type = type;
		m_callName = callName;
		m_shortDesc = shortDesc;
		m_mpiReference = mpiReference;
	}

	CDICE_Message::~CDICE_Message(void)
	{
	}

	int CDICE_Message::getSourceRank (void) const
	{
		return m_sourceRank;
	}

	void CDICE_Message::setSourceRank (int sourceRank)
	{
		m_sourceRank = sourceRank;
	}

	int CDICE_Message::getSourceThread (void) const
	{
		return m_sourceThread;
	}

	void CDICE_Message::setSourceThread (int sourceThread)
	{
		m_sourceThread = sourceThread;
	}

	std::string CDICE_Message::getText (void) const
	{
		return m_text;
	}

	void CDICE_Message::setText (std::string text)
	{
		m_text = text;
	}

	std::string CDICE_Message::getLocationFileName (void) const
	{
		return m_locationFileName;
	}

	void CDICE_Message::setLocationFileName (std::string locationFileName)
	{
		m_locationFileName = locationFileName;
	}

	long CDICE_Message::getLocationLine (void) const
	{
		return m_locationLine;
	}

	void CDICE_Message::setLocationLine (long locationLine)
	{
		m_locationLine = locationLine;
	}

	EMESSAGE_TYPE CDICE_Message::getType (void) const
	{
		return m_type;
	}

	void CDICE_Message::setType (EMESSAGE_TYPE type)
	{
		m_type = type;
	}

	std::string CDICE_Message::getShortDesc (void) const
	{
		return m_shortDesc;
	}

	void CDICE_Message::setShortDesc (std::string shortDesc)
	{
		m_shortDesc = shortDesc;
	}
		 
	std::string CDICE_Message::getCallName (void) const
	{
		return m_callName;
	}
		
	void CDICE_Message::setCallName (std::string callName)
	{
		m_callName = callName;
	}

	long CDICE_Message::getTimestamp (void) const
	{
		return m_timestamp;
	}

	void CDICE_Message::setTimestamp (long timestamp)
	{
		m_timestamp = timestamp;
	}

	std::string CDICE_Message::getMpiReference (void) const
	{
		return m_mpiReference;
	}

	void CDICE_Message::setMpiReference (std::string mpiReference)
	{
		m_mpiReference = mpiReference;
	}

	bool CDICE_Message::operator== (const CDICE_Message& x) const
	{
		if (m_timestamp != x.getTimestamp())
			return false;

		if (m_sourceRank != x.getSourceRank())
			return false;
		
		if (m_sourceThread != x.getSourceThread())
			return false;

		if (m_text.compare(x.getText()) != 0)
			return false;

		if (m_locationFileName.compare(x.getLocationFileName()) != 0)
			return false;

		if (m_locationLine != x.getLocationLine())
			return false;

		if (m_type != x.getType())
			return false;

		if (m_callName.compare(x.getCallName()) != 0)
			return false;

		if (m_shortDesc.compare(x.getShortDesc()) != 0)
			return false;

		return true;
	}

} //end of namespace DICE_INTERFACE