#include "StdAfx.h"
#include "DICE_Report.h"
#include <fstream>

namespace DICE_BACK
{

	CDICE_Report::CDICE_Report(void)
	{
		m_IDmessages = -1;
		m_IDlocations = -1;
		m_IDsystem = -1;
	}

	CDICE_Report::~CDICE_Report(void)
	{
	}

	CDICE_Report::CDICE_Report(
		long timestamp,
		int sourceRank,
		int sourceThread,
		std::string callName,
		std::string shortDesc,
		std::string text,
		std::string locationFileName,
		long locationLine,
		DICE_INTERFACE::EMESSAGE_TYPE type,
		std::string mpiReference,
		long IDmessages,
		long IDlocations,
		long IDsystem)
		: DICE_INTERFACE::CDICE_Message(timestamp,sourceRank,sourceThread,callName,shortDesc,text,locationFileName,locationLine,type,mpiReference)
	{
		m_IDmessages = IDmessages;
		m_IDlocations = IDlocations;
		m_IDsystem = IDsystem;
	}

	CDICE_Report::CDICE_Report(
		DICE_INTERFACE::CDICE_Message msg,
		long IDmessages,
		long IDlocations,
		long IDsystem)
		: DICE_INTERFACE::CDICE_Message(msg.getTimestamp(),msg.getSourceRank(),msg.getSourceThread(),msg.getCallName(),msg.getShortDesc(),msg.getText(),msg.getLocationFileName(),msg.getLocationLine(),msg.getType(),msg.getMpiReference())
	{
		m_IDmessages = IDmessages;
		m_IDlocations = IDlocations;
		m_IDsystem = IDsystem;
	}

	long CDICE_Report::getIDmessages (void)
	{
		return m_IDmessages;
	}

	void CDICE_Report::setIDmessages (long id)
	{
		m_IDmessages = id;
	}

	long CDICE_Report::getIDlocations (void)
	{
		return m_IDlocations;
	}

	void CDICE_Report::setIDlocations (long id)
	{
		m_IDlocations = id;
	}

	long CDICE_Report::getIDsystem (void)
	{
		return m_IDsystem;
	}

	void CDICE_Report::setIDsystem (long id)
	{
		m_IDsystem = id;
	}

	DICE_INTERFACE::CDICE_Message CDICE_Report::getMessage (void)
	{
		DICE_INTERFACE::CDICE_Message ret (m_timestamp,m_sourceRank,m_sourceThread,m_callName,m_shortDesc,m_text,m_locationFileName,m_locationLine,m_type,m_mpiReference);
		return ret;
	}

	bool CDICE_Report::save (std::ofstream &file)
	{
		//CDICE_Report
		file << m_IDmessages << std::endl;
		file << m_IDlocations << std::endl;
		file << m_IDsystem << std::endl;

		//DICE_INTERFACE::CDICE_Message
		file << m_timestamp << std::endl;
		file << m_sourceRank << std::endl;
		file << m_sourceThread << std::endl;
		save_string(file, m_text);            //std::string !
		save_string(file, m_shortDesc);       //std::string !
		save_string(file, m_callName);        //std::string !
		save_string(file, m_locationFileName);//std::string !
		file << m_locationLine << std::endl;
		file << m_type << std::endl;	        //enum
		save_string(file, m_mpiReference);    //std::string !

		return true;
	}

	bool CDICE_Report::load (std::ifstream &file)
	{
		//CDICE_Report
		file >> m_IDmessages;
		file >> m_IDlocations;
		file >> m_IDsystem;

		//DICE_INTERFACE::CDICE_Message
		unsigned long type;
		file >> m_timestamp;
		file >> m_sourceRank;
		file >> m_sourceThread;
		
		std::string temp;
		std::getline (file, temp);
		m_text				= load_string(file);            //std::string !
		m_shortDesc			= load_string(file);       //std::string !
		m_callName			= load_string(file);        //std::string !
		m_locationFileName	= load_string(file);//std::string !
		file >> m_locationLine;
		file >> type;  	       //enum
		m_type = (DICE_INTERFACE::EMESSAGE_TYPE)type;

		std::getline (file, temp);
		m_mpiReference		= load_string(file);    //std::string !

		return true;
	}

	void CDICE_Report::save_string (std::ofstream &file, std::string string)
	{
		size_t pos;

		do 
		{
			pos = string.find ("\r\n");
			if (pos != std::string::npos)
			{
				string.replace (pos,2,"<RN-DICE-BREAK-LINE>");
			}
			else
			{
				pos = string.find ("\n");
				if (pos != std::string::npos)
					string.replace (pos,1,"<N-DICE-BREAK-LINE>");
			}
		} while (pos != std::string::npos);
		
		file << string << std::endl;
	}

	std::string CDICE_Report::load_string (std::ifstream &file)
	{
		std::string ret;
		size_t pos;

		std::getline (file, ret);

		do 
		{
			pos = ret.find ("<RN-DICE-BREAK-LINE>");
			if (pos != std::string::npos)
			{
				ret.replace (pos,strlen("<RN-DICE-BREAK-LINE>"),"\r\n");
			}
			else
			{
				pos = ret.find ("<N-DICE-BREAK-LINE>");
				if (pos != std::string::npos)
					ret.replace (pos,strlen("<N-DICE-BREAK-LINE>"),"\n");
			}
		} while (pos != std::string::npos);

		return ret;
	}
}//end of namespace DICE_BACK