#include "StdAfx.h"
#include "DICE_TreeDataBase.h"
#include "DICE_DataLeaf.h"
#include "DICE_DataNode.h"
#include <fstream>

namespace DICE_BACK
{

	CDICE_TreeDataBase::CDICE_TreeDataBase(void)
	{
		m_tree = NULL;
		m_nextID = 1;
	}

	CDICE_TreeDataBase::~CDICE_TreeDataBase(void)
	{
		if (m_tree)
			delete (m_tree);
		m_tree = NULL;
	}

	long CDICE_TreeDataBase::convertDescToID (std::string desc)
	{
		if (m_tree)
		{
			return m_tree->findDesc (desc); //might return -1 -> not present in tree
		}

		return -1; //not present in tree
	}

	long CDICE_TreeDataBase::convertDescToID (long parentID,std::string desc)
	{
		if (m_tree)
		{
			CDICE_DataItem* parent = m_tree->findDICEid(parentID);
			if (parent)
			{
				return parent->findDesc (desc); //might return -1 -> not present in tree
			}
		}

		return -1; //not present in tree
	}

	long CDICE_TreeDataBase::addNewLeaf (std::string desc, long parentID, bool *out_pWasPresent)
	{
		//was not present
		if (out_pWasPresent)
			*out_pWasPresent = false;

		if (parentID <= 0)
		{
			//new root
			if (m_tree)
			{
				delete m_tree;
				//TODO: WARNING SHOULD BE ISSUED HERE !
				//(New root deleted tree)
			}

			m_tree = new CDICE_DataLeaf (desc,m_nextID,0.0);
			m_nextID++;
			return m_nextID-1;
		}

		if (m_tree)
		{
			CDICE_DataItem* parent = m_tree->findDICEid(parentID);

			if (parent == NULL)
			{
				//TODO: ERROR SHOULD BE ISSUED HERE !
				//(parent not present)
				return -1;
			}
			
			if (parent->isLeaf())
			{
				//TODO: ERROR SHOULD BE ISSUED HERE !
				//(parent is a leaf not a node)
				return -1;
			}

			//cast parent to a node
			CDICE_DataNode* parent_node = (CDICE_DataNode*) parent;

			//is the child already present ?
			std::list<CDICE_DataItem*> childs = parent_node->getChilds();
			std::list<CDICE_DataItem*>::iterator i;
			for (i = childs.begin();i != childs.end();i++)
			{
				if ((*i)->getDesc().compare(desc) == 0)
				{
					if (out_pWasPresent)
						*out_pWasPresent = true;
					return (*i)->getDICEid();
				}
			}

			//not yet present so create it !
			CDICE_DataItem* child = new CDICE_DataLeaf (desc,m_nextID,0.0);
			parent_node->addChild(child);
			m_nextID++;
			return m_nextID-1;
		}

		return -1; //TODO: ERROR SHOULD BE ISSUED HERE !
	               //(parent speciefied but no tree present)
	}

	long CDICE_TreeDataBase::addNewNode (std::string desc, long parentID, bool *out_pWasPresent)
	{
		//standard decission it was not present
		if (out_pWasPresent)
			*out_pWasPresent = false;

		if (parentID <= 0)
		{
			//new root
			if (m_tree)
			{
				delete m_tree;
				//TODO: WARNING SHOULD BE ISSUED HERE !
				//(New root deleted tree)
			}

			m_tree = new CDICE_DataNode (desc,m_nextID);
			m_nextID++;
			return m_nextID-1;
		}

		if (m_tree)
		{
			CDICE_DataItem* parent = m_tree->findDICEid(parentID);

			if (parent == NULL)
			{
				//TODO: ERROR SHOULD BE ISSUED HERE !
				//(parent not present)
				return -1;
			}
			
			if (parent->isLeaf())
			{
				//TODO: ERROR SHOULD BE ISSUED HERE !
				//(parent is a leaf not a node)
				return -1;
			}

			//cast parent to a node
			CDICE_DataNode* parent_node = (CDICE_DataNode*) parent;

			//is the child already present ?
			std::list<CDICE_DataItem*> childs = parent_node->getChilds();
			std::list<CDICE_DataItem*>::iterator i;
			for (i = childs.begin();i != childs.end();i++)
			{
				if ((*i)->getDesc().compare(desc) == 0)
				{
					if (out_pWasPresent)
						*out_pWasPresent = true; //was already present
					return (*i)->getDICEid();
				}
			}

			//not yet present so add it !
			CDICE_DataItem* child = new CDICE_DataNode (desc,m_nextID);
			parent_node->addChild(child);
			m_nextID++;
			return m_nextID-1;
		}

		return -1; //TODO: ERROR SHOULD BE ISSUED HERE !
	               //(parent speciefied but no tree present)
	}

	bool CDICE_TreeDataBase::addLink (long itemID, long locationsID, long systemID, double severity)
	{
		if (m_tree)
		{
			CDICE_DataItem* item = m_tree->findDICEid(itemID);

			if (item == NULL)
			{
				//TODO: ERROR SHOULD BE ISSUED HERE !
				//(item not present)
				return false;
			}
			
			if (!item->isLeaf())
			{
				//TODO: ERROR SHOULD BE ISSUED HERE !
				//(parent is not a leaf)
				return false;
			}

			//cast item to a leaf
			CDICE_DataLeaf* item_leaf = (CDICE_DataLeaf*) item;
			item_leaf->addLink(locationsID,systemID,severity);
			item_leaf->addSeverity(severity);

			return true;
		}

		return false;
	}

	std::list<SDICE_LinkType> CDICE_TreeDataBase::collectLinksForSelection(long selection)
	{
		//return nothing for an empty tree
		if (!m_tree)
		{
			return std::list<SDICE_LinkType>();
		}	

		//find the node for the selection
		CDICE_DataItem* item = m_tree->findDICEid(selection);

		if (item == NULL)
		{
			//if invalid selection we assume root !
			item = m_tree;
		}

		if (item == NULL) //still NULL ?
			return std::list<SDICE_LinkType>();

		return item->accumulateLinks();
	}

	std::list<long> CDICE_TreeDataBase::collectIDsBySelection(long selection)
	{
		if (m_tree == NULL)
		{
			return std::list<long>();
		}

		//find the node for the selection
		CDICE_DataItem* item = m_tree->findDICEid(selection);

		if (item == NULL)
		{
			//if invalid selection we assume root !
			item = m_tree;
		}

		if (item == NULL) //still NULL ?
			return std::list<long>();

		return item->accumulateIDs();
	}
	
	DICE_INTERFACE::CDICE_TreeItem* CDICE_TreeDataBase::createFullInterfaceTree (void)
	{
		if (!m_tree)
			return NULL;

		return m_tree->copyToInterfaceTree();
	}

	DICE_INTERFACE::CDICE_TreeItem* CDICE_TreeDataBase::createWeightedInterfaceTree (std::list<std::pair<long,double> > weights)
	{
		if (!m_tree)
			return NULL;

		return m_tree->copyToInterfaceTreeWeighted(weights);
	}

	bool CDICE_TreeDataBase::deleteTree ()
	{
		bool ret = true;

		m_nextID = 1;
		if (m_tree)
		{
			ret = m_tree->deleteTree ();
			delete (m_tree);
		}
		m_tree = NULL;

		return ret;
	}

	bool CDICE_TreeDataBase::save (std::ofstream &file)
	{
		file << m_nextID << std::endl;

		if (!m_tree)
		{
			file << "0" << std::endl;		
			return true;
		}
		
		file << "1" <<std::endl;
    	return m_tree->save (file);		
	}

	bool CDICE_TreeDataBase::load (std::ifstream &file)
	{
		file >> m_nextID;
		
		int isTreePresent;
		file >> isTreePresent;

		if (isTreePresent)
		{
			int type;
			file >> type;
			if (type == 0) //leaf
				m_tree = new CDICE_DataLeaf ();
			else
				m_tree = new CDICE_DataNode ();

			return m_tree->load(file);
		}

		return true;
	}

	bool CDICE_TreeDataBase::isEmpty (void)
	{
		if (m_tree)
			return false;
		return true;
	}

}//end of namespace DICE_BACK