/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**********************************************************************
 * @file   ThreadLib.h
 * @brief  Declaration of the classes CThread, CMutex ,CLock 
 * 
 * @author Blasius Czink
 * @date   20.12.2003
 **********************************************************************
 * Version History:
 * V 0.1  20.12.2003  : First Revision
 **********************************************************************/

#ifndef ThreadLib_H
#define ThreadLib_H

#ifdef _WIN32
#include <windows.h>
#define STDCALLPREFIX __stdcall
typedef unsigned threadfunc_t;
typedef void * threadparam_t;
#else
#include <pthread.h>
#define STDCALLPREFIX
typedef void * threadfunc_t;
typedef void * threadparam_t;
#endif

///////////////////////////
// CThread class
//
class CThread
{
public:
	CThread(bool release = true);
	virtual ~CThread();

	static threadfunc_t STDCALLPREFIX startThread(threadparam_t);

	virtual void run() = 0;

	bool isRunning(void);
	void setRunning(bool x);
	bool isReleased(void);
	void setRelease(bool x);
	bool isDeletedOnExit(void);
	void setDeletedOnExit(bool x = true);
	bool isDestructor(void);

private:
	CThread(const CThread& ) {}
	CThread& operator=(const CThread& ) { return *this; }
#ifdef _WIN32
	HANDLE    m_thread;
	unsigned m_dwThreadId;
#else
	pthread_t m_thread;
#endif
	bool m_running;
	bool m_release;
	bool m_delete_on_exit;
	bool m_destructor;
};

///////////////////////////
// CMutex class
//
class CMutex
{
	friend class CLock;
public:
	CMutex();
	~CMutex();

	void lock();
	void unlock();
private:
#ifdef _WIN32
	HANDLE m_mutex;
#else
	pthread_mutex_t m_mutex;
#endif
};

///////////////////////////
// CLock class
//
class CLock
{
public:
	CLock(CMutex&);
	~CLock();

private:
	CMutex & m_mutex;
};


#endif
