#ifdef USE_MARMOT_DICE

#include "ToolWinComm.h"
#include "stddef.h"
#include <list>
#include <atlstr.h>
#include <assert.h>

CToolWinComm* CToolWinComm::m_theInstance = NULL;

CToolWinComm::~CToolWinComm(void)
{
}

CToolWinComm::CToolWinComm(CComPtr<DTE2> pDTE, CComPtr<AddIn> pAddInInstance,CComPtr<OutputWindowPane> pBuildOutput)
{
	setToolWinObject(this);

	m_pDTE = pDTE;
	m_pAddInInstance = pAddInInstance;
	m_pBuildOutput = m_pBuildOutput;
}

CToolWinComm* CToolWinComm::InitInstance (CComPtr<DTE2> pDTE, CComPtr<AddIn> pAddInInstance,CComPtr<OutputWindowPane> pBuildOutput)
{
	if (m_theInstance)
		return m_theInstance;

	m_theInstance = new CToolWinComm (pDTE,pAddInInstance,pBuildOutput);
	return m_theInstance;
}

CToolWinComm* CToolWinComm::GetInstance (void)
{
	if (m_theInstance == NULL)
	{
		//TODO warn about this ...
		assert(0);
	}

	return m_theInstance;
}

void CToolWinComm::pushNewMarmotMsg (std::vector<std::string> messageparts)
{
	m_pToolWinObject->popNewMarmotMsg(messageparts);
}

void CToolWinComm::popOpenSourceFileLocation (std::string filename,int line)
{
	USES_CONVERSION;

	HRESULT hr;
	CComQIPtr<_Solution> pSol;
	CComQIPtr<Projects> pProjects;
	CComBSTR bstr ("");

	//get the solution and the projects
	IfFailGoCheck(m_pDTE->get_Solution(&pSol), pSol);
	IfFailGoCheck(pSol->get_Projects(&pProjects), pProjects);	
	
	// prepare file name
	std::string::size_type pos = filename.find("./",0);
	if (pos != std::string::npos)
		filename.replace(pos, 2, "");
	pos = filename.find(".\\",0);
	if (pos != std::string::npos)
		filename.replace(pos, 2, "");
	{
		std::wstring str(filename.length(), L' '); // Make room for characters
		std::copy(filename.begin(), filename.end(), str.begin());
		bstr.Append(str.c_str());
	}

	//make the filename lower case (it doesnt matter in windows)
	for(int i=0; i<filename.size();++i)
		filename[i] = tolower(filename[i]);

	//search in all the projects
	long lCount = 0;
	if (SUCCEEDED(pProjects->get_Count(&lCount)))
    {
		//loop over all the projects
        for (long i=1; i<=lCount; i++)
        {
			//get a reference for the project
			CComVariant projectIndex = i;
			CComPtr<Project> pCurrProj;
			pProjects->Item(projectIndex, &pCurrProj);
			
			//get the projects project items
			CComPtr<ProjectItems> pProjItems;
			pCurrProj->get_ProjectItems(&pProjItems);

			std::list<ProjectItems* > projectItemsStack;
			projectItemsStack.push_back(pProjItems);

			while (projectItemsStack.size() > 0)
			{
				//get project items from stack
				pProjItems = projectItemsStack.front();
				projectItemsStack.pop_front();

				//find the amount of project items
				long lProItemCount = 0;
				if (pProjItems) //obviously NULL pointers may be in the stack
				{
					if (SUCCEEDED(pProjItems->get_Count(&lProItemCount)))
					{
						//loop over all the project items
						for (long j=1; j<=lProItemCount; j++)
						{
							//get the project item for this index
							CComVariant projectItemIndex = j;
							CComPtr<ProjectItem> pProjectItem;
							pProjItems->Item(projectItemIndex, &pProjectItem);

							//get child project items and add them to the stack
							CComPtr<ProjectItems> pProjectItemsRecursive;
							pProjectItem->get_ProjectItems(&pProjectItemsRecursive);
							projectItemsStack.push_back(pProjectItemsRecursive);

							//get the amount of files
							short sFileCount;
							if (SUCCEEDED(pProjectItem->get_FileCount(&sFileCount)))
							{
								//loop over all the files
								for (short k=1; k<=sFileCount; k++)
								{
									//get the filename
									CComBSTR strFileName;
									pProjectItem->get_FileNames(k,&strFileName);

									//compare the file name to the given name
									std::string useableItemName(W2A(strFileName));
									for(int i=0; i<useableItemName.size();++i) //make the filename lower case (it doesnt matter in windows)
										useableItemName[i] = tolower(useableItemName[i]);
									std::string::size_type pos = useableItemName.find(filename,0);

									if (pos != std::string::npos)
									{
										//it contains the needed file name
										//open it if necessary
										CComBSTR kind ("");
										pProjectItem->get_Kind(&kind);

										VARIANT_BOOL isOpen;
										pProjectItem->get_IsOpen(kind,&isOpen);
										
										CComQIPtr<EnvDTE::Document> document;
										CComQIPtr<EnvDTE::Window> window;
										CComQIPtr<IDispatch> selection;
										CComQIPtr<EnvDTE::TextSelection> textSelection;
										
										if (isOpen == VARIANT_FALSE)
										{
											kind.Empty();
											kind.Append(vsViewKindCode);
											IfFailGoCheck(m_pDTE->OpenFile(kind,strFileName,&window),window);
											window->get_Document(&document);
											window->Activate();
										}
										else
										{
											//already open 
											//TODO: For some strange reason get_IsOpen always returns
											//VARIANT_FALSE ....
											/*CComQIPtr<EnvDTE::Windows> windows;

											pProjectItem->get_Document(&document);

											document->get_Windows(&windows);

											long windowCount;
											windows->get_Count(&windowCount);

											for (int windowIndex = 1; windowIndex <= windowCount;windowCount++)
											{
												CComVariant windowIndexVariant = windowIndex;
												windows->Item(windowIndexVariant,&window);

												CComBSTR str;
												window->get_Caption(&str);
												int xxxx = 1 +1;
											}
											*/
										}
		
										document->get_Selection(&selection);
										textSelection = (CComQIPtr<EnvDTE::TextSelection>)selection;
										textSelection->GotoLine(line,VARIANT_TRUE);										
									}
								}
							}
						}
					}
				}
			}//while project items in the stack
		}
	}
	
	return;

Error:
	//TODO handle errors !!!
	return;

}

void CToolWinComm::popOpenHTMLFile (std::string filename)
{
	USES_CONVERSION;

	CComQIPtr<EnvDTE::Window> window;
	CComQIPtr<EnvDTE::ItemOperations> itemOperations;

	m_pDTE->get_ItemOperations(&itemOperations);
	itemOperations->Navigate(A2BSTR(filename.c_str()),vsNavigateOptionsDefault,&window);
}

void CToolWinComm::pushClear (void)
{
	m_pToolWinObject->popClear();
}

#endif //USE_MARMOT_DICE