/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */

#include <QFileInfo>
#include <QDateTime>
#include "MarmotGUI.h"

#if !defined(DEBUG) && !defined(_DEBUG)
#define QT_NO_DEBUG_OUTPUT
#endif

MarmotGUI::MarmotGUI(QWidget *parent)
{
	    setupUi(this); // this sets up GUI
	    connect( listen_pushButton, SIGNAL ( clicked() ), this, SLOT ( listen() ) );//listen pressed
	    connect( cancel_pushButton, SIGNAL ( clicked() ), this, SLOT ( cancel() ) );//cancel pressed
	    connect( exepath_pushButton, SIGNAL ( clicked() ), this, SLOT ( getExePath() ) );//choose executable pressed
	    connect( run_pushButton, SIGNAL ( clicked() ), this, SLOT ( runExe() ) );//run pressed
	    connect( show_pushButton, SIGNAL ( clicked() ), this, SLOT ( showResults() ) );//show pressed
	    connect( logfiletype_comboBox, SIGNAL ( currentIndexChanged(int)), this, SLOT ( updateShowButton(int) ) );//logfiletype changed
	    connect( runlisten_checkBox, SIGNAL ( stateChanged(int)), this, SLOT ( runlistenChanged(int) ) );//run includes listen checked/unchecked
	    connect( output_tableWidget, SIGNAL ( cellDoubleClicked(int,int)), this, SLOT ( tableDoubleClicked(int,int)));//cell in table doubleclicked
	    connect( &m_sthread, SIGNAL ( NewMessage(QString) ), this, SLOT ( updateMessages(QString)));//message from the server thread
	    connect( &m_sthread, SIGNAL ( sconnected() ), this, SLOT (setStatusConnected()));//client has connected
	    connect( &m_sthread, SIGNAL ( sdisconnected() ), this, SLOT (setStatusDisconnected()));//client has disconnected
	    connect( &m_ethread, SIGNAL ( commanddone(int) ), this, SLOT (runFinished(int)));//execution finished
	    connect( errcodes_pushButton, SIGNAL ( clicked() ), this, SLOT (selectErrCodesClicked()));//"select ErrCodes" pressed
	    port_spinBox->setValue(30777);		//the default port
	    m_runincludeslisten=false;			//run doesn not include listen by default
	    debugmode_comboBox->insertItem(0,"Errors");	//corresponds to MARMOT_DEBUG_MODE
	    debugmode_comboBox->insertItem(1,"Errors and Warnings");
	    debugmode_comboBox->insertItem(2,"Errors, Warnings and Remarks");
	    logfiletype_comboBox->insertItem(0,"ASCII Logging");	//corresponds to MARMOT_LOGFILE_TYPE
	    logfiletype_comboBox->insertItem(1,"HTML Logging");
	    logfiletype_comboBox->insertItem(2,"CUBE Logging");
	    output_tableWidget->setColumnCount(11);			
	    output_tableWidget->setColumnWidth(0,30);
	    output_tableWidget->setColumnWidth(1,30);
	    output_tableWidget->setHorizontalHeaderItem(0,new QTableWidgetItem("Time",0));	//timesatamp column
	    output_tableWidget->setHorizontalHeaderItem(1,new QTableWidgetItem("P",0));		//process column
	    output_tableWidget->setHorizontalHeaderItem(2,new QTableWidgetItem("T",0));		//thread column
	    output_tableWidget->setHorizontalHeaderItem(3,new QTableWidgetItem("Type",0));	//type column
	    output_tableWidget->setHorizontalHeaderItem(4,new QTableWidgetItem("Detail",0));	//detail column
	    output_tableWidget->setHorizontalHeaderItem(5,new QTableWidgetItem("File",0));	//file column
	    output_tableWidget->setHorizontalHeaderItem(6,new QTableWidgetItem("Line",0));	//line column
	    output_tableWidget->setHorizontalHeaderItem(7,new QTableWidgetItem("Call",0));	//line column
	    output_tableWidget->setColumnWidth(2,30);
	    output_tableWidget->setColumnWidth(3,30);
	    output_tableWidget->setColumnWidth(4,300);
	    m_error_pixmap.load("error.png");	//displayed in the table in case of error
	    m_error_icon.addPixmap(m_error_pixmap);
	    m_warning_pixmap.load("warning.png");	//displayed in the table in case of warning
	    m_warning_icon.addPixmap(m_warning_pixmap);
	    m_info_pixmap.load("info.png");	//displayed in the table in case of info
	    m_info_icon.addPixmap(m_info_pixmap);
	    m_note_pixmap.load("note.png");	//displayed in the table in case of note
	    m_note_icon.addPixmap(m_note_pixmap);

	    getConfiguration();			//load the current configuration

      if (m_env_param == "")
      {
        // not in config
        qDebug("DEBUG: \"MpiexecEnvParam\" not in config. Going to call \"mpiexec -help\"");
        if (!getMpiexecEnvParam(m_env_param))
        {
          qDebug("DEBUG: Could not get mpiexec env.param style!");
        } else {
          qDebug("DEBUG: mpiexec env.-param is \"%s\"", qPrintable(m_env_param));
        }
      }
}

MarmotGUI::~MarmotGUI()
{
  qDebug("DEBUG: MarmotGUI::~MarmotGUI() called\n");
}

void MarmotGUI::listen()
{
	QString str;
	int port;	//port to listen on
	qDebug("DEBUG: MarmotGUI::listen() called\n");
	port = port_spinBox->value();	//get the port
	qDebug("DEBUG: port = %d\n", port);
	if (!m_sthread.isRunning())	//the server thread is not running
	{
		listen_pushButton->setEnabled(false);	//disable the listen button
		m_sthread.listen(port);			//set the listen port
		str+="Listening on port ";		//for the status
		str+=QString::number(port);
		status_lineEdit->setText(str);
	}
	else				//the server thread is already running
	{
		qDebug("DEBUG: m_sthread already running\n");
	}
}

void MarmotGUI::cancel()
{
  qDebug("DEBUG: MarmotGUI::cancel() called\n");
	if(m_sthread.isRunning())	//the server thread is running
	{
		qDebug("DEBUG: Terminating m_sthread\n");
		m_sthread.terminate();	//terminate the server thread
		m_sthread.wait();
		setStatusDisconnected(); //the client is now disconnected
	}
}

void MarmotGUI::updateMessages(QString message)
{
	QString tmp;
	QStringList strlist;
	strlist = message.split("\xF8");
	int num_items;			//number of strings in the string list
	int actual_row;			//actual row in the output table
	QTableWidgetItem *item;		//pointer to the item
	//cerr << "DEBUG: MarmotGUI::updateMessages("<<message.toStdString()<<") called " <<endl;
	//output_textEdit->append(message);
	actual_row = output_tableWidget->rowCount();	
	output_tableWidget->insertRow(actual_row);	//new row
	num_items = strlist.size();		//should be 11
	qDebug("DEBUG: MarmotGUI::updateMessages: got %d items!", num_items);
	if (num_items != 11)
	{
		qDebug("DEBUG: MarmotGUI: num_items!=11, here is the message:\n%s", qPrintable(message));
    //qDebug("DEBUG: MarmotGUI: num_items != 11\n");
	}
	for (int i=0;(i<num_items)&&(i<11);i++)
	{
		tmp=strlist[i];
		if (1==i)	//rank
		{
			if(!tmp.compare("-1"))	//global
			{
				item=new QTableWidgetItem("G",0);
			}
			else
			{
				item=new QTableWidgetItem(tmp,0);
			}
		}
		else if(3==i)	//type
		{
			if(!tmp.compare("0"))		//info
			{
				item=new QTableWidgetItem(m_info_icon,"",0);
			}
			else if(!tmp.compare("1"))	//warning
			{		
				item=new QTableWidgetItem(m_warning_icon,"",0);
			}
			else if(!tmp.compare("2"))	//error
			{
				item=new QTableWidgetItem(m_error_icon,"",0);
			}
			else if(!tmp.compare("3"))	//note
			{
				item=new QTableWidgetItem(m_note_icon,"",0);
			}
			else
			{
				item=new QTableWidgetItem(tmp,0);	//display the text
			}
		}
		else
		{
			item=new QTableWidgetItem(tmp,0);
		}
		output_tableWidget->setItem(actual_row,i,item);
	}

}

void MarmotGUI::setStatusConnected()
{
	qDebug("DEBUG: MarmotGUI::setStatusConnected() called\n");
	status_lineEdit->setText("Connected.");
}

void MarmotGUI::setStatusDisconnected()
{
  qDebug("DEBUG: MarmotGUI::setStatusDisconnected() called\n");
	status_lineEdit->setText("Disconnected.");
	if (!m_runincludeslisten)
		listen_pushButton->setEnabled(true);
}

//void MarmotGUI::socketError(QTcpSocket::SocketError err)
//{
//  qDebug("DEBUG: SocketError %d\n", (int)err);
//}

void MarmotGUI::getExePath()
{
  qDebug("DEBUG: MarmotGUI::getExePath() called\n");
	QString path;
	path = QFileDialog::getOpenFileName(this, "Choose an executable", QString::null, QString::null);
  exepath_lineEdit->setText(path);
}

void MarmotGUI::runExe()
{
	QString LogFileName;
	QString EnvAssignSymbol;	// " " for mpich, "=" for openmpi
	QString command;
	bool eff_errcodes[181];		//TODO: avoid this hardcoding
	QString eff_errcodes_str;
	int numprocs;
	int marmot_debug_mode;
	int marmot_logfile_type;
	QString marmot_logfile_name;
	qDebug("DEBUG: MarmotGUI::runExe() called\n");
	numprocs = numprocs_spinBox->value();	//get numprocs
	command = "mpiexec";
	if(!m_env_param.compare(" -x "))	//i.e. openmpi
	{
		EnvAssignSymbol="=";
	}
	if(!m_env_param.compare(" -env "))	//i.e. mpich
	{
		EnvAssignSymbol=" ";
	}
	if(m_runincludeslisten)
	{
		//Set MARMOT_LOG_HOST
  		command += m_env_param;
  		command += "MARMOT_LOG_HOST";
  		command+=EnvAssignSymbol;
		command += m_LogHost;
		//Set MARMOT_LOG_PORT
  		command += m_env_param;
		command += "MARMOT_LOG_PORT";
		command+=EnvAssignSymbol;
		command += m_LogPort;
	}
	//Set MARMOT_DEBUG_MODE
	marmot_debug_mode = debugmode_comboBox->currentIndex(); //get debugmode
  	command += m_env_param;
	command += "MARMOT_DEBUG_MODE";
	command+=EnvAssignSymbol;
	command += QString::number(marmot_debug_mode);
	//Set MARMOT_ERRCODES_SET
	m_conf.getEffectiveErrCodes(eff_errcodes);
	for(int i=0;i<181;i++)		//TODO: avoid hardcoding
	{
		if(!eff_errcodes[i])
		{
			eff_errcodes_str+="^";
		}
		eff_errcodes_str+=QString::number(i);
		if(i<181-1)
		{
			eff_errcodes_str+=",";
		}
	}
	command += m_env_param;
	command += "MARMOT_ERRCODES_SET";
	command+=EnvAssignSymbol;
	command+=eff_errcodes_str;
	//Set MARMOT_LOGFILE_TYPE
	marmot_logfile_type = logfiletype_comboBox->currentIndex(); //get logfile type
  	command += m_env_param;
	command += "MARMOT_LOGFILE_TYPE";
	command+=EnvAssignSymbol;
	command += QString::number(marmot_logfile_type);
	//Set MARMOT_LOGFILE_NAME
	marmot_logfile_name = getLogFileName();	//get logfile name
  	command += m_env_param;
	command += "MARMOT_LOGFILE_NAME";
	command+=EnvAssignSymbol;
	command += marmot_logfile_name;
	command += " -n ";
	command += QString::number(numprocs);
	command += " ";
	command += exepath_lineEdit->text();
	command += " ";
	command += arguments_lineEdit->text();
	qDebug("DEBUG: command = %s\n", qPrintable(command));
	output_tableWidget->setRowCount(0);

	//update the current configuration
	m_conf.setExePath(exepath_lineEdit->text());
	m_conf.setExeArguments(arguments_lineEdit->text());
	m_conf.setExeNumprocs(numprocs_spinBox->value());
	m_conf.setEnvDebugmode(debugmode_comboBox->currentIndex());
	m_conf.setEnvLogfiletype(logfiletype_comboBox->currentIndex());
	m_conf.setEnvLogport(port_spinBox->value());
	m_conf.setEnvLoghost(host_lineEdit->text());
	m_conf.setConfiguration();

	if (!m_ethread.isRunning())	//exe thread is not running
	{
		LogFileName = getLogFileName();	//get logfile name
		qDebug("DEBUG: MarmotGUI::runExe(): LogFileName = %s\n", qPrintable(LogFileName));
		logfilename_lineEdit->setText(LogFileName);
		run_pushButton->setEnabled(false);
		if (m_runincludeslisten)
		{
			listen();
		}
		m_ethread.runexe(command);
  } else {  //exe thread is running
    qDebug("DEBUG: m_ethread already running\n");
	}
}

QString MarmotGUI::getLogFileName()
{
  /*
	std::string ExeName;
	ExeName=(exepath_lineEdit->text()).toStdString();
	std::string::size_type pos = ExeName.find_last_of("/\\");
	if (pos != std::string::npos)
	{
		ExeName = ExeName.substr(pos+1);
	}
	std::string LogFileName;  // for the name of the log file
	char TimeStr[16];         // for the timestamp
	time_t now;
	struct tm *snow = NULL;
	time(&now);
	snow = localtime(&now);
	sprintf(TimeStr, "%i%02i%02i_%02i%02i%02i", snow->tm_year+1900, snow->tm_mon+1, snow->tm_mday, snow->tm_hour, snow->tm_min, snow->tm_sec);
	LogFileName = "Marmot_";
	LogFileName.append(ExeName);
	LogFileName.append("_");
	LogFileName.append(TimeStr);
	return QString::fromStdString(LogFileName);
  */
  QFileInfo exe_info(exepath_lineEdit->text());
  QString exe_path = exe_info.absolutePath();
  QString exe_name = exe_info.completeBaseName();
  QDateTime dt = QDateTime::currentDateTime();
  QString timestr = dt.toString(tr("yyyyMMdd_hhmmss"));
  QString logfile_name("Marmot_");
  logfile_name.append(exe_name);
  logfile_name.append("_");
  logfile_name.append(timestr);
  return logfile_name;
}

void MarmotGUI::updateShowButton(int type)
{
	qDebug("DEBUG: MarmotGUI::updateShowButton(%d) called\n", type);
	switch (type)
	{
		case 0:
			show_pushButton->setText("Show ASCII Log");
			break;
		case 1:
			show_pushButton->setText("Show HTML Log");
			break;
		case 2:
			show_pushButton->setText("Show CUBE Log");
			break;
	}
}


void MarmotGUI::showResults()
{
	int ret;
	int type;
	QString command;
	QString extension;
	QString filename;

  qDebug("DEBUG: MarmotGUI::showResults() called\n");
	type=logfiletype_comboBox->currentIndex();
	filename=logfilename_lineEdit->text();
	switch (type)
	{
		case 0:
#ifdef _WIN32
      command+="notepad";
#else
		 	command+="kwrite";	//make that configurable
#endif
			extension="txt";
			break;
		case 1:
			command+="firefox";	//make that configurable
			extension="html";
			break;
		case 2:
			command+="cube3-qt";
			extension="cube";
			break;
	}
	command += " ";
	command += filename;
	command += ".";
	command += extension;
	qDebug("DEBUG: MarmotGUI::showResults() command = %s\n", qPrintable(command));
	ret = system((command.toStdString()).c_str());
  qDebug("DEBUG: MarmotGUI::showResults(): err = %d\n", ret);
	if (ret)
	{
		QString err;
		err = "The following command was not successful:\r\n";
		err += command;
		err += "\r\n";
		err += "The return value was ";
		err += QString::number(ret);
		err += "\r\n";
		QMessageBox::critical(this, "MarmotGUI error", err);
	}
}

void MarmotGUI::runFinished(int ret)
{
  qDebug("DEBUG: MarmotGUI::runFinished() called\n");
  qDebug("DEBUG: MarmotGUI::runFinished(): ret = %d", ret);
	if (ret)
	{
		QString err;
		err = "The mpirun command was not successful\r\n";
		err += "The return value was ";
		err += QString::number(ret);
		err += "\r\n";
		QMessageBox::critical(this, "MarmotGUI error", err);
	}
	run_pushButton->setEnabled(true);
}

void MarmotGUI::runlistenChanged(int value)
{
  qDebug("DEBUG: MarmotGUI::runlistenChanged(%d) called\n", value);
	if (value)
	{
		m_runincludeslisten = true;
		listen_pushButton->setEnabled(false);
		//m_LogHost = "localhost";
		m_LogHost = host_lineEdit->text();
		m_LogPort = QString::number(port_spinBox->value());
	} else {
		m_runincludeslisten=false;
		if (!m_sthread.isRunning())
			listen_pushButton->setEnabled(true);
		//m_LogHost="";
		//m_LogPort="";
	}
}

void MarmotGUI::tableDoubleClicked(int row, int column)
{
	QString str;
  qDebug("DEBUG: MarmotGUI::tableDoubleClicked(row=%d, column=%d) called\n", row, column);
	str = (output_tableWidget->item(row,column))->text();
	QMessageBox::information(this, "MarmotGUI message box", str);
  qDebug("%s\n", qPrintable(str));
}

bool MarmotGUI::getConfiguration()
{
  qDebug("DEBUG: MarmotGUI::getConfiguration() called\n");
  // uses default in user's home-dir
  return getConfiguration(QString(""));
}

bool MarmotGUI::getConfiguration(QString filename)
{
	char hostname[256];	//the host the GUI is running on
	qDebug("DEBUG: MarmotGUI::getConfiguration(%s) called\n", qPrintable(filename));
	m_conf.setConfFile(filename);
	if (m_conf.getConfiguration())
	{
		exepath_lineEdit->setText(m_conf.getExePath());
		arguments_lineEdit->setText(m_conf.getExeArguments());
		numprocs_spinBox->setValue(m_conf.getExeNumprocs());
		debugmode_comboBox->setCurrentIndex(m_conf.getDebugMode());
		logfiletype_comboBox->setCurrentIndex(m_conf.getLogfileType());
		port_spinBox->setValue(m_conf.getLogPort());
		gethostname(hostname,256);		//get the hostname of the local machine
		//host_lineEdit->setText(m_conf.getLogHost());	
		host_lineEdit->setText(QString(hostname));	//the hostname is the default setting
    m_env_param = m_conf.getMpiexecEnvParam();
    return true;
	}
	return false;
}

bool MarmotGUI::getMpiexecEnvParam(QString &envparam)
{
  QProcess proc;
  proc.setProcessChannelMode(QProcess::MergedChannels);
  proc.start("mpiexec", QStringList() << "-help");
  if (!proc.waitForStarted())
  {
    qDebug("DEBUG: Could not launch \"mpiexec -help\"");
    return false;
  }

  if (!proc.waitForFinished())
  {
    qDebug("DEBUG: waitForFinished() failed!");
    return false;
  }

  QByteArray output = proc.readAll();
  int pos = output.indexOf("-env ");
  if (pos != -1)
  {
    envparam = " -env ";
    return true;
  }
  pos = output.indexOf("-x ");
  if (pos != -1)
  {
    envparam = " -x ";
    return true;
  }
  // could not figure out
  return false;
}

void MarmotGUI::selectErrCodesClicked()
{
      qDebug("DEBUG: MarmotGUI::selectErrCodesClicked() called\n");
      MarmotErrCodes *errcodes_dialog = new MarmotErrCodes(&m_conf,this);
      errcodes_dialog->setModal(true);
      errcodes_dialog->show();
      return;
}
