/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
#ifndef MARMOTGUI_H
#define MARMOTGUI_H

#include <QtGui>
//#include <string>
//#include <iostream>
//#include <QTcpSocket>
#include <QWidget>
#include "ui_marmotgui.h"
#include "ServerThread.h"
#include "ExeThread.h"
#include "MarmotGUIConfiguration.h"
#include "MarmotErrCodes.h"
//#include <unistd.h>
#include "NetUtil.h"

/*!
 this is the main class of the MarmotGUI
 */
class MarmotGUI : public QWidget, private Ui::MarmotGUIDlg
{
  Q_OBJECT
		     
	public:
    MarmotGUI(QWidget *parent = 0);
		~MarmotGUI(); 
			 
	public slots:
	private slots:
		/*!
		 is called when the listen button is pressed
		 */
		void listen();
		/*!
		 is called when the cancel button is pressed
		 */
		void cancel();
		/*!
		 is called when a new message from the server thread arrives
		 \param message the message from the server thread
		 */
		void updateMessages(QString message);
		/*!
		 is called when the client has connected 
		 */
		void setStatusConnected();
		/*!
		 is called when the client has disconnected
		 */
		void setStatusDisconnected();

    //void socketError(QTcpSocket::SocketError socketError);

		/*!
		 is called when the choose executable button is pressed
		 */
		void getExePath();
		/*!
		 is called when the  run button is pressed
		 */
		void runExe();
		/*!
		 is called when the logfiletype has changed to adjust the show button
		 \param type 0==ASCII 1==HTML 2==CUBE
		 */
		void updateShowButton(int type);
		/*!
		 is called when the show button is pressed
		 */
		void showResults();
		/*!
		 is called when the execution has finished
		 \param ret the return value of the command
		 */
		void runFinished(int ret);
		/*!
		 is called when the run includes listen chechbox' state has changed
		 \param value index of the current item
		 */
		void runlistenChanged(int value);
		/*!
		 is called when a cell of the table has been doubeclicked
		 \param row the cell's row
		 \param column the cell's column
		 */
		void tableDoubleClicked(int row, int column);
		/*!
		 *is called when the button "select ErrCodes" has been clicked
		 */
		void selectErrCodesClicked();

	private:
		/*!
		 this object holds the configuration
		 */
		MarmotGUIConfiguration m_conf;
		/*!
		 gets the configuration from the file "$HOME/.marmot_current.conf"
		 */
		bool getConfiguration();
		/*!
		 gets the configuration
		 \param filename the file to to read the configuration from
		 */
		bool getConfiguration(QString filename);

    bool getMpiexecEnvParam(QString &envparam);

		/*!
		 the server thread (communicates with Marmot's debug server)
		 */
		ServerThread m_sthread;
		/*!
		 the thread running the executable
		 */
		ExeThread m_ethread;
		/*!
		 \return the logfile name as known from Marmot (with timestamp)
		 */
		QString getLogFileName();
		/*!
		 if true, pressing the run button triggers the server thread
		 */
		bool m_runincludeslisten;
		/*!
		 the host the messages are sent to by Marmot (currently always localhost)
		 */
		QString m_LogHost;
		/*!
		 the port the server thread listen on (default 30777)
		 */
		QString m_LogPort;
    		QString m_env_param;

		QPixmap m_warning_pixmap;
		QIcon m_warning_icon;
		QPixmap m_error_pixmap;
		QIcon m_error_icon;
		QPixmap m_info_pixmap;
		QIcon m_info_icon;
		QPixmap m_note_pixmap;
		QIcon m_note_icon;
};
 
 
#endif
