/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */

#include <QtCore>
#include <QMessageBox>
#include <QString>
#include <QByteArray>
#include <QList>
#include <string>
#include "SocketAddr.h"
#include "ServerThread.h"

#if !defined(DEBUG) && !defined(_DEBUG)
#define QT_NO_DEBUG_OUTPUT
#endif

ServerThread::ServerThread(QObject *parent) : QThread(parent), m_port(30777), m_abort(false), m_serversocket(NULL), m_channel(NULL)
{
  qDebug("DEBUG: ServerThread::ServerThread called\n");
}

ServerThread::~ServerThread()
{
  qDebug("DEBUG: ServerThread::~ServerThread() called\n");
  resetSockets();
}

void ServerThread::run()
{
	int num_messages=0;	// number of messages received from client
	int num_acks=0;		  // number of acks send to the client

	QString str;
  std::string ack_msg("ACK");

  qDebug("DEBUG: ServerThread::run() called\n");

  if (!m_serversocket)
  {
    qDebug("DEBUG: ServerThread started without listen-socket!!!");
    return;
  }

  m_channel = m_serversocket->accept();
  // Could we establish a client channel?
  if (m_channel == NULL)
  {
    qDebug("DEBUG: Could not establish client-channel!");
    return;
  }

  if (m_channel->isOk())
  {
    // print peer address
    CSocketAddr peer_sa = m_channel->getPeerAddress();
    qDebug("DEBUG: Client connection established from %s", peer_sa.toString().c_str());
    // set linger to 1 sec.
    m_channel->setLinger(true, 1);
    emit sconnected();
  } else {
    qDebug("DEBUG: m_channel not OK!");
  }

  // read stuff from marmot debug-server
  while (!m_abort && m_channel->isOk())
  {
    std::string tmp_data = m_channel->readln();
    QByteArray line_data = tmp_data.c_str();
    if (line_data.isEmpty())
    {
      qDebug("DEBUG: read empty line!");
      continue;
    }
    num_messages++;  //increase number of messages received
    str = line_data;
    emit NewMessage(str);  //send the message to the GUI
    // peek to the message and confirm errors
    QList<QByteArray> parts = line_data.split('\xF8');
    if (parts.size() > 4)
    {
      QString hstr = parts.at(3);
      int msg_type = hstr.toInt();
      if (msg_type == 2)  // it's an error message
      {
        // send an ACK
        m_channel->writeln(ack_msg);
        num_acks++;
      }
    }
    // qDebug("DEBUG: ServerThread: num_messages = %d  num_acks = %d\n", num_messages, num_acks);
  }
  qDebug("DEBUG: ServerThread::run() before exit\n");
  if (m_channel)
  {
    delete m_channel;  m_channel = NULL;
  }
  emit sdisconnected();	//tell the GUI the client is disconnected

}

void ServerThread::cancel()
{
	qDebug("DEBUG: ServerThread::cancel() called\n");
	m_abort = true;
}

bool ServerThread::listen(int port)
{
  qDebug("DEBUG: ServerThread::listen(%d) called\n", port);
  if (isRunning())
  {
    qDebug("DEBUG: ServerThread is running! Cannot listen()!\n");
    return false;
  }
  // create server-socket if there is none yet or the listen-port changed
  if ((m_serversocket == NULL) || (m_port != port))
  {
    resetSockets();
    m_port = port;
    m_serversocket = new CTcpSocket(m_port);
    if (!m_serversocket)
    {
      QMessageBox::critical(NULL, tr("MarmotGUI"), tr("Could not create listen socket! Alloc failed!"));
      return false;
    }
    if (!m_serversocket->isOk())
    {
      QMessageBox::critical(NULL, tr("MarmotGUI"), tr(m_serversocket->getError().c_str()));
      return false;
    }
  }
	m_abort = false;
	start(LowPriority);
  return true;
}

void ServerThread::resetSockets()
{
  if (m_channel)
  {
    delete m_channel;  m_channel = NULL;
  }
  if (m_serversocket)
  {
    delete m_serversocket;  m_serversocket = NULL;
  }
}
