/**
 *  @file
 *
 *  This program is a very simple MPI program to verify that the 
 *  library works correctly.
 *
 *  It just calls 
 *  - MPI_Init
 *  - MPI_Finalize 
 *  - MPI_Comm_rank
 *  - MPI_Comm_size
 *  - MPI_Send
 *  - MPI_Recv
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller
 *
 *  $Id: basic_mult.c 319 2004-08-16 11:25:02Z rusbetti $  
 */

#include <stdio.h>
#include <assert.h>
#include "mpi.h"

int main(int argc, char **argv)
{
    const int MSG_TAG = 17;
    const int COUNT = 1;

    int size = -1;
    int rank = -1;
    int value0 = -1;
    int value = -1;
    int value2 = -1;

    MPI::Status status;

    MPI::Init(argc, argv);
    size=MPI::COMM_WORLD.Get_size();
    rank=MPI::COMM_WORLD.Get_rank();
    printf(" I am rank %d of %d PEs\n", rank, size);

    if (size < 3)
    {
        printf("At least 3 nodes for application needed! \n");
    }
    else
    {
        if (rank == 0)
        {
		MPI::COMM_WORLD.Recv(&value0, COUNT, MPI::INT, 1, MSG_TAG,status);
            assert(value0 == 19);

	    MPI::COMM_WORLD.Send(&value0, COUNT, MPI::INT, 2, MSG_TAG);
        }
        if (rank == 1)
        {
            value = 19;
	    MPI::COMM_WORLD.Send(&value, COUNT, MPI::INT, 0, MSG_TAG);
	    MPI::COMM_WORLD.Recv(&value, COUNT, MPI::INT, 2, MSG_TAG,status);
            assert(value == 19);
        }
        if (rank == 2)
        {
		MPI::COMM_WORLD.Recv(&value2, COUNT, MPI::INT, 0, MSG_TAG,status);
            assert(value2 == 19);

	    MPI::COMM_WORLD.Send(&value2, COUNT, MPI::INT, 1, MSG_TAG);
        }
    }

    MPI::Finalize();

    return 0;
}
