/**
 *  @file
 *
 *  This program is a very simple MPI program to verify that the 
 *  library works correctly.
 *  
 *  It just calls 
 *  - MPI_Init
 *  - MPI_Finalize 
 *  - MPI_Comm_rank
 *  - MPI_Comm_size
 *  - MPI_Comm_group
 *  - MPI_Group_incl
 *  - MPI_Group_excl
 *  - MPI_Comm_create
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller
 *
 *  $Id: comm-create-f2.c 319 2004-08-16 11:25:02Z rusbetti $  
 */

#include <stdio.h>
#include <assert.h>
#include "mpi.h"

int main(int argc, char **argv)
{
    int size = -1;
    int rank = -1;
    int ranks1 = -1;
    int ranks2 = 1;
 
    MPI::Intracomm falsecomm;
    MPI::Intracomm uppercomm;

    MPI::Group worldgroup;
    MPI::Group lowgroup;
    MPI::Group uppergroup;

    printf("We call Comm_create twice:\n"
           "1: correctly, 2: group s not a subset of communicator \n");
    MPI::Init(argc, argv);
    size=MPI::COMM_WORLD.Get_size();
    rank=MPI::COMM_WORLD.Get_rank();

    ranks1 = size - 1;

    printf(" I am rank %d of %d PEs\n", rank, size);

    worldgroup=MPI::COMM_WORLD.Get_group();
    lowgroup=worldgroup.Excl(1, &ranks1);
    uppergroup=worldgroup.Excl(1, &ranks2);
    uppercomm=MPI::COMM_WORLD.Create(uppergroup);

    if (uppercomm != MPI::COMM_NULL)
    {
        falsecomm=uppercomm.Create(lowgroup);
    }

    MPI::Finalize();

    return 1;
}
