/**
 *  @file
 *
 *  This program is a very simple MPI program to verify that the 
 *  library works correctly.
 *
 *  It just calls 
 *  - MPI_Init
 *  - MPI_Finalize 
 *  - MPI_Comm_rank
 *  - MPI_Comm_size
 *  - MPI_Comm_group
 *  - MPI_Group_incl
 *  - MPI_Group_excl
 *  - MPI_Comm_create
 *  - MPI_Group_translate_ranks
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller
 *
 *  $Id: group-translate-ranks.c 319 2004-08-16 11:25:02Z rusbetti $  
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "mpi.h"

int main(int argc, char **argv)
{
    int size = -1;
    int rank = -1;
    int undef = 0;
    int *ranks = 0;
    int *ranks1 = 0;
    int *ranks2 = 0;
    int num = -1;
    int num2 = -1;
    int i = 0;

    MPI::Group worldgroup;
    MPI::Group lowergroup;
    MPI::Group uppergroup;

    MPI::Init(argc, argv);
    size=MPI::COMM_WORLD.Get_size();
    rank=MPI::COMM_WORLD.Get_rank();

    num = size / 2;
    ranks = (int *) malloc(num * sizeof(int));

    for (i = 0; i <= num - 1; i++)
    {
        ranks[i] = i;
    }

    printf(" I am rank %d of %d PEs\n", rank, size);

    worldgroup=MPI::COMM_WORLD.Get_group();
    /* lowergroup contains lower half of ranks in COMM_WORLD */
    lowergroup=worldgroup.Incl(num, ranks);
    /* uppergroup is disjunct to lowergroup an contains upper half of ranks */
    uppergroup=worldgroup.Excl(num, ranks);
    num2=lowergroup.Get_size();

    ranks1 = (int *) malloc(num2 * sizeof(int));
    ranks2 = (int *) malloc(num2 * sizeof(int));

    for (i = 0; i <= num2 - 1; i++)
    {
        ranks1[i] = i;
        ranks2[i] = 0;
    }

    //MPI_Group_translate_ranks(lowergroup, num2, ranks1, uppergroup, ranks2);
    MPI::Group::Translate_ranks(lowergroup, num2, ranks1, uppergroup, ranks2);

    for (i = 0; i <= num2 - 1; i++)
    {
        if (ranks2[i] == MPI::UNDEFINED)
        {
            undef++;
        }
    }

    if (rank == 0)
    {
        printf(" -> %d ranks not in second group! \n", undef);
    }

    free(ranks);
    free(ranks1);
    free(ranks2);

    MPI::Finalize();

    return 1;
}
