/**
 *  @file
 *
 *  This program is a very simple MPI program to verify that the 
 *  library works correctly.
 *
 *  It just calls 
 *  - MPI_Init
 *  - MPI_Finalize 
 *  - MPI_Comm_rank
 *  - MPI_Comm_size
 *  - MPI_Send_init
 *  - MPI_Startall
 *  - MPI_Waitall
 *  - MPI_Recv
 *  - MPI_Request_free
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller
 *
 *  $Id: pers_requests2.c 319 2004-08-16 11:25:02Z rusbetti $  
 */

 /*
  ** We have 3 requests in the array - one is reused (but freed before),
  ** one usual and one which is non-valid
  */

#include <stdio.h>
#include <assert.h>
#include "mpi.h"

int main(int argc, char **argv)
{
    const int COUNT = 1;
    const int MSG_TAG_1 = 17;
    const int MSG_TAG_2 = 18;
    const int MSG_TAG_3 = 19;

    int size = -1;
    int rank = -1;
    int value[3] = { 0, 0, 0 };
    int value2[3] = { 0, 0, 0 };
    int i = 0;

    MPI::Status status[3];
    MPI::Status status_arr[3];

    MPI::Prequest prequest_arr[3];

    MPI::Init(argc, argv);
    size=MPI::COMM_WORLD.Get_size();
    rank=MPI::COMM_WORLD.Get_rank();
    printf(" I am rank %d of %d PEs\n", rank, size);

    for (i = 0; i <= 2; i++)
    {
        value[i] = 37;
        value2[i] = -1;
    }

    if (rank == 0)
    {
        /* going to receive message */
	MPI::COMM_WORLD.Recv(&value2[0], COUNT, MPI::INT, 1, MSG_TAG_1, status[0]);
	MPI::COMM_WORLD.Recv(&value2[COUNT], 1, MPI::INT, 1, MSG_TAG_2, status[1]);
	MPI::COMM_WORLD.Recv(&value2[2], COUNT, MPI::INT, 1, MSG_TAG_3, status[2]);
    }

    if (rank == 1)
    {
        /* going to send message */
        prequest_arr[0]=MPI::COMM_WORLD.Send_init(&value[0], COUNT, MPI::INT, 0, MSG_TAG_1);
        prequest_arr[1]=MPI::COMM_WORLD.Send_init(&value[0], COUNT, MPI::INT, 0, MSG_TAG_2);
        prequest_arr[2]=MPI::COMM_WORLD.Send_init(&value[0], COUNT, MPI::INT, 0, MSG_TAG_3);
	MPI::Prequest::Startall(3, prequest_arr);
	MPI::Request::Waitall(3, prequest_arr, status_arr);

        for (i = 0; i <= 2; i++)
        {
            prequest_arr[i].Free();
        }
    }

    MPI::Finalize();

    return 0;
}
