/**
 *  @file
 *
 *  This program is a very simple MPI program to verify that the 
 *  library works correctly.
 *
 *  It just calls 
 *  - MPI_Init
 *  - MPI_Finalize 
 *  - MPI_Comm_rank
 *  - MPI_Comm_size
 *  - MPI_Reduce-scatter
 *  - MPI_Barrier
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller
 *
 *  $Id: reduce_scatter.c 319 2004-08-16 11:25:02Z rusbetti $  
 */


#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "mpi.h"

int main(int argc, char **argv)
{
    int i = 0;
    int size = -1;
    int rank = -1;
    int sendvalue[5] = { 0, 0, 0, 0, 0 };
    int recvvalue[2] = { 0, 0 };
    int *recvcounts = 0;

    MPI::Init(argc, argv);
    size=MPI::COMM_WORLD.Get_size();
    rank=MPI::COMM_WORLD.Get_rank();
    printf(" I am rank %d of %d PEs\n", rank, size);

    if (size < 5)
    {
        printf("We need at least 5 nodes! \n");
    }
    else
    {
        if (rank < 4)
        {
            for (i = 0; i < 5; i++)
                sendvalue[i] = size + rank - i;
        }
        else
        {
            for (i = 0; i < 5; i++)
            {
                sendvalue[i] = 0;
            }
        }

        /* define recvcounts */
        recvcounts = (int *) malloc(size * sizeof(int));
        for (i = 0; i < 4; i++)
        {
            recvcounts[i] = 1;
        }
        recvcounts[2] = 2;
        for (i = 4; i < size; i++)
        {
            recvcounts[i] = 0;
        }

	MPI::COMM_WORLD.Barrier();
	MPI::COMM_WORLD.Reduce_scatter(&sendvalue, &recvvalue, recvcounts, MPI::INT,
                           MPI::MAX);


        switch (rank)
        {
        case (0):
            assert(recvvalue[0] == size + 3);
            break;
        case (1):
            assert(recvvalue[0] == size + 2);
            break;
        case (2):
            assert(recvvalue[0] == size + 1);
            assert(recvvalue[1] == size);
            break;
        case (3):
            assert(recvvalue[0] == size - 1);
            break;
        default:
            /* do nothing. */
            break;
        }

        free(recvcounts);
    }                           /* end of else (enough nodes) */

    MPI::Finalize();

    return 0;
}
