dnl =========================================================================
dnl MARMOT's macros (named MAC_*) to be used in configure.ac.
dnl
dnl Parts of this script are based on the script found in PACX-MPI,
dnl thanks to Rainer Keller.
dnl 
dnl
dnl $Id: acinclude.m4 812 2008-04-11 12:23:32Z tobias $
dnl
dnl @authors: Bettina Krammer, Rainer Keller, Tobias Hilbrich
dnl =========================================================================


dnl ==================== 
dnl MAC_GET_FORTNAMES
dnl ====================

dnl Dependencies: F77, FFLAGS

AC_DEFUN([MAC_GET_FORTNAMES], [
   # Check for strange behavior of Fortran.  For example, some FreeBSD
   # systems use f2c to implement f77, and the version of f2c that they 
   # use generates TWO (!!!) trailing underscores
   # Currently, WDEF is not used but could be...
   #
   # Eventually, we want to be able to override the choices here and
   # force a particular form.  This is particularly useful in systems
   # where a Fortran compiler option is used to force a particular
   # external name format (rs6000 xlf, for example).
   AC_MSG_CHECKING([for Fortran names])
   cat > confftest.f <<EOF
       subroutine mpir_init_fop( a )
       integer a
       a = 1
       return
       end
EOF
   $F77 $FFLAGS -c confftest.f > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "Unable to test Fortran compiler"
        echo "compiling a test program failed to produce an object file."
        NOF77=1
        HAS_FORTRAN=0
   elif test -z "$FORTRANNAMES" ; then
    # We have to be careful here, since the name may occur in several
    # forms.  We try to handle this by testing for several forms
    # directly.
    if test $arch_CRAY ; then
     # Cray does not accept -a ...
     nameform1=`strings confftest.o | grep mpir_init_fop_  | sed -n -e '1p'`
     nameform2=`strings confftest.o | grep MPIR_INIT_FOP   | sed -n -e '1p'`
     nameform3=`strings confftest.o | grep mpir_init_fop   | sed -n -e '1p'`
     nameform4=`strings confftest.o | grep mpir_init_fop__ | sed -n -e '1p'`
    else
     nameform1=`strings -a confftest.o | grep mpir_init_fop_  | sed -n -e '1p'`
     nameform2=`strings -a confftest.o | grep MPIR_INIT_FOP   | sed -n -e '1p'`
     nameform3=`strings -a confftest.o | grep mpir_init_fop   | sed -n -e '1p'`
     nameform4=`strings -a confftest.o | grep mpir_init_fop__ | sed -n -e '1p'`
    fi
    /bin/rm -f confftest.f confftest.o
    if test -n "$nameform4" ; then
        AC_MSG_RESULT(Fortran externals are lower case and have 1 or 2 trailing underscores)
        FORTRANNAMES="FORTRANDOUBLEUNDERSCORE"
        AWK_COMMAND_PACX='[{printf("#define %s %s__\n",$][2,tolower($][2))}]'
        AWK_COMMAND_PPACX='[{printf("#define %s p%s__\n",$][2,tolower($][2))}]'
dnl        AC_DEFINE(FORTRANDOUBLEUNDERSCORE)
    elif test -n "$nameform1" ; then
        # We do not set this in CFLAGS; it is a default case
        AC_MSG_RESULT(Fortran externals have a trailing underscore and are lowercase)
        FORTRANNAMES="FORTRANUNDERSCORE"
        AWK_COMMAND_PACX='[{printf("#define %s %s_\n",$][2,tolower($][2))}]'
        AWK_COMMAND_PPACX='[{printf("#define %s p%s_\n",$][2,tolower($][2))}]'
dnl        AC_DEFINE(FORTRANUNDERSCORE)
    elif test -n "$nameform2" ; then
        AC_MSG_RESULT(Fortran externals are uppercase)
        FORTRANNAMES="FORTRANCAPS"
        AWK_COMMAND_PACX='[{printf("#define %s %s\n",$][2,toupper($][2))}]'
        AWK_COMMAND_PPACX='[{printf("#define %s P%s\n",$][2,toupper($][2))}]'
dnl        AC_DEFINE(FORTRANCAPS)
    elif test -n "$nameform3" ; then
        AC_MSG_RESULT(Fortran externals are lower case)
        FORTRANNAMES="FORTRANNOUNDERSCORE"
        AWK_COMMAND_PACX='[{printf("#define %s %s\n",$][2,tolower($][2))}]'
        AWK_COMMAND_PPACX='[{printf("#define %s p%s\n",$][2,tolower($][2))}]'
dnl        AC_DEFINE(FORTRANNOUNDERSCORE)
    else
        echo "Unable to determine the form of Fortran external names"
        echo "Make sure that the compiler $F77 can be run on this system"
        NOF77=1
        HAS_FORTRAN=0
    fi
    fi
dnl    if test -n "$FORTRANNAMES" ; then
dnl        WDEF="-D$FORTRANNAMES"
dnl    fi
    ])dnl end of MAC_GET_FORTNAMES




dnl ====================
dnl MAC_PROG_AR
dnl ====================

dnl Sets AR.
dnl Check for AR and override AR for some compilers.
dnl Because of KCC compiler that doesn't like "ar" but "KCC -o"
dnl to create our libraries.
dnl And on SX, we use the sxar command (ar does not work there properly).
dnl we could do that by testing for the host
dnl but currently we test for the compiler.

AC_DEFUN([MAC_PROG_AR], [
  AC_ARG_VAR([AR],
             [Archiver to create libraries, useful for cross compilation])
  AC_CHECK_TOOL([AR], [ar])

  dnl For KCC compiler
  if test "x$CXX" = "xKCC" ; then
    ar="KCC -o"
  dnl For SX compiler
  elif test "x$CXX" = "xsxc++" ; then
    ar="sxar -rvs"
  elif test "x$CXX" = "xsxmpic++" ; then
    ar="sxar -rvs"
  dnl all other compilers
  else
    ar="$AR cru"
  fi
  AR=$ar
  AC_SUBST([AR])
])


dnl ====================
dnl MAC_PROG_PRELINK
dnl ====================

dnl Sets PRELINK for SX-* machines (cross compilation).

AC_DEFUN([MAC_PROG_PRELINK], [
  AC_ARG_VAR([PRELINK],
             [use -prelink on SX machines])

  dnl For SX compiler
  if test "x$CXX" = "xsxc++" ; then
    prelink="sxc++ -prelink "
  dnl all other compilers
  else
    prelink=": "
  fi
  PRELINK=$prelink
  AC_SUBST([PRELINK])
])


dnl ====================
dnl MAC_PROG_RANLIB
dnl ====================

dnl Sets RANLIB.
dnl Check for RANLIB and override RANLIB for SX-* machines.

AC_DEFUN([MAC_PROG_RANLIB], [
  AC_ARG_VAR([RANLIB],
             [ranlib to bless libraries])
  AC_CHECK_TOOL([RANLIB], [ranlib])

  dnl For SX compiler
  if test "x$CXX" = "xsxc++" ; then
    ranlib=": "
  dnl all other compilers
  else
    ranlib="$RANLIB "
  fi
  RANLIB=$ranlib
  AC_SUBST([RANLIB])
])




dnl ==================== 
dnl MAC_SEARCH_LIBS
dnl ====================

# MAC_SEARCH_LIBS(FUNCTION, SEARCH-DIR, SEARCH-LIBS,
#                 [ACTION-IF-FOUND], [ACTION-IF-NOT-FOUND],
#                 [OTHER-LIBRARIES])
# --------------------------------------------------------
# Search for a library defining FUNCTION, if it's not already available.
# Like AC_SEARCH_LIBS, but we look in SEARCH-DIR.
# The result is $ac_cv_found_lib (just lib name, e.g. mpi instead of -lmpi)
AC_DEFUN([MAC_SEARCH_LIBS],[
  AC_CACHE_CHECK([for library containing $1], [ac_cv_search_$1],
  [ac_func_search_save_LIBS=$LIBS
  ac_cv_search_$1=no
  AC_TRY_LINK_FUNC([$1], [ac_cv_search_$1="none required"])
  if test "$ac_cv_search_$1" = no; then
    for ac_lib in $3; do
      LIBS="-L$2 -l$ac_lib $6 $ac_func_search_save_LIBS"
      AC_TRY_LINK_FUNC([$1],
                       [ac_cv_search_$1="-l$ac_lib"
                        ac_cv_found_lib="$ac_lib"
break],
                       [ac_cv_found_lib=""] 
  )
    done
  fi
  LIBS=$ac_func_search_save_LIBS])
  AS_IF([test "$ac_cv_search_$1" != no],
    [test "$ac_cv_search_$1" = "none required" || LIBS="-L$2 $ac_cv_search_$1 $LIBS"
    $4],
        [$5])dnl
]) dnl end of  MAC_SEARCH_LIBS



dnl ==================== 
dnl MAC_SEARCH_MPI_LIBS
dnl ====================

dnl Find name of MPI library. 
dnl Test, whether the name of the MPI library is libmpi.a, 
dnl libmpi.so or libmpich.a.
dnl Determine whether to use signal-based MPI-library on AIX.
dnl IBM supports 2 MPI libraries:
dnl - the old single-thread signal-based MPI-1.2 32-bit version
dnl   uses something like libmpi.a
dnl - the new multi-threaded non-signal-based mpi-2.0 32-and 64-bit version
dnl   uses something like libmpi_r.a
dnl The new version should be used preferrably.
dnl We do not only need the mpi library containing "normal" calls 
dnl (e.g. MPI_Send in libmpich), 
dnl but also the mpi library containing the calls 
dnl for the profiling interface (e.g. PMPI_Send in libpmpich).
dnl Why not use AC_SEARCH_LIBS([MPI_Send], ...)?
AC_DEFUN([MAC_SEARCH_MPI_LIBS], [
lib_mpi=""
case "$host" in
  *-ibm-aix*)                dnl IBM machines
    dnl If we are using signal-based MPI check for traditional MPI-libraries.
    if test "x$SIGNAL" = "xyes" ; then
      if test -f "$mpi_lib_dir/libmpi.a" ; then
        lib_mpi="mpi"
      elif test -f "$mpi_lib_dir/libmpi.so" ; then
        lib_mpi="mpi"
      elif test -f "$mpi_lib_dir/libmpich.a" ; then
        lib_mpi="mpich"
      elif test -f "$mpi_lib_dir/libmpich.so" ; then
        lib_mpi="mpich"
      elif test -f "$mpi_lib_dir/libmpichg2.a" ; then
        lib_mpi="mpichg2"
      elif test -f "$mpi_lib_dir/libmpichg2.so" ; then
        lib_mpi="mpichg2"
      else
        AC_MSG_ERROR([neither libmpi nor libmpich found; \
check path for MPI package first...])
      fi
    else
      if test -f "$mpi_lib_dir/libmpi_r.a" ; then
         lib_mpi="mpi_r"
      else
         AC_MSG_ERROR([libmpi_r not found; check path for MPI package...])
      fi
    fi
  ;;
  *)                         dnl All other machines
    if test -f "$mpi_lib_dir/libmpi.a" ; then
      lib_mpi="mpi"
    elif test -f "$mpi_lib_dir/libmpi.so" ; then
      lib_mpi="mpi"
    elif test -f "$mpi_lib_dir/libmpi.dylib" ; then
      lib_mpi="mpi"
    elif test -f "$mpi_lib_dir/libmpich.a" ; then
      lib_mpi="mpich"
    elif test -f "$mpi_lib_dir/libmpich.so" ; then
      lib_mpi="mpich"
    elif test -f "$mpi_lib_dir/libmpich.dylib" ; then
      lib_mpi="mpich"
    elif test -f "$mpi_lib_dir/libmpichg2.a" ; then
      lib_mpi="mpichg2"
    elif test -f "$mpi_lib_dir/libmpichg2.so" ; then
      lib_mpi="mpichg2"
    else
      AC_MSG_ERROR([neither libmpi nor libmpich found; \
check path for MPI package first...])
    fi
  ;;
esac
AC_MSG_RESULT([found $lib_mpi])


dnl ------------------------
dnl Check if $lib_mpi contains calls for profiling interface (e.g.
dnl PMPI_Send etc.), else find $lib_pmpi.
dnl TODO...don't use AC_CHECK_LIB...
lib_pmpi=""
dnl AC_CHECK_LIB([$lib_mpi], [PMPI_Send], [],
             dnl The same as above, just with "pmpi" instead of "mpi".
dnl             [
              case "$host" in
               *-ibm-aix*)                dnl IBM machines
                 dnl If we are using signal-based MPI, 
                 dnl check for traditional MPI-libraries
                 if test "x$SIGNAL" = "xyes" ; then
                   if test -f "$mpi_lib_dir/libpmpi.a" ; then
                     lib_pmpi="pmpi"
                   elif test -f "$mpi_lib_dir/libpmpi.so" ; then
                     lib_pmpi="pmpi"
                   elif test -f "$mpi_lib_dir/libpmpich.a" ; then
                     lib_pmpi="pmpich"
                   elif test -f "$mpi_lib_dir/libpmpich.so" ; then
                     lib_pmpi="pmpich"
                   elif test -f "$mpi_lib_dir/libpmpichg2.a" ; then
                     lib_pmpi="pmpichg2"
                   elif test -f "$mpi_lib_dir/libpmpichg2.so" ; then
                     lib_pmpi="pmpichg2"
                   else
                     AC_MSG_WARN([neither libpmpi nor libpmpich found.])
                   fi
                 else
                   if test -f "$mpi_lib_dir/libpmpi_r.a" ; then
                     lib_pmpi="pmpi_r"
                   else
                     AC_MSG_ERROR([libpmpi_r not found; \
check path for MPI package...])
                   fi
                 fi
               ;;
             *)                         dnl All other machines
               if test -f "$mpi_lib_dir/libpmpi.a" ; then
                 lib_pmpi="pmpi"
               elif test -f "$mpi_lib_dir/libpmpi.so" ; then
                 lib_pmpi="pmpi"
               elif test -f "$mpi_lib_dir/libpmpi.dylib" ; then
                 lib_pmpi="pmpi"
               elif test -f "$mpi_lib_dir/libpmpich.a" ; then
                 lib_pmpi="pmpich"
               elif test -f "$mpi_lib_dir/libpmpich.so" ; then
                 lib_pmpi="pmpich"
               elif test -f "$mpi_lib_dir/libpmpich.dylib" ; then
                 lib_pmpi="pmpich"
               elif test -f "$mpi_lib_dir/libpmpichg2.a" ; then
                 lib_pmpi="pmpichg2"
               elif test -f "$mpi_lib_dir/libpmpichg2.so" ; then
                 lib_pmpi="pmpichg2"
               else
                 AC_MSG_WARN([neither libpmpi nor libpmpich found.])
               fi
             ;;
           esac
AC_MSG_RESULT([found $lib_pmpi])
dnl]) end of AC_CHECK_LIB

 
dnl ------------------------
dnl How to find out correct order of libraries?
dnl guess: pmpi first
dnl: heuristic: -lpmpi -lmpi -lpmpi
dnl Override MPI_LIBS,
dnl default: the MPI_LIBS found automatically by configure.
if test "x$lib_pmpi" = "x" ; then
  mpi_libs="-l$lib_mpi"
else
  mpi_libs="-l$lib_pmpi -l$lib_mpi -l$lib_pmpi"
fi

]) dnl end of MAC_SEARCH_MPI_LIBS



dnl ==================== 
dnl MAC_SEARCH_MPICXX
dnl ====================

dnl Find name and path of MPI cxx compiler. 
dnl Test, whether the name of the MPI cxx compiler is mpiCC, mpCC or mpCC_r
dnl or whatever.
dnl Determine whether to use signal-based MPI-library on AIX.
dnl IBM supports 2 MPI libraries:
dnl - the old single-thread signal-based MPI-1.2 32-bit version
dnl   uses mpxlf, mpcc, mpCC
dnl - the new multi-threaded non-signal-based mpi-2.0 32-and 64-bit version
dnl   uses mpxlf_r, mpcc_r, mpCC_r
dnl The new version should be used preferrably.
AC_DEFUN([MAC_SEARCH_MPICXX], [
mpicxx=""
case "$host" in
  *-ibm-aix*)                dnl IBM machines
    dnl If we are using signal-based MPI check for traditional MPI compilers.
    if test "x$SIGNAL" = "xyes" ; then
      if test -f "$mpi_bin_dir/mpCC" ; then
        mpicxx="mpCC"
      else
        AC_MSG_WARN([mpCC not found])
      fi
    else
      if test -f "$mpi_bin_dir/mpCC_r" ; then
         mpicxx="mpCC_r"
      else
         AC_MSG_WARN([mpCC_r not found])
      fi
    fi
  ;;
  *-apple-darwin*)
    if test -f "$mpi_bin_dir/mpicxx" ; then
      mpicxx="mpicxx"
    else
      AC_MSG_WARN([mpicxx not found])
    fi
  ;;
  *)                         dnl All other machines
    if test -f "$mpi_bin_dir/mpiCC" ; then
      mpicxx="mpiCC"
    else
      AC_MSG_WARN([mpiCC not found])
    fi
  ;;
esac
AC_MSG_RESULT([found $mpicxx])

]) dnl end of MAC_SEARCH_MPICXX



dnl ==================== 
dnl MAC_SEARCH_MPICC
dnl ====================

dnl Find name and path of MPI c compiler. 
dnl Test, whether the name of the MPI c compiler is mpicc, mpcc or mpcc_r
dnl or whatever.
dnl Determine whether to use signal-based MPI-library on AIX.
dnl IBM supports 2 MPI libraries:
dnl - the old single-thread signal-based MPI-1.2 32-bit version
dnl   uses mpxlf, mpcc, mpCC
dnl - the new multi-threaded non-signal-based mpi-2.0 32-and 64-bit version
dnl   uses mpxlf_r, mpcc_r, mpCC_r
dnl The new version should be used preferrably.
AC_DEFUN([MAC_SEARCH_MPICC], [
mpicc=""
case "$host" in
  *-ibm-aix*)                dnl IBM machines
    dnl If we are using signal-based MPI check for traditional MPI compilers.
    if test "x$SIGNAL" = "xyes" ; then
      if test -f "$mpi_bin_dir/mpcc" ; then
        mpicc="mpcc"
      else
        AC_MSG_WARN([mpcc not found])
      fi
    else
      if test -f "$mpi_bin_dir/mpcc_r" ; then
         mpicc="mpcc_r"
      else
         AC_MSG_WARN([mpcc_r not found])
      fi
    fi
  ;;
  *)                         dnl All other machines
    if test -f "$mpi_bin_dir/mpicc" ; then
      mpicc="mpicc"
    else
      AC_MSG_WARN([mpicc not found])
    fi
  ;;
esac
AC_MSG_RESULT([found $mpicc])

]) dnl end of MAC_SEARCH_MPICC



dnl ==================== 
dnl MAC_SEARCH_MPIF77
dnl ====================

dnl Find name and path of MPI f77 compiler. 
dnl Test, whether the name of the MPI f77 compiler is mpif77, mpxlf or mpxlf_r
dnl or whatever.
dnl Determine whether to use signal-based MPI-library on AIX.
dnl IBM supports 2 MPI libraries:
dnl - the old single-thread signal-based MPI-1.2 32-bit version
dnl   uses mpxlf, mpcc, mpCC
dnl - the new multi-threaded non-signal-based mpi-2.0 32-and 64-bit version
dnl   uses mpxlf_r, mpcc_r, mpCC_r
dnl The new version should be used preferrably.
AC_DEFUN([MAC_SEARCH_MPIF77], [
mpif77=""
case "$host" in
  *-ibm-aix*)                dnl IBM machines
    dnl If we are using signal-based MPI check for traditional MPI compilers.
    if test "x$SIGNAL" = "xyes" ; then
      if test -f "$mpi_bin_dir/mpxlf" ; then
        mpif77="mpxlf"
      else
        AC_MSG_WARN([mpxlf not found])
      fi
    else
      if test -f "$mpi_bin_dir/mpxlf_r" ; then
         mpif77="mpxlf_r"
      else
         AC_MSG_WARN([mpxlf_r not found])
      fi
    fi
  ;;
  *)                         dnl All other machines
    if test -f "$mpi_bin_dir/mpif77" ; then
      mpif77="mpif77"
    else
      AC_MSG_WARN([mpif77 not found])
    fi
  ;;
esac
AC_MSG_RESULT([found $mpif77])

]) dnl end of MAC_SEARCH_MPIF77



dnl ==================== 
dnl MAC_SEARCH_MPIF90
dnl ====================

dnl Find name and path of MPI f90 compiler. 
dnl Test, whether the name of the MPI f90 compiler is mpif90, mpxlf90
dnl or mpxlf90_r or whatever.
dnl Determine whether to use signal-based MPI-library on AIX.
dnl IBM supports 2 MPI libraries:
dnl - the old single-thread signal-based MPI-1.2 32-bit version
dnl   uses mpxlf, mpcc, mpCC
dnl - the new multi-threaded non-signal-based mpi-2.0 32-and 64-bit version
dnl   uses mpxlf_r, mpcc_r, mpCC_r
dnl The new version should be used preferrably.
AC_DEFUN([MAC_SEARCH_MPIF90], [
mpif90=""
case "$host" in
  *-ibm-aix*)                dnl IBM machines
    dnl If we are using signal-based MPI check for traditional MPI compilers.
    if test "x$SIGNAL" = "xyes" ; then
      if test -f "$mpi_bin_dir/mpxlf90" ; then
        mpif90="mpxlf90"
      else
        AC_MSG_WARN([mpxlf90 not found])
      fi
    else
      if test -f "$mpi_bin_dir/mpxlf90_r" ; then
         mpif90="mpxlf90_r"
      else
         AC_MSG_WARN([mpxlf90_r not found])
      fi
    fi
  ;;
  *)                         dnl All other machines
    if test -f "$mpi_bin_dir/mpif90" ; then
      mpif90="mpif90"
    else
      AC_MSG_WARN([mpif90 not found])
    fi
  ;;
esac
AC_MSG_RESULT([found $mpif90])

]) dnl end of MAC_SEARCH_MPIF90



dnl ==================== 
dnl MAC_SEARCH_MPIF95
dnl ====================

dnl Find name and path of MPI f95 compiler. 
dnl Test, whether the name of the MPI f95 compiler is mpif95, mpxlf95
dnl or mpxlf95_r or whatever.
dnl Determine whether to use signal-based MPI-library on AIX.
dnl IBM supports 2 MPI libraries:
dnl - the old single-thread signal-based MPI-1.2 32-bit version
dnl   uses mpxlf, mpcc, mpCC
dnl - the new multi-threaded non-signal-based mpi-2.0 32-and 64-bit version
dnl   uses mpxlf_r, mpcc_r, mpCC_r
dnl The new version should be used preferrably.
AC_DEFUN([MAC_SEARCH_MPIF95], [
mpif95=""
case "$host" in
  *-ibm-aix*)                dnl IBM machines
    dnl If we are using signal-based MPI check for traditional MPI compilers.
    if test "x$SIGNAL" = "xyes" ; then
      if test -f "$mpi_bin_dir/mpxlf95" ; then
        mpif95="mpxlf95"
      else
        AC_MSG_WARN([mpxlf95 not found])
      fi
    else
      if test -f "$mpi_bin_dir/mpxlf95_r" ; then
         mpif95="mpxlf95_r"
      else
         AC_MSG_WARN([mpxlf95_r not found])
      fi
    fi
  ;;
  *)                         dnl All other machines
    if test -f "$mpi_bin_dir/mpif95" ; then
      mpif95="mpif95"
    else
      AC_MSG_WARN([mpif95 not found])
    fi
  ;;
esac
AC_MSG_RESULT([found $mpif95])

]) dnl end of MAC_SEARCH_MPIF95



dnl ===================== 
dnl MAC_CHECK_MPI_COMPILE
dnl =====================

dnl Check compilation of MPI program.
dnl Try with MPI_Finalize and also with PMPI_Finalize.

AC_DEFUN([MAC_CHECK_MPI_COMPILE], [
AC_MSG_CHECKING([for compilation of an MPI program])
old_CFLAGS=$CFLAGS
old_LIBS=$LIBS
CFLAGS="-I$MPI_INCDIR"

dnl Only test for MPI_Finalize; MPI_Init takes argc, argv,
dnl which configure-main doesn't provide
LIBS="-L$MPI_LIBDIR -l$lib_mpi"
AC_COMPILE_IFELSE(
  [AC_LANG_PROGRAM([#include "mpi.h"], 
                   [MPI_Finalize();])], 
  [AC_MSG_RESULT([found])
   AC_DEFINE([HAVE_MPI], [1], 
             [Define if program for MPI could be compiled and linked])],
  [AC_MSG_ERROR([MPI not found; check paths for MPI package first...])])

dnl Only test for PMPI_Finalize; MPI_Init takes argc, argv,
dnl which configure-main doesn't provide
LIBS="-L$MPI_LIBDIR $MPI_LIBS"
AC_MSG_CHECKING([for compilation of an MPI program \
using profiling interface])
AC_COMPILE_IFELSE(
  [AC_LANG_PROGRAM([#include "mpi.h"], 
                   [PMPI_Finalize();])], 
  [AC_MSG_RESULT([found])
   AC_DEFINE([HAVE_PMPI], [1], 
             [Define if program for MPI could be compiled and linked \
using profiling interface])],
  [AC_MSG_ERROR([PMPI not found; check paths for MPI package first...])])

CFLAGS=$old_CFLAGS
LIBS=$old_LIBS
]) dnl end of MAC_CHECK_MPI_COMPILE



dnl ======================== 
dnl MAC_SEARCH_GLOBUS_LIBS
dnl ========================

dnl Set GLOBUS_LIBS.
dnl To run applications with MARMOT and mpich-g2, additional globus libraries
dnl are required.
dnl We don't do any clever things to find them automatically,
dnl they were found by trial and error.
dnl The order of these libraries is important!

AC_DEFUN([MAC_SEARCH_GLOBUS_LIBS], [
GLOBUS_LIBS=""

if test "x$GLOBUS" = "xyes" ; then
  AC_MSG_CHECKING([for globus path])
  AC_MSG_RESULT([$globus_dir])

  AC_MSG_CHECKING([for globus lib path])
  AC_MSG_RESULT([$globus_lib_dir])

  AC_MSG_CHECKING([for globus libs])
  GLOBUS_LIBS=" -L$globus_lib_dir -lglobus_ftp_client_gcc32dbg -lglobus_ftp_control_gcc32dbg -lglobus_duroc_runtime_gcc32dbg -lglobus_duroc_common_gcc32dbg -lglobus_duct_runtime_gcc32dbg -lglobus_nexus_gcc32dbg -lglobus_gram_client_gcc32dbg -lglobus_gram_protocol_gcc32dbg -lglobus_gram_myjob_gcc32dbg -lglobus_io_gcc32dbg -lglobus_gss_assist_gcc32dbg -lglobus_gssapi_gsi_gcc32dbg -lglobus_gsi_callback_gcc32dbg -lglobus_gsi_cert_utils_gcc32dbg -lglobus_gsi_credential_gcc32dbg -lglobus_gsi_proxy_core_gcc32dbg -lglobus_gsi_sysconfig_gcc32dbg -lglobus_proxy_ssl_gcc32dbg -lglobus_openssl_gcc32dbg -lglobus_openssl_error_gcc32dbg -lglobus_oldgaa_gcc32dbg -lglobus_common_gcc32dbg -lglobus_dc_gcc32dbg -lssl_gcc32dbg -lcrypto_gcc32dbg "
  AC_MSG_RESULT([$GLOBUS_LIBS])
fi
])


dnl Found by error and trial in /opt/cg/globus/lib:
dnl  GLOBUS_LIBS=" -L$globus_lib_dir \
dnl          -lglobus_ftp_client_gcc32dbg \
dnl          -lglobus_ftp_control_gcc32dbg \
dnl          -lglobus_duroc_runtime_gcc32dbg \
dnl          -lglobus_duroc_common_gcc32dbg \
dnl          -lglobus_duct_runtime_gcc32dbg \
dnl          -lglobus_nexus_gcc32dbg \
dnl          -lglobus_gram_client_gcc32dbg \
dnl          -lglobus_gram_protocol_gcc32dbg \
dnl          -lglobus_gram_myjob_gcc32dbg \
dnl          -lglobus_io_gcc32dbg \
dnl          -lglobus_gss_assist_gcc32dbg \
dnl          -lglobus_gssapi_gsi_gcc32dbg \
dnl          -lgssapi_error_gcc32dbg \
dnl          -lglobus_gsi_callback_gcc32dbg \
dnl          -lglobus_gsi_cert_utils_gcc32dbg \
dnl          -lglobus_gsi_credential_gcc32dbg \
dnl          -lglobus_gsi_proxy_core_gcc32dbg \
dnl          -lglobus_gsi_sysconfig_gcc32dbg \
dnl          -lglobus_proxy_ssl_gcc32dbg \
dnl          -lglobus_openssl_gcc32dbg \
dnl          -lglobus_openssl_error_gcc32dbg \
dnl          -lglobus_oldgaa_gcc32dbg \
dnl          -lglobus_common_gcc32dbg \
dnl          -lglobus_dc_gcc32dbg \
dnl          -lssl_gcc32dbg \
dnl          -lcrypto_gcc32dbg "

dnl  -lgssapi_error_gcc32dbg does not exist anymore in new globus version...,
dnl  so simply  skip it




dnl ======================== 
dnl MAC_SEARCH_MYRINET_LIBS
dnl ========================

dnl Set MYRINET_LIBS.
dnl Find name of Myrinet library. 
dnl Test, whether the name of the MPI library is libgm.a or libgm.so. 

AC_DEFUN([MAC_SEARCH_MYRINET_LIBS], [
MYRINET_LIBS=""

if test "x$MYRINET" = "xyes" ; then
  AC_MSG_CHECKING([for Myrinet path])
  AC_MSG_RESULT([$myrinet_dir])

  AC_MSG_CHECKING([for Myrinet lib path])
  AC_MSG_RESULT([$myrinet_lib_dir])

  AC_MSG_CHECKING([for Myrinet lib])
  if test -f "$myrinet_lib_dir/libgm.a" ; then
    MYRINET_LIBS="-L$myrinet_lib_dir -lgm"
  elif test -f "$myrinet_lib_dir/libgm.so" ; then
    MYRINET_LIBS="-L$myrinet_lib_dir -lgm"
  else
    AC_MSG_ERROR([libgm not found; check path for Myrinet first...])
  fi
  AC_MSG_RESULT([$MYRINET_LIBS])
fi
])



dnl ======================== 
dnl MAC_SEARCH_PTHREAD_LIBS
dnl ========================

dnl Set PTHREAD_LIBS
dnl Find name of pthread library. 
dnl Test, whether the name of the pthread library is pthread or pthreads. 

AC_DEFUN([MAC_SEARCH_PTHREAD_LIBS], [
PTHREAD_LIBS=""
AC_CHECK_LIB([pthread], [pthread_mutex_trylock],
             [PTHREAD_LIBS=" -lpthread"],
             [AC_CHECK_LIB([pthreads], [pthread_mutex_trylock],
                           [PTHREAD_LIBS=" -lpthreads"],
                           [AC_MSG_WARN([neither libpthread nor \
libpthreads found - for some MPI implementations, this might be required \
to link test programs!])
                           ])
             ]) 
])



dnl ==================== 
dnl MAC_CHECK_PTHREAD
dnl ====================
dnl
dnl Check for threads in general
dnl   Substitutes the pthread_cflags && pthread_ldflags according to system
dnl
dnl I don't fully understand that, ask Rainer first..
dnl Do we have to change CC, F77 and CXX?



dnl ==================== 
dnl MAC_CHECK_MPI_HANDLES
dnl ====================

dnl Dependencies: CC, MPI_INCDIR

AC_DEFUN([MAC_CHECK_MPI_HANDLES], [

   #set the correct compiler
   COMPILER=$CXX
   
   #Find out whether the mpi objects realy are basic datatypes like an 
   #int-id or structs/classes.
   #This is necessary to determine whether we can print them or not.
   # -------- MPI_Comm ----------
   AC_MSG_CHECKING([for type of MPI_Comm])
   cat > confftest.c <<EOF
		#include <mpi.h>
		#include "SRC/INCLUDE/mpo-binaryIO.h"
		#include <fstream>		
		using namespace MPO;		
		int main(int argc, char **argv)
		{
		   MPI_Comm my_comm = MPI_COMM_NULL;
		   std::ofstream out;		
		   out < my_comm;		
		   return 0;
		}
EOF
   $COMPILER $CXXFLAGS -I$MPI_INCDIR -c confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "MPI_Comm is a non basic type (i.e. struct)"
        AC_DEFINE([MPI_COMM_NON_BASIC], 1,
            [Define wheter we can print MPI_Comm or not])
   else
   		echo "MPI_Comm is a basic type"
   		rm confftest.o
   fi
   rm confftest.c
   
   # -------- MPI_Group ----------
   AC_MSG_CHECKING([for type of MPI_Group])
   cat > confftest.c <<EOF
		#include <mpi.h>
		#include "SRC/INCLUDE/mpo-binaryIO.h"
		#include <fstream>		
		using namespace MPO;		
		int main(int argc, char **argv)
		{
		   MPI_Group my_group = MPI_GROUP_NULL;
		   std::ofstream out;		
		   out < my_group;		
		   return 0;
		}
EOF
   $COMPILER $CXXFLAGS -I$MPI_INCDIR -c confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "MPI_Group is a non basic type (i.e. struct)"
        AC_DEFINE([MPI_GROUP_NON_BASIC], 1,
            [Define wheter we can print MPI_Group or not])
   else
   		echo "MPI_Group is a basic type"
   		rm confftest.o
   fi
   rm confftest.c

   # -------- MPI_Datatype ----------
   AC_MSG_CHECKING([for type of MPI_Datatype])
   cat > confftest.c <<EOF
		#include <mpi.h>
		#include "SRC/INCLUDE/mpo-binaryIO.h"
		#include <fstream>		
		using namespace MPO;		
		int main(int argc, char **argv)
		{
		   MPI_Datatype my_datatype = MPI_DATATYPE_NULL;
		   std::ofstream out;		
		   out < my_datatype;		
		   return 0;
		}
EOF
   $COMPILER $CXXFLAGS -I$MPI_INCDIR -c confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "MPI_Datatype is a non basic type (i.e. struct)"
        AC_DEFINE([MPI_DATATYPE_NON_BASIC], 1,
            [Define wheter we can print MPI_Datatype or not])
   else
   		echo "MPI_Datatype is a basic type"
   		rm confftest.o
   fi
   rm confftest.c
   
   # -------- MPI_Op ----------
   AC_MSG_CHECKING([for type of MPI_Op])
   cat > confftest.c <<EOF
		#include <mpi.h>
		#include "SRC/INCLUDE/mpo-binaryIO.h"
		#include <fstream>		
		using namespace MPO;		
		int main(int argc, char **argv)
		{
		   MPI_Op my_op = MPI_OP_NULL;
		   std::ofstream out;		
		   out < my_op;		
		   return 0;
		}
EOF
   $COMPILER $CXXFLAGS -I$MPI_INCDIR -c confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "MPI_Op is a non basic type (i.e. struct)"
        AC_DEFINE([MPI_OP_NON_BASIC], 1,
            [Define wheter we can print MPI_Op or not])
   else
   		echo "MPI_Op is a basic type"
   		rm confftest.o
   fi
   rm confftest.c
   
   # -------- MPI_Errhandler ----------
   AC_MSG_CHECKING([for type of MPI_Errhandler])
   cat > confftest.c <<EOF
		#include <mpi.h>
		#include "SRC/INCLUDE/mpo-binaryIO.h"
		#include <fstream>		
		using namespace MPO;		
		int main(int argc, char **argv)
		{
		   MPI_Errhandler my_errhandler = MPI_ERRHANDLER_NULL;
		   std::ofstream out;		
		   out < my_errhandler;		
		   return 0;
		}
EOF
   $COMPILER $CXXFLAGS -I$MPI_INCDIR -c confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "MPI_Errhandler is a non basic type (i.e. struct)"
        AC_DEFINE([MPI_ERRHANDLER_NON_BASIC], 1,
            [Define wheter we can print MPI_Errhandler or not])
   else
   		echo "MPI_Errhandler is a basic type"
   		rm confftest.o
   fi
   rm confftest.c  
   
   # -------- MPI_Request ----------
   AC_MSG_CHECKING([for type of MPI_Request])
   cat > confftest.c <<EOF
		#include <mpi.h>
		#include "SRC/INCLUDE/mpo-binaryIO.h"
		#include <fstream>		
		using namespace MPO;		
		int main(int argc, char **argv)
		{
		   MPI_Request my_request = MPI_REQUEST_NULL;
		   std::ofstream out;		
		   out < my_request;		
		   return 0;
		}
EOF
   $COMPILER $CXXFLAGS -I$MPI_INCDIR -c confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "MPI_Request is a non basic type (i.e. struct)"
        AC_DEFINE([MPI_REQUEST_NON_BASIC], 1,
            [Define wheter we can print MPI_Request or not])
   else
   		echo "MPI_Request is a basic type"
   		rm confftest.o
   fi
   rm confftest.c   
    ])dnl end of MAC_CHECK_MPI_HANDLES
    
    
dnl ==================== 
dnl MAC_CHECK_HANDLE_CONVERT_FUNCTIONS
dnl ====================

dnl Dependencies: CC, MPI_INCDIR

AC_DEFUN([MAC_CHECK_HANDLE_CONVERT_FUNCTIONS], [

   #set the correct compiler
#   COMPILER=$MPICC
#   if test "$COMPILER" = ""
#   then COMPILER=$CC
#   fi
   COMPILER=$CC

   #Test whether the MPI-II Handle convert functions are present
   # -------- MPI_Comm ----------
   HANDLE_CONVERT="NO"
   AC_MSG_CHECKING([for MPI-II handle conversion functions])
   cat > confftest.c <<EOF
		#include <mpi.h>
		int main(int argc, char **argv)
		{
		   MPI_Comm my_comm = MPI_COMM_NULL;
		   MPI_Fint my_fort_comm = MPI_Comm_c2f(my_comm);
		   return 0;
		}
		
EOF
#TODO: does not include all cases and converts, see autoconf "TryCompile"
   
   $COMPILER $CFLAGS -I$MPI_INCDIR $MPI_LIBS -c -o confftest.o confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "not present"        
        HANDLE_CONVERT="NO"
   else
   		echo "present"
   		AC_DEFINE([MPI_HANLDE_CONVERT_PRESENT], 1,
            [Define whether MPI-II handle convert functions are present])
   		rm confftest.o
	    HANDLE_CONVERT="YES"
   fi
   rm confftest.c
   AC_SUBST([HANDLE_CONVERT])
   
   # we will test for conversion of requests and statuses seperatly
   # -------- MPI_Request_c2f ----------
   REQUEST_CONVERT="NO"
   AC_MSG_CHECKING([for MPI-II MPI_Request conversion functions])
   cat > confftest.c <<EOF
		#include <mpi.h>
		int main(int argc, char **argv)
		{
		   MPI_Request my_request = MPI_REQUEST_NULL;
		   MPI_Fint my_fort_request = MPI_Request_c2f(my_request);
		   MPI_Request my_request_reconvert = MPI_Request_f2c(my_fort_request);
		   return 0;
		}
		
EOF
   $COMPILER $CFLAGS -I$MPI_INCDIR $MPI_LIBS  -c -o confftest.o confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "not present"        
        REQUEST_CONVERT="NO"
   else
   		echo "present"
   		AC_DEFINE([MPI_REQUEST_CONVERT_PRESENT], 1,
            [Define whether MPI-II MPI_Request convert functions are present])
   		rm confftest.o
	    REQUEST_CONVERT="YES"
   fi
   rm confftest.c
   AC_SUBST([REQUEST_CONVERT])
   
   # -------- MPI_Status_c2f ----------
   # we have to check 
   # 1.) for the converter function
   # 2.) for the presence of MPI_F_STATUS_IGNORE, MPI_F_STATUSES_IGNORE
   #     (MPICH provides the converter function but not these two global 
   #	  variables, see MPI-II Standard for details)
   STATUS_CONVERT="NO"
   AC_MSG_CHECKING([for MPI-II MPI_Status conversion functions])
   cat > confftest.c <<EOF
		#include <mpi.h>
		int main(int argc, char **argv)
		{
		   MPI_Status *my_status;
		   MPI_Fint *my_fort_status;
		   MPI_Status_c2f(my_status,my_fort_status);
		   if (my_fort_status == MPI_F_STATUS_IGNORE)
		   	my_fort_status = MPI_F_STATUS_IGNORE;
		   if (my_fort_status == MPI_F_STATUSES_IGNORE)
		   	my_fort_status = MPI_F_STATUSES_IGNORE;	
		   return 0;
		}
		
EOF
   $COMPILER $CFLAGS -I$MPI_INCDIR $MPI_LIBS -c -o confftest.o confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "not present"        
        STATUS_CONVERT="NO"
   else
   		echo "present"
   		AC_DEFINE([MPI_STATUS_CONVERT_PRESENT], 1,
            [Define whether MPI-II MPI_Status convert functions are present])
   		rm confftest.o
	    STATUS_CONVERT="YES"
   fi
   rm confftest.c
   AC_SUBST([STATUS_CONVERT])
   
])dnl end of MAC_CHECK_HANDLE_CONVERT_FUNCTIONS

 


dnl ==================== 
dnl MAC_CHECK_MPI_HANDLES
dnl ====================

dnl Dependencies: CC, MPI_INCDIR

AC_DEFUN([MAC_CHECK_MPI_HANDLES], [

   #set the correct compiler
   COMPILER=$CXX
   
   #Find out what the mpi objects realy are basic datatypes like an 
   #int id or structs/classes.
   #This is necessary to determine whether we can print them or not.
   # -------- MPI_Comm ----------
   AC_MSG_CHECKING([for type of MPI_Comm])
   cat > confftest.c <<EOF
		#include <mpi.h>
		#include "SRC/INCLUDE/mpo-binaryIO.h"
		#include <fstream>		
		using namespace MPO;		
		int main(int argc, char **argv)
		{
		   MPI_Comm my_comm = MPI_COMM_NULL;
		   std::ofstream out;		
		   out < my_comm;		
		   return 0;
		}
EOF
   $COMPILER $CXXFLAGS -I$MPI_INCDIR -c confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "MPI_Comm is a non basic type (i.e. struct)"
        AC_DEFINE([MPI_COMM_NON_BASIC], 1,
            [Define wheter we can print MPI_Comm or not])
   else
   		echo "MPI_Comm is a basic type"
   		rm confftest.o
   fi
   rm confftest.c
   
   # -------- MPI_Group ----------
   AC_MSG_CHECKING([for type of MPI_Group])
   cat > confftest.c <<EOF
		#include <mpi.h>
		#include "SRC/INCLUDE/mpo-binaryIO.h"
		#include <fstream>		
		using namespace MPO;		
		int main(int argc, char **argv)
		{
		   MPI_Group my_group = MPI_GROUP_NULL;
		   std::ofstream out;		
		   out < my_group;		
		   return 0;
		}
EOF
   $COMPILER $CXXFLAGS -I$MPI_INCDIR -c confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "MPI_Group is a non basic type (i.e. struct)"
        AC_DEFINE([MPI_GROUP_NON_BASIC], 1,
            [Define wheter we can print MPI_Group or not])
   else
   		echo "MPI_Group is a basic type"
   		rm confftest.o
   fi
   rm confftest.c

   # -------- MPI_Datatype ----------
   AC_MSG_CHECKING([for type of MPI_Datatype])
   cat > confftest.c <<EOF
		#include <mpi.h>
		#include "SRC/INCLUDE/mpo-binaryIO.h"
		#include <fstream>		
		using namespace MPO;		
		int main(int argc, char **argv)
		{
		   MPI_Datatype my_datatype = MPI_DATATYPE_NULL;
		   std::ofstream out;		
		   out < my_datatype;		
		   return 0;
		}
EOF
   $COMPILER $CXXFLAGS -I$MPI_INCDIR -c confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "MPI_Datatype is a non basic type (i.e. struct)"
        AC_DEFINE([MPI_DATATYPE_NON_BASIC], 1,
            [Define wheter we can print MPI_Datatype or not])
   else
   		echo "MPI_Datatype is a basic type"
   		rm confftest.o
   fi
   rm confftest.c
   
   # -------- MPI_Op ----------
   AC_MSG_CHECKING([for type of MPI_Op])
   cat > confftest.c <<EOF
		#include <mpi.h>
		#include "SRC/INCLUDE/mpo-binaryIO.h"
		#include <fstream>		
		using namespace MPO;		
		int main(int argc, char **argv)
		{
		   MPI_Op my_op = MPI_OP_NULL;
		   std::ofstream out;		
		   out < my_op;		
		   return 0;
		}
EOF
   $COMPILER $CXXFLAGS -I$MPI_INCDIR -c confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "MPI_Op is a non basic type (i.e. struct)"
        AC_DEFINE([MPI_OP_NON_BASIC], 1,
            [Define wheter we can print MPI_Op or not])
   else
   		echo "MPI_Op is a basic type"
   		rm confftest.o
   fi
   rm confftest.c
   
   # -------- MPI_Errhandler ----------
   AC_MSG_CHECKING([for type of MPI_Errhandler])
   cat > confftest.c <<EOF
		#include <mpi.h>
		#include "SRC/INCLUDE/mpo-binaryIO.h"
		#include <fstream>		
		using namespace MPO;		
		int main(int argc, char **argv)
		{
		   MPI_Errhandler my_errhandler = MPI_ERRHANDLER_NULL;
		   std::ofstream out;		
		   out < my_errhandler;		
		   return 0;
		}
EOF
   $COMPILER $CXXFLAGS -I$MPI_INCDIR -c confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "MPI_Errhandler is a non basic type (i.e. struct)"
        AC_DEFINE([MPI_ERRHANDLER_NON_BASIC], 1,
            [Define wheter we can print MPI_Errhandler or not])
   else
   		echo "MPI_Errhandler is a basic type"
   		rm confftest.o
   fi
   rm confftest.c  
   
   # -------- MPI_Request ----------
   AC_MSG_CHECKING([for type of MPI_Request])
   cat > confftest.c <<EOF
		#include <mpi.h>
		#include "SRC/INCLUDE/mpo-binaryIO.h"
		#include <fstream>		
		using namespace MPO;		
		int main(int argc, char **argv)
		{
		   MPI_Request my_request = MPI_REQUEST_NULL;
		   std::ofstream out;		
		   out < my_request;		
		   return 0;
		}
EOF
   $COMPILER $CXXFLAGS -I$MPI_INCDIR -c confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "MPI_Request is a non basic type (i.e. struct)"
        AC_DEFINE([MPI_REQUEST_NON_BASIC], 1,
            [Define wheter we can print MPI_Request or not])
   else
   		echo "MPI_Request is a basic type"
   		rm confftest.o
   fi
   rm confftest.c   
    ])dnl end of MAC_CHECK_MPI_HANDLES
    
dnl ==================== 
dnl MAC_CHECK_MPI_THREAD_SUPPORT
dnl ====================
dnl Dependencies: CC, MPI_INCDIR

AC_DEFUN([MAC_CHECK_MPI_THREAD_SUPPORT], [

   #set the correct compiler
   COMPILER=$CC

   #Test whether the MPI-II thread functions are present
   # -------- MPI_Comm ----------
   MPI_THREAD_FUNCS="NO"
   AC_MSG_CHECKING([for MPI-II thread support functions])
   
   cat > confftest.c <<EOF
		#include <mpi.h>
		int main(int argc, char **argv)
		{
		   int temp;
		   
		   MPI_Init_thread(&argc, &argv, MPI_THREAD_MULTIPLE, &temp);
		   MPI_Is_thread_main(&temp);
		   MPI_Finalize();
		   
		   return 0;
		}
		
EOF
   
   $COMPILER $CCFLAGS -I$MPI_INCDIR $MPI_LIBS -c -o confftest.o confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "not present"        
        MPI_THREAD_FUNCS="NO"
   else
   		echo "present"
   		AC_DEFINE([MARMOT_THREADED], 1,
            [Define whether MPI-II thread support functions are present])
   		rm confftest.o
	    MPI_THREAD_FUNCS="YES"
   fi
   rm confftest.c
   AC_SUBST([MPI_THREAD_FUNCS])
   
]) dnl end of MAC_CHECK_MPI_THREAD_SUPPORT

dnl ==================== 
dnl MAC_TEST_OPENMP_CONFIGURATION
dnl ====================
dnl Dependencies: CC, OPENMP_FLAG, OPENMP_LIB, OPENMP_INCDIR

AC_DEFUN([MAC_TEST_OPENMP_CONFIGURATION], [

   #set the correct compiler
   COMPILER=$CC

   #Test whether the MPI-II thread functions are present
   # -------- MPI_Comm ----------
   MPI_OPENMP_WORKING="NO"
   AC_MSG_CHECKING([for correctnes of OpenMP configuration])
   
   cat > confftest.c <<EOF
		#include <omp.h>
		int main(int argc, char **argv)
		{
		   int temp;
		   
		   omp_set_num_threads(4);
		   #pragma omp parallel private (temp)
    	   {
              temp = omp_get_thread_num();
              temp = omp_get_num_threads();
           }   
		   
		   return 0;
		}
		
EOF
   $COMPILER $CFLAGS $OPENMP_FLAG $OPENMP_INCDIR $OPENMP_LIB -c -o confftest.o confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "not working !!!"        
        MPI_OPENMP_WORKING="NO"
        
        AC_MSG_RESULT([])
        AC_MSG_RESULT([============================================])
        AC_MSG_RESULT([ERROR: you set --enable-openmp but you speciefied an invalid configuration])
        AC_MSG_RESULT([Please check the values of:])
        AC_MSG_RESULT([   --with-openmp-flag])
        AC_MSG_RESULT([   --with-openmp-libs])
        AC_MSG_RESULT([   --with-openmp-lib-dir])
        AC_MSG_RESULT([   --with-openmp-inc-dir])
        AC_MSG_RESULT([============================================])
        AC_MSG_ERROR([])
   else
   		echo "working"
   		AC_DEFINE([MARMOT_OPENMP], 1,
            [Define whether MARMOT may use OpenMP])
   		rm confftest.o
	    MPI_OPENMP_WORKING="YES"
   fi
   rm confftest.c
   AC_SUBST([MPI_OPENMP_WORKING])
   
]) dnl end of MAC_TEST_OPENMP_CONFIGURATION

   
dnl ==================== 
dnl MAC_CHECK_ELEMENTARY_FORTRAN_DATATYPES_IN_MPI_H
dnl ====================

dnl Dependencies: CC, MPI_INCDIR

AC_DEFUN([MAC_CHECK_ELEMENTARY_FORTRAN_DATATYPES_IN_MPI_H], [

   #set the correct compiler
#   COMPILER=$MPICC
#   if test "$COMPILER" = ""
#   then COMPILER=$CC
#   fi
   COMPILER=$CC

   #Test whether MPI_INTEGER is present in mpi.h
   # -------- MPI_INTEGER ----------
   MPI_INTEGER_PRESENT_IN_MPI_H="YES"
   AC_MSG_CHECKING([for MPI_INTEGER in mpi.h])
   cat > confftest.c <<EOF
		#include <mpi.h>
		int main(int argc, char **argv)
		{
                   MPI_Datatype my_type = MPI_INTEGER;
		   return 0;
		}
		
EOF
   
   $COMPILER $CFLAGS -I$MPI_INCDIR $MPI_LIBS -c -o confftest.o confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "not present"        
        MPI_INTEGER_PRESENT_IN_MPI_H="NO"
   else
   		echo "present"
   		AC_DEFINE([MPI_INTEGER_PRESENT_IN_MPI_H], 1,
            [Define whether MPI_INTEGER is present in mpi.h])
   		rm confftest.o
	    MPI_INTEGER_PRESENT_IN_MPI_H="YES"
   fi
   rm confftest.c
   AC_SUBST([MPI_INTEGER_PRESENT_IN_MPI_H])
   
   #Test whether MPI_REAL is present in mpi.h
   # -------- MPI_REAL ----------
   MPI_REAL_PRESENT_IN_MPI_H="YES"
   AC_MSG_CHECKING([for MPI_REAL in mpi.h])
   cat > confftest.c <<EOF
		#include <mpi.h>
		int main(int argc, char **argv)
		{
                   MPI_Datatype my_type = MPI_REAL;
		   return 0;
		}
		
EOF
   
   $COMPILER $CFLAGS -I$MPI_INCDIR $MPI_LIBS -c -o confftest.o confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "not present"        
        MPI_REAL_PRESENT_IN_MPI_H="NO"
   else
   		echo "present"
   		AC_DEFINE([MPI_REAL_PRESENT_IN_MPI_H], 1,
            [Define whether MPI_REAL is present in mpi.h])
   		rm confftest.o
	    MPI_REAL_PRESENT_IN_MPI_H="YES"
   fi
   rm confftest.c
   AC_SUBST([MPI_REAL_PRESENT_IN_MPI_H])

   #Test whether MPI_DOUBLE_PRECISION is present in mpi.h
   # -------- MPI_DOUBLE_PRECISION ----------
   MPI_DOUBLE_PRECISION_PRESENT_IN_MPI_H="YES"
   AC_MSG_CHECKING([for MPI_DOUBLE_PRECISION in mpi.h])
   cat > confftest.c <<EOF
		#include <mpi.h>
		int main(int argc, char **argv)
		{
                   MPI_Datatype my_type = MPI_DOUBLE_PRECISION;
		   return 0;
		}
		
EOF
   
   $COMPILER $CFLAGS -I$MPI_INCDIR $MPI_LIBS -c -o confftest.o confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "not present"        
        MPI_DOUBLE_PRECISION_PRESENT_IN_MPI_H="NO"
   else
   		echo "present"
   		AC_DEFINE([MPI_DOUBLE_PRECISION_PRESENT_IN_MPI_H], 1,
            [Define whether MPI_DOUBLE_PRECISION is present in mpi.h])
   		rm confftest.o
	    MPI_DOUBLE_PRECISION_PRESENT_IN_MPI_H="YES"
   fi
   rm confftest.c
   AC_SUBST([MPI_DOUBLE_PRECISION_PRESENT_IN_MPI_H])

   #Test whether MPI_COMPLEX is present in mpi.h
   # -------- MPI_COMPLEX ----------
   MPI_COMPLEX_PRESENT_IN_MPI_H="YES"
   AC_MSG_CHECKING([for MPI_COMPLEX in mpi.h])
   cat > confftest.c <<EOF
		#include <mpi.h>
		int main(int argc, char **argv)
		{
                   MPI_Datatype my_type = MPI_COMPLEX;
		   return 0;
		}
		
EOF
   
   $COMPILER $CFLAGS -I$MPI_INCDIR $MPI_LIBS -c -o confftest.o confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "not present"        
        MPI_COMPLEX_PRESENT_IN_MPI_H="NO"
   else
   		echo "present"
   		AC_DEFINE([MPI_COMPLEX_PRESENT_IN_MPI_H], 1,
            [Define whether MPI_COMPLEX is present in mpi.h])
   		rm confftest.o
	    MPI_COMPLEX_PRESENT_IN_MPI_H="YES"
   fi
   rm confftest.c
   AC_SUBST([MPI_COMPLEX_PRESENT_IN_MPI_H])

   #Test whether MPI_LOGICAL is present in mpi.h
   # -------- MPI_LOGICAL ----------
   MPI_LOGICAL_PRESENT_IN_MPI_H="YES"
   AC_MSG_CHECKING([for MPI_LOGICAL in mpi.h])
   cat > confftest.c <<EOF
		#include <mpi.h>
		int main(int argc, char **argv)
		{
                   MPI_Datatype my_type = MPI_LOGICAL;
		   return 0;
		}
		
EOF
   
   $COMPILER $CFLAGS -I$MPI_INCDIR $MPI_LIBS -c -o confftest.o confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "not present"        
        MPI_LOGICAL_PRESENT_IN_MPI_H="NO"
   else
   		echo "present"
   		AC_DEFINE([MPI_LOGICAL_PRESENT_IN_MPI_H], 1,
            [Define whether MPI_LOGICAL is present in mpi.h])
   		rm confftest.o
	    MPI_LOGICAL_PRESENT_IN_MPI_H="YES"
   fi
   rm confftest.c
   AC_SUBST([MPI_LOGICAL_PRESENT_IN_MPI_H])

   #Test whether MPI_CHARACTER is present in mpi.h
   # -------- MPI_CHARACTER ----------
   MPI_CHARACTER_PRESENT_IN_MPI_H="YES"
   AC_MSG_CHECKING([for MPI_CHARACTER in mpi.h])
   cat > confftest.c <<EOF
		#include <mpi.h>
		int main(int argc, char **argv)
		{
                   MPI_Datatype my_type = MPI_CHARACTER;
		   return 0;
		}
		
EOF
   
   $COMPILER $CFLAGS -I$MPI_INCDIR $MPI_LIBS -c -o confftest.o confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "not present"        
        MPI_CHARACTER_PRESENT_IN_MPI_H="NO"
   else
   		echo "present"
   		AC_DEFINE([MPI_CHARACTER_PRESENT_IN_MPI_H], 1,
            [Define whether MPI_CHARACTER is present in mpi.h])
   		rm confftest.o
	    MPI_CHARACTER_PRESENT_IN_MPI_H="YES"
   fi
   rm confftest.c
   AC_SUBST([MPI_CHARACTER_PRESENT_IN_MPI_H])
   
])dnl end of MAC_CHECK_ELEMENTARY_FORTRAN_DATATYPES_IN_MPI_H
   



dnl ==================== 
dnl MAC_CHECK_REDUCTION_FORTRAN_DATATYPES_IN_MPI_H
dnl ====================

dnl Dependencies: CC, MPI_INCDIR

AC_DEFUN([MAC_CHECK_REDUCTION_FORTRAN_DATATYPES_IN_MPI_H], [

   #set the correct compiler
#   COMPILER=$MPICC
#   if test "$COMPILER" = ""
#   then COMPILER=$CC
#   fi
   COMPILER=$CC

   #Test whether MPI_2REAL is present in mpi.h
   # -------- MPI_2REAL ----------
   MPI_2REAL_PRESENT_IN_MPI_H="YES"
   AC_MSG_CHECKING([for MPI_2REAL in mpi.h])
   cat > confftest.c <<EOF
		#include <mpi.h>
		int main(int argc, char **argv)
		{
                   MPI_Datatype my_type = MPI_2REAL;
		   return 0;
		}
		
EOF
   
   $COMPILER $CFLAGS -I$MPI_INCDIR $MPI_LIBS -c -o confftest.o confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "not present"        
        MPI_2REAL_PRESENT_IN_MPI_H="NO"
   else
   		echo "present"
   		AC_DEFINE([MPI_2REAL_PRESENT_IN_MPI_H], 1,
            [Define whether MPI_2REAL is present in mpi.h])
   		rm confftest.o
	    MPI_2REAL_PRESENT_IN_MPI_H="YES"
   fi
   rm confftest.c
   AC_SUBST([MPI_2REAL_PRESENT_IN_MPI_H])
   
   #Test whether MPI_2DOUBLE_PRECISION is present in mpi.h
   # -------- MPI_2DOUBLE_PRECISION ----------
   MPI_2DOUBLE_PRECISION_PRESENT_IN_MPI_H="YES"
   AC_MSG_CHECKING([for MPI_2DOUBLE_PRECISION in mpi.h])
   cat > confftest.c <<EOF
		#include <mpi.h>
		int main(int argc, char **argv)
		{
                   MPI_Datatype my_type = MPI_2DOUBLE_PRECISION;
		   return 0;
		}
		
EOF
   
   $COMPILER $CFLAGS -I$MPI_INCDIR $MPI_LIBS -c -o confftest.o confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "not present"        
        MPI_2DOUBLE_PRECISION_PRESENT_IN_MPI_H="NO"
   else
   		echo "present"
   		AC_DEFINE([MPI_2DOUBLE_PRECISION_PRESENT_IN_MPI_H], 1,
            [Define whether MPI_2DOUBLE_PRECISION is present in mpi.h])
   		rm confftest.o
	    MPI_2DOUBLE_PRECISION_PRESENT_IN_MPI_H="YES"
   fi
   rm confftest.c
   AC_SUBST([MPI_2DOUBLE_PRECISION_PRESENT_IN_MPI_H])

   #Test whether MPI_2INTEGER is present in mpi.h
   # -------- MPI_2INTEGER ----------
   MPI_2INTEGER_PRESENT_IN_MPI_H="YES"
   AC_MSG_CHECKING([for MPI_2INTEGER in mpi.h])
   cat > confftest.c <<EOF
		#include <mpi.h>
		int main(int argc, char **argv)
		{
                   MPI_Datatype my_type = MPI_2INTEGER;
		   return 0;
		}
		
EOF
   
   $COMPILER $CFLAGS -I$MPI_INCDIR $MPI_LIBS -c -o confftest.o confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "not present"        
        MPI_2INTEGER_PRESENT_IN_MPI_H="NO"
   else
   		echo "present"
   		AC_DEFINE([MPI_2INTEGER_PRESENT_IN_MPI_H], 1,
            [Define whether MPI_2INTEGER is present in mpi.h])
   		rm confftest.o
	    MPI_2INTEGER_PRESENT_IN_MPI_H="YES"
   fi
   rm confftest.c
   AC_SUBST([MPI_2INTEGER_PRESENT_IN_MPI_H])
      
])dnl end of MAC_CHECK_REDUCTION_FORTRAN_DATATYPES_IN_MPI_H
   



dnl ==================== 
dnl MAC_CHECK_OPTIONAL_C_DATATYPES_IN_MPI_H
dnl ====================

dnl Dependencies: CC, MPI_INCDIR

AC_DEFUN([MAC_CHECK_OPTIONAL_C_DATATYPES_IN_MPI_H], [

   #set the correct compiler
#   COMPILER=$MPICC
#   if test "$COMPILER" = ""
#   then COMPILER=$CC
#   fi
   COMPILER=$CC

   #Test whether MPI_LONG_LONG_INT is present in mpi.h
   # -------- MPI_LONG_LONG_INT ----------
   MPI_LONG_LONG_INT_PRESENT_IN_MPI_H="YES"
   AC_MSG_CHECKING([for MPI_LONG_LONG_INT in mpi.h])
   cat > confftest.c <<EOF
		#include <mpi.h>
		int main(int argc, char **argv)
		{
                   MPI_Datatype my_type = MPI_LONG_LONG_INT;
		   return 0;
		}
		
EOF
   
   $COMPILER $CFLAGS -I$MPI_INCDIR $MPI_LIBS -c -o confftest.o confftest.c > /dev/null 2>&1
   if test ! -s confftest.o ; then
        echo "not present"        
        MPI_LONG_LONG_INT_PRESENT_IN_MPI_H="NO"
   else
   		echo "present"
   		AC_DEFINE([MPI_LONG_LONG_INT_PRESENT_IN_MPI_H], 1,
            [Define whether MPI_LONG_LONG_INT is present in mpi.h])
   		rm confftest.o
	    MPI_LONG_LONG_INT_PRESENT_IN_MPI_H="YES"
   fi
   rm confftest.c
   AC_SUBST([MPI_LONG_LONG_INT_PRESENT_IN_MPI_H])

])dnl end of MAC_CHECK_OPTIONAL_C_DATATYPES_IN_MPI_H  
