#!/bin/bash

#Marmot installation script for cacau (cacau.hww.de)
#12/06/2007

# documentation disabled (no pdflatex)
# tests disabled (linker errors occur with almost every combination of compiler/MPI library)

#results with different compiler/MPI combinations (13/06/07):
# 
#  					build LIB		make in TEST_C		deadlock1.c			deadlock1.c w/marmot
#														(manual)			(manual)
#-----------------|-------------|-------------------|-------------------|-------------------------
#				
#	gcc-nec		  		OK      		FAIL				OK					OK
#
#	intel-nec			OK				FAIL				OK					OK
#
#	gcc-lam				FAIL			-					-					-
#
#	intel-lam			FAIL			-					-					-
#
#	gcc-openmpi			OK				FAIL				OK					FAIL
#
#	intel-openmpi		OK				OK					(OK)				(OK)
#
#	gcc-voltaire		OK				FAIL				OK					FAIL
#
#	intel-voltaire		OK				FAIL				OK					FAIL
#			  

function build_gcc-nec
{
	#We use the GCC Compiler (default)
	#MPI Library
	#module switch mpi mpi/NECmpi
	module list 
	#as GCC is the default compiler, we can simply unset the environment variables
	unset CXX
	unset CC
	unset CPP
	unset F77
	#configure marmot
	./configure --with-mpi-dir=/opt/NECmpi --disable-tests --disable-doc --prefix=$INSTALL_DIR CXXFLAGS="-g -Wall " CFLAGS="-pedantic -g -Wall " FFLAGS="-g -Wall "  
	make 2>&1 |tee make_${DATE}.log
	#we cannot make the examples in TEST_C because of linker errors...
	#( cd TEST_C ; make 2>&1|tee make_${DATE}.log )
	make install
}

function build_gcc-lam
{
	#We use the GCC Compiler (default)
	#MPI Library
	#module switch mpi mpi/lam-7.1.1
	module list 
	#as GCC is the default compiler, we can simply unset the environment variables
	unset CXX
	unset CC
	unset CPP
	unset F77
	#configure marmot
	./configure --with-mpi-dir=/opt/lam-7.1.1-gcc --disable-doc --prefix=$INSTALL_DIR CXXFLAGS="-g -Wall " CFLAGS="-pedantic -g -Wall " FFLAGS="-g -Wall "  
	make 2>&1 |tee make_${DATE}.log
	( cd TEST_C ; make 2>&1|tee make_${DATE}.log )
	make install
}

function build_gcc-openmpi
{
	#We use the GCC Compiler (default)
	#MPI Library
	#module switch mpi mpi/openmpi-1.2.2
	module list 
	#as GCC is the default compiler, we can simply unset the environment variables
	unset CXX
	unset CC
	unset CPP
	unset F77
	#configure marmot
	./configure --with-mpi-dir=/opt/OpenMPI/1.2.2 --disable-tests --disable-doc --prefix=$INSTALL_DIR CXXFLAGS="-g -Wall -I/opt/OpenMPI/1.2.2/include/openmpi" CFLAGS="-pedantic -g -Wall " FFLAGS="-g -Wall "  
	make 2>&1 |tee make_${DATE}.log
	#we cannot make the examples in TEST_C because of linker errors...
	#( cd TEST_C ; make 2>&1|tee make_${DATE}.log )
	make install
}

function build_gcc-voltaire
{
	#We use the GCC Compiler (default)
	#MPI Library
	#module switch mpi mpi/voltaire_gcc
	module list 
	#as GCC is the default compiler, we can simply unset the environment variables
	unset CXX
	unset CC
	unset CPP
	unset F77
	#configure marmot
	./configure --with-mpi-dir=/usr/voltaire/mpi.lowmemory.gcc.rsh --disable-tests --disable-doc --prefix=$INSTALL_DIR CXXFLAGS="-g -Wall" CFLAGS="-pedantic -g -Wall " FFLAGS="-g -Wall "  
	make 2>&1 |tee make_${DATE}.log
	#we cannot make the examples in TEST_C because of linker errors...
	#( cd TEST_C ; make 2>&1|tee make_${DATE}.log )
	make install
}

function build_intel-nec
{
	#We use the Intel Compiler
	#MPI Library
	#module switch mpi/mpi/NECmpi
	module list
	./configure --with-mpi-dir=/opt/NECmpi --disable-tests --disable-doc --prefix=$INSTALL_DIR CXXFLAGS="-g -Wall " CFLAGS="-g -Wall " FFLAGS="-g -Wall " CXX="icpc" CC="icc" CPP="icc -E" F77="ifort"
	make 2>&1 |tee make_${DATE}.log
	#we cannot make the examples in TEST_C because of linker errors...
	#( cd TEST_C ; make 2>&1|tee make_${DATE}.log )
	make install
}

function build_intel-lam
{
	#We use the Intel Compiler
	#MPI Library
	#module switch mpi/mpi/lam-7.1.1-icc
	module list
	./configure --with-mpi-dir=/opt/lam-7.1.1-icc --disable-tests --disable-doc --prefix=$INSTALL_DIR CXXFLAGS="-g -Wall " CFLAGS="-g -Wall " FFLAGS="-g -Wall " CXX="icpc" CC="icc" CPP="icc -E" F77="ifort"
	make 2>&1 |tee make_${DATE}.log
	( cd TEST_C ; make 2>&1|tee make_${DATE}.log )
	make install
}

function build_intel-openmpi
{
	#We use the Intel Compiler
	#MPI Library
	#module switch mpi mpi/openmpi-1.2.2
	module list
	./configure --with-mpi-dir=/opt/OpenMPI/1.2.2 --disable-tests --disable-doc --prefix=$INSTALL_DIR CXXFLAGS="-g -Wall -I/opt/OpenMPI/1.2.2/include/openmpi" CFLAGS="-g -Wall " FFLAGS="-g -Wall " CXX="icpc" CC="icc" CPP="icc -E" F77="ifort"
	make 2>&1 |tee make_${DATE}.log
	( cd TEST_C ; make 2>&1|tee make_${DATE}.log )
	make install
}

function build_intel-voltaire
{
	#We use the Intel Compiler
	#MPI Library
	#module switch mpi mpi/voltaire_icc
	module list
	./configure --with-mpi-dir=/usr/voltaire/mpi.lowmemory.icc.rsh --disable-doc --prefix=$INSTALL_DIR CXXFLAGS="-g -Wall -I/opt/OpenMPI/1.2.2/include/openmpi" CFLAGS="-g -Wall " FFLAGS="-g -Wall " CXX="icpc" CC="icc" CPP="icc -E" F77="ifort"
	make 2>&1 |tee make_${DATE}.log
	#( cd TEST_C ; make 2>&1|tee make_${DATE}.log )
	make install
}

function prepare
{
	set -x
	make clean
	aclocal -I config
	autoheader
	automake 
	autoconf -Wall
	DATE=`date +%Y%m%d_%H%M`
	INSTALL_DIR=$HOME/marmot_build_${1}_$DATE
	echo "Marmot will be installed in $INSTALL_DIR"
	case "$1" in 
		gcc-nec)
			build_gcc-nec
			;;
		gcc-lam)
			build_gcc-lam
			;;
		gcc-openmpi)
			build_gcc-openmpi
			;;
		gcc-voltaire)
			build_gcc-voltaire
			;;
		intel-nec)
			build_intel-nec
			;;
		intel-lam)
			build_intel-lam
			;;
		intel-openmpi)
			build_intel-openmpi
			;;
		intel-voltaire)
			build_intel-voltaire
			;;
		
		*)
			echo "todo..."
			exit
		;;
	esac
}





#compiler selecction
case "$1" in

	intel)

		echo "Building marmot with Intel Compiler"
		COMPILER=intel
		;;

	gcc)

		echo "Building marmot with GCC Compiler"
		COMPILER=gcc
		;;

	*)
		echo "Usage: $0 {intel|gcc} {openmpi|lam|voltaire|NECmpi}"
		exit
		;;

esac

#MPI selection
case "$2" in

	openmpi)
		
		echo "Building with openmpi-1.2.2"
		MPI=openmpi
		;;
		
	lam)
	
		echo "Building with lam-7.1.1"
		MPI=lam
		;;
		
	voltaire)
		
		echo "Building with voltaire"
		MPI=voltaire
		;;
		
	NECmpi)
	
		echo "Building with NECmpi"
		MPI=nec
		;;
		
	*)
		echo "Usage: $0 {intel|gcc} {openmpi|lam|voltaire|NECmpi}"
		exit
		;;
	
esac

CONF=${COMPILER}-${MPI}
echo "Using configuration $CONF"
prepare $CONF

###############################################################################
#						Building TEST_C/deadlock1.c							  #	
###############################################################################
#
#GCC 3.4.3
#NECmpi
#
#
#$mpicc -show deadlock1.c
#gcc deadlock1.c -c -I/opt/NECmpi/include
#gcc deadlock1.o -L/opt/NECmpi/lib -lmpi -lvapi -lmosal -lmtl_common -lmpga -lm
#
#$/cacau/HLRS/hlrs/hpchimml/marmot_build_gcc-nec_20070618_1516/bin/marmotcc -show deadlock1.c
#
#mpicc -show deadlock1.c -L/cacau/HLRS/hlrs/hpchimml/marmot_build_gcc-nec_20070618_1516/lib -lmarmot-profile -lmarmot-core -L/opt/NECmpi/lib -lpmpi -lmpi -lpmpi -lpthread -L/usr/lib -lstdc++ -I/cacau/HLRS/hlrs/hpchimml/marmot_build_gcc-nec_20070618_1516/include
#
#gcc deadlock1.c -L/cacau/HLRS/hlrs/hpchimml/marmot_build_gcc-nec_20070618_1516/lib -L/opt/NECmpi/lib -L/usr/lib -I/cacau/HLRS/hlrs/hpchimml/marmot_build_gcc-nec_20070618_1516/include -c -I/opt/NECmpi/include
#gcc deadlock1.o -L/cacau/HLRS/hlrs/hpchimml/marmot_build_gcc-nec_20070618_1516/lib -lmarmot-profile -lmarmot-core -L/opt/NECmpi/lib -lpmpi -lmpi -lpmpi -lpthread -L/usr/lib -lstdc++ -I/cacau/HLRS/hlrs/hpchimml/marmot_build_gcc-nec_20070618_1516/include -L/opt/NECmpi/lib -lmpi -lvapi -lmosal -lmtl_common -lmpga -lm
#
#
#$mpicc -o deadlock1_normal deadlock1.c -- OK
#
#$/cacau/HLRS/hlrs/hpchimml/marmot_build_gcc-nec_20070618_1516/bin/marmotcc -o deadlock_marmot deadlock1.c
#
#mpicc -o deadlock_marmot deadlock1.c -L/cacau/HLRS/hlrs/hpchimml/marmot_build_gcc-nec_20070618_1516/lib -lmarmot-profile -lmarmot-core -L/opt/NECmpi/lib -lpmpi -lmpi -lpmpi -lpthread -L/usr/lib -lstdc++ -I/cacau/HLRS/hlrs/hpchimml/marmot_build_gcc-nec_20070618_1516/include
#
#/usr/bin/ld: skipping incompatible /usr/lib/libpthread.so when searching for -lpthread
#/usr/bin/ld: skipping incompatible /usr/lib/libpthread.a when searching for -lpthread
#/usr/bin/ld: skipping incompatible /usr/lib/libm.so when searching for -lm
#/usr/bin/ld: skipping incompatible /usr/lib/libm.a when searching for -lm
#/usr/bin/ld: skipping incompatible /usr/lib/libc.so when searching for -lc
#/usr/bin/ld: skipping incompatible /usr/lib/libc.a when searching for -lc
#/cacau/HLRS/hlrs/hpchimml/marmot_build_gcc-nec_20070618_1516/lib/libmarmot-core.a(mpo_fortran_datatypehelpers.o)(.data+0x0): undefined reference to `mpi_win_dup_fn__'
#/cacau/HLRS/hlrs/hpchimml/marmot_build_gcc-nec_20070618_1516/lib/libmarmot-core.a(mpo_fortran_datatypehelpers.o)(.data+0x8): undefined reference to `mpi_win_null_delete_fn__'
#(...)
#
#
#$mpirun -np 2 ./deadlock1_normal
#Warning: No xauth data; using fake authentication data for X11 forwarding.
#Warning: No xauth data; using fake authentication data for X11 forwarding.
#Warning: No xauth data; using fake authentication data for X11 forwarding.
# I am rank 0 of 2 PEs
#  I am rank 1 of 2 PEs
#  noco022.nec: mpid: MPI program execution aborted
#  Killed by signal 15.
#  Killed by signal 15.
#  Killed by signal 15.
#  noco022.nec: mpid: MPI process terminated by exit(255)
#  noco022.nec: mpid: MPI process terminated by exit(255)
#  noco022.nec: mpid: MPI process terminated by exit(255)
#
#  OK
#
#
###############################################################################
#
#GCC 3.4.3
#OpenMPI 1.2.2
#
#
#$mpicc -show deadlock1.c
#mpicc: error while loading shared libraries: libsvml.so: cannot open shared object file: No such file or directory
#
#$/cacau/HLRS/hlrs/hpchimml/marmot_build_gcc-openmpi_20070618_1433/bin/marmotcc -show deadlock1.c
#
#mpicc -show deadlock1.c -L/cacau/HLRS/hlrs/hpchimml/marmot_build_gcc-openmpi_20070618_1433/lib -lmarmot-profile -lmarmot-core -L/opt/OpenMPI/1.2.2/lib -lmpi -lpthread -L/usr/lib -lstdc++ -I/cacau/HLRS/hlrs/hpchimml/marmot_build_gcc-openmpi_20070618_1433/include
#
#mpicc: error while loading shared libraries: libsvml.so: cannot open shared object file: No such file or directory
#
#$gcc -o deadlock1_normal deadlock1.c -I/opt/OpenMPI/1.2.2/include/ -L/opt/OpenMPI/1.2.2/lib/ -lmpi
#/usr/bin/ld: warning: libsvml.so, needed by /opt/OpenMPI/1.2.2/lib//libmpi.so, not found (try using -rpath or -rpath-link)
#/usr/bin/ld: warning: libimf.so, needed by /opt/OpenMPI/1.2.2/lib//libmpi.so, not found (try using -rpath or -rpath-link)
#/usr/bin/ld: warning: libirc.so, needed by /opt/OpenMPI/1.2.2/lib//libmpi.so, not found (try using -rpath or -rpath-link)
#/opt/OpenMPI/1.2.2/lib//libmpi.so: undefined reference to `_intel_fast_memcpy'
#/opt/OpenMPI/1.2.2/lib//libmpi.so: undefined reference to `_intel_fast_memset'
#/opt/OpenMPI/1.2.2/lib//libmpi.so: undefined reference to `_intel_fast_memcmp'
#collect2: ld returned 1 exit status
#
#$gcc -o deadlock1_normal deadlock1.c -I/opt/OpenMPI/1.2.2/include/ -L/opt/OpenMPI/1.2.2/lib/ -lmpi -L/opt/intel/compiler/9.0/lib/ -lsvml -limf -lirc -- OK
#
#$mpirun -np 2 ./deadlock1_normal
# I am rank 0 of 2 PEs
#  I am rank 1 of 2 PEs
# mpirun: killing job...
#
#  mpirun noticed that job rank 0 with PID 9778 on node noco157.nec exited on signal 42 (Real-time signal 8).
#
#  OK
#
#
###############################################################################
#
#
#GCC 3.4.3
#OpenMPI 1.2.2
#
#
#$mpicc -show deadlock1.c
#mpicc: error while loading shared libraries: libsvml.so: cannot open shared object file: No such file or directory
#
#$/cacau/HLRS/hlrs/hpchimml/marmot_build_gcc-openmpi_20070618_1433/bin/marmotcc -show deadlock1.c
#
#mpicc -show deadlock1.c -L/cacau/HLRS/hlrs/hpchimml/marmot_build_gcc-openmpi_20070618_1433/lib -lmarmot-profile -lmarmot-core -L/opt/OpenMPI/1.2.2/lib -lmpi -lpthread -L/usr/lib -lstdc++ -I/cacau/HLRS/hlrs/hpchimml/marmot_build_gcc-openmpi_20070618_1433/include
#
#mpicc: error while loading shared libraries: libsvml.so: cannot open shared object file: No such file or directory
#
#$gcc -o deadlock1_normal deadlock1.c -I/opt/OpenMPI/1.2.2/include/ -L/opt/OpenMPI/1.2.2/lib/ -lmpi
#/usr/bin/ld: warning: libsvml.so, needed by /opt/OpenMPI/1.2.2/lib//libmpi.so, not found (try using -rpath or -rpath-link)
#/usr/bin/ld: warning: libimf.so, needed by /opt/OpenMPI/1.2.2/lib//libmpi.so, not found (try using -rpath or -rpath-link)
#/usr/bin/ld: warning: libirc.so, needed by /opt/OpenMPI/1.2.2/lib//libmpi.so, not found (try using -rpath or -rpath-link)
#/opt/OpenMPI/1.2.2/lib//libmpi.so: undefined reference to `_intel_fast_memcpy'
#/opt/OpenMPI/1.2.2/lib//libmpi.so: undefined reference to `_intel_fast_memset'
#/opt/OpenMPI/1.2.2/lib//libmpi.so: undefined reference to `_intel_fast_memcmp'
#collect2: ld returned 1 exit status
#
#$gcc -o deadlock1_normal deadlock1.c -I/opt/OpenMPI/1.2.2/include/ -L/opt/OpenMPI/1.2.2/lib/ -lmpi -L/opt/intel/compiler/9.0/lib/ -lsvml -limf -lirc -- OK
#
#$mpirun -np 2 ./deadlock1_normal
# I am rank 0 of 2 PEs
#  I am rank 1 of 2 PEs
#  mpirun: killing job...
#
#  mpirun noticed that job rank 0 with PID 9778 on node noco157.nec exited on signal 42 (Real-time signal 8).
#
#  OK
#
###############################################################################
#
#
#Intel 9.0
#NECmpi
#
#
#$mpicc -show deadlock1.c
#gcc deadlock1.c -c -I/opt/NECmpi/include
#gcc deadlock1.o -L/opt/NECmpi/lib -lmpi -lvapi -lmosal -lmtl_common -lmpga -lm
#
#$/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-nec_20070618_1405/bin/marmotcc --show deadlock1.c
#
#mpicc --show deadlock1.c -L/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-nec_20070618_1405/lib -lmarmot-profile -lmarmot-core -L/opt/NECmpi/lib -lpmpi -lmpi -lpmpi -lpthread -L/usr/lib -lstdc++ -I/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-nec_20070618_1405/include
#
#cc1: error: unrecognized command line option "-fshow"
#
#$mpicc -o deadlock1_normal deadlock1.c -- OK
#
#$/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-nec_20070618_1405/bin/marmotcc -o deadlock1_marmot deadlock1.c
#
#mpicc -o deadlock1_marmot deadlock1.c -L/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-nec_20070618_1405/lib -lmarmot-profile -lmarmot-core -L/opt/NECmpi/lib -lpmpi -lmpi -lpmpi -lpthread -L/usr/lib -lstdc++ -I/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-nec_20070618_1405/include
#
#/usr/bin/ld: skipping incompatible /usr/lib/libpthread.so when searching for -lpthread
#/usr/bin/ld: skipping incompatible /usr/lib/libpthread.a when searching for -lpthread
#/usr/bin/ld: skipping incompatible /usr/lib/libm.so when searching for -lm
#/usr/bin/ld: skipping incompatible /usr/lib/libm.a when searching for -lm
#/usr/bin/ld: skipping incompatible /usr/lib/libc.so when searching for -lc
#/usr/bin/ld: skipping incompatible /usr/lib/libc.a when searching for -lc -- OK
#
#$mpirun -np 2 ./deadlock1_normal
#Warning: No xauth data; using fake authentication data for X11 forwarding.
#Warning: No xauth data; using fake authentication data for X11 forwarding.
#Warning: No xauth data; using fake authentication data for X11 forwarding.
#/usr/X11R6/bin/xauth:  error in locking authority file /cacau/HLRS/hlrs/hpchimml/.Xauthority
#/usr/X11R6/bin/xauth:  error in locking authority file /cacau/HLRS/hlrs/hpchimml/.Xauthority
# I am rank 1 of 2 PEs
#  I am rank 0 of 2 PEs
#  noco157.nec: mpid: MPI program execution aborted
#  Killed by signal 15.
#  Killed by signal 15.
#  Killed by signal 15.
#  noco157.nec: mpid: MPI process terminated by exit(255)
#  noco157.nec: mpid: MPI process terminated by exit(255)
#  noco157.nec: mpid: MPI process terminated by exit(255)
#
#  OK
#
#$mpirun -np 3 ./deadlock1_marmot
#Warning: No xauth data; using fake authentication data for X11 forwarding.
#Warning: No xauth data; using fake authentication data for X11 forwarding.
#Warning: No xauth data; using fake authentication data for X11 forwarding.
#/usr/X11R6/bin/xauth:  error in locking authority file /cacau/HLRS/hlrs/hpchimml/.Xauthority
#/usr/X11R6/bin/xauth:  error in locking authority file /cacau/HLRS/hlrs/hpchimml/.Xauthority
# I am rank 0 of 2 PEs
#  I am rank 1 of 2 PEs
#  WARNING: all clients are pending! (Details see the LogFile)
#  noco157.nec: mpid: MPI program execution aborted
#  Killed by signal 15.
#  Killed by signal 15.
#  Killed by signal 15.
#  noco157.nec: mpid: MPI process terminated by exit(255)
#  noco157.nec: mpid: MPI process terminated by exit(255)
#  noco157.nec: mpid: MPI process terminated by exit(255)
#
#  OK
#
###############################################################################
#
#
#Intel 9.0
#NECmpi
#
#
#$mpicc -show deadlock1.c
#gcc deadlock1.c -c -I/opt/NECmpi/include
#gcc deadlock1.o -L/opt/NECmpi/lib -lmpi -lvapi -lmosal -lmtl_common -lmpga -lm
#
#$/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-nec_20070618_1405/bin/marmotcc --show deadlock1.c
#
#mpicc --show deadlock1.c -L/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-nec_20070618_1405/lib -lmarmot-profile -lmarmot-core -L/opt/NECmpi/lib -lpmpi -lmpi -lpmpi -lpthread -L/usr/lib -lstdc++ -I/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-nec_20070618_1405/include
#
#cc1: error: unrecognized command line option "-fshow"
#
#$mpicc -o deadlock1_normal deadlock1.c -- OK
#
#$/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-nec_20070618_1405/bin/marmotcc -o deadlock1_marmot deadlock1.c
#
#mpicc -o deadlock1_marmot deadlock1.c -L/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-nec_20070618_1405/lib -lmarmot-profile -lmarmot-core -L/opt/NECmpi/lib -lpmpi -lmpi -lpmpi -lpthread -L/usr/lib -lstdc++ -I/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-nec_20070618_1405/include
#
#/usr/bin/ld: skipping incompatible /usr/lib/libpthread.so when searching for -lpthread
#/usr/bin/ld: skipping incompatible /usr/lib/libpthread.a when searching for -lpthread
#/usr/bin/ld: skipping incompatible /usr/lib/libm.so when searching for -lm
#/usr/bin/ld: skipping incompatible /usr/lib/libm.a when searching for -lm
#/usr/bin/ld: skipping incompatible /usr/lib/libc.so when searching for -lc
#/usr/bin/ld: skipping incompatible /usr/lib/libc.a when searching for -lc -- OK
#
#$mpirun -np 2 ./deadlock1_normal
#Warning: No xauth data; using fake authentication data for X11 forwarding.
#Warning: No xauth data; using fake authentication data for X11 forwarding.
#Warning: No xauth data; using fake authentication data for X11 forwarding.
#/usr/X11R6/bin/xauth:  error in locking authority file /cacau/HLRS/hlrs/hpchimml/.Xauthority
#/usr/X11R6/bin/xauth:  error in locking authority file /cacau/HLRS/hlrs/hpchimml/.Xauthority
# I am rank 1 of 2 PEs
#  I am rank 0 of 2 PEs
#  noco157.nec: mpid: MPI program execution aborted
#  Killed by signal 15.
#  Killed by signal 15.
#  Killed by signal 15.
#  noco157.nec: mpid: MPI process terminated by exit(255)
#  noco157.nec: mpid: MPI process terminated by exit(255)
#  noco157.nec: mpid: MPI process terminated by exit(255)
#
#  OK
#
#$mpirun -np 3 ./deadlock1_marmot
#Warning: No xauth data; using fake authentication data for X11 forwarding.
#Warning: No xauth data; using fake authentication data for X11 forwarding.
#Warning: No xauth data; using fake authentication data for X11 forwarding.
#/usr/X11R6/bin/xauth:  error in locking authority file /cacau/HLRS/hlrs/hpchimml/.Xauthority
#/usr/X11R6/bin/xauth:  error in locking authority file /cacau/HLRS/hlrs/hpchimml/.Xauthority
# I am rank 0 of 2 PEs
#  I am rank 1 of 2 PEs
#  WARNING: all clients are pending! (Details see the LogFile)
#  noco157.nec: mpid: MPI program execution aborted
#  Killed by signal 15.
#  Killed by signal 15.
#  Killed by signal 15.
#  noco157.nec: mpid: MPI process terminated by exit(255)
#  noco157.nec: mpid: MPI process terminated by exit(255)
#  noco157.nec: mpid: MPI process terminated by exit(255)
#
#  OK
#
###############################################################################
#
#
#Intel 10.0
#voltaire_icc
#
#
#$mpicc -show deadlock1.c
#icc -fPIC -D_SMP_ -DUSE_INLINE -DEARLY_SEND_COMPLETION -DVIADEV_RPUT_SUPPORT -DLAZY_MEM_UNREGISTER -DRDMA_FAST_PATH -D_EM64T_ -DVBUF_TOTAL_SIZE=4096 -O3 -DMINISM -DMCST_SUPPORT -I/usr/mellanox/include -DHAVE_MPICHCONF_H -c deadlock1.c -I/usr/voltaire/mpi.lowmemory.icc.rsh/include
#icc -fPIC -D_SMP_ -DUSE_INLINE -DEARLY_SEND_COMPLETION -DVIADEV_RPUT_SUPPORT -DLAZY_MEM_UNREGISTER -DRDMA_FAST_PATH -D_EM64T_ -DVBUF_TOTAL_SIZE=4096 -O3 -DMINISM -DMCST_SUPPORT -I/usr/mellanox/include -DHAVE_MPICHCONF_H -Wl,-rpath-link -Wl,/usr/voltaire/mpi.lowmemory.icc.rsh/lib/shared -L/usr/voltaire/mpi.lowmemory.icc.rsh/lib/shared -L/usr/voltaire/mpi.lowmemory.icc.rsh/lib deadlock1.o -lmpich -lmpichfstub -L/usr/mellanox/lib -L/usr/voltaire/mpi.lowmemory.icc.rsh/lib -L/usr/voltaire/mpi.lowmemory.icc.rsh/lib/shared -lmtl_common -lvapi -lmosal -lmpga -lpthread
#
#$/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-voltaire_20070618_1352/bin/marmotcc -show deadlock1.c
#
#mpicc -show deadlock1.c -L/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-voltaire_20070618_1352/lib -lmarmot-profile -lmarmot-core -L/usr/voltaire/mpi.lowmemory.icc.rsh/lib -lpmpich -lmpich -lpmpich -lpthread -L/usr/lib -lstdc++ -I/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-voltaire_20070618_1352/include
#
#icc -fPIC -D_SMP_ -DUSE_INLINE -DEARLY_SEND_COMPLETION -DVIADEV_RPUT_SUPPORT -DLAZY_MEM_UNREGISTER -DRDMA_FAST_PATH -D_EM64T_ -DVBUF_TOTAL_SIZE=4096 -O3 -DMINISM -DMCST_SUPPORT -I/usr/mellanox/include -DHAVE_MPICHCONF_H -c deadlock1.c -L/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-voltaire_20070618_1352/lib -L/usr/voltaire/mpi.lowmemory.icc.rsh/lib -L/usr/lib -I/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-voltaire_20070618_1352/include -I/usr/voltaire/mpi.lowmemory.icc.rsh/include
#icc -fPIC -D_SMP_ -DUSE_INLINE -DEARLY_SEND_COMPLETION -DVIADEV_RPUT_SUPPORT -DLAZY_MEM_UNREGISTER -DRDMA_FAST_PATH -D_EM64T_ -DVBUF_TOTAL_SIZE=4096 -O3 -DMINISM -DMCST_SUPPORT -I/usr/mellanox/include -DHAVE_MPICHCONF_H -Wl,-rpath-link -Wl,/usr/voltaire/mpi.lowmemory.icc.rsh/lib/shared -L/usr/voltaire/mpi.lowmemory.icc.rsh/lib/shared -L/usr/voltaire/mpi.lowmemory.icc.rsh/lib deadlock1.o -L/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-voltaire_20070618_1352/lib -lmarmot-profile -lmarmot-core -L/usr/voltaire/mpi.lowmemory.icc.rsh/lib -lpmpich -lmpich -lpmpich -lpthread -L/usr/lib -lstdc++ -I/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-voltaire_20070618_1352/include -lmpich -lmpichfstub -L/usr/mellanox/lib -L/usr/voltaire/mpi.lowmemory.icc.rsh/lib -L/usr/voltaire/mpi.lowmemory.icc.rsh/lib/shared -lmtl_common -lvapi -lmosal -lmpga -lpthread
#
#$mpicc -o deadlock1_normal deadlock1.c
#/opt/intel/compiler/10.0/cce/lib/libimf.so: warning: warning: feupdateenv is not implemented and will always fail -- OK
#
#$/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-voltaire_20070618_1352/bin/marmotcc -o deadlock1_marmot deadlock1.c
#
#mpicc -o deadlock1_marmot deadlock1.c -L/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-voltaire_20070618_1352/lib -lmarmot-profile -lmarmot-core -L/usr/voltaire/mpi.lowmemory.icc.rsh/lib -lpmpich -lmpich -lpmpich -lpthread -L/usr/lib -lstdc++ -I/cacau/HLRS/hlrs/hpchimml/marmot_build_intel-voltaire_20070618_1352/include
#
#ld: skipping incompatible /usr/lib/libpthread.so when searching for -lpthread
#ld: skipping incompatible /usr/lib/libpthread.a when searching for -lpthread
#ld: skipping incompatible /usr/lib/libpthread.so when searching for -lpthread
#ld: skipping incompatible /usr/lib/libpthread.a when searching for -lpthread
#ld: skipping incompatible /usr/lib/libm.so when searching for -lm
#ld: skipping incompatible /usr/lib/libm.a when searching for -lm
#ld: skipping incompatible /usr/lib/libc.so when searching for -lc
#ld: skipping incompatible /usr/lib/libc.a when searching for -lc
#ld: skipping incompatible /usr/lib/libdl.so when searching for -ldl
#ld: skipping incompatible /usr/lib/libdl.a when searching for -ldl
#ld: skipping incompatible /usr/lib/libc.so when searching for -lc
#ld: skipping incompatible /usr/lib/libc.a when searching for -lc
#/opt/intel/compiler/10.0/cce/lib/libimf.so: warning: warning: feupdateenv is not implemented and will always fail -- OK
#
#
#$mpirun -np 2 -machinefile $PBS_NODEFILE ./deadlock1_normal
#/usr/voltaire/mpi.lowmemory.icc.rsh/bin/mpirun.vapi.args: line 112: syntax error: unexpected end of file
#/usr/voltaire/mpi.lowmemory.icc.rsh/bin/mpirun.vapi.args: line 112: syntax error: unexpected end of file
#Warning: No xauth data; using fake authentication data for X11 forwarding.
#Warning: No xauth data; using fake authentication data for X11 forwarding.
#/usr/X11R6/bin/xauth:  error in locking authority file /cacau/HLRS/hlrs/hpchimml/.Xauthority
# I am rank 0 of 2 PEs
#  I am rank 1 of 2 PEs
#  Signal 2 received.
#  Cleaning up all processes ...
#  Killed by signal 2.
#  Killed by signal 2.
#  done.
#
#  OK
#
#$mpirun -np 3 -machinefile $PBS_NODEFILE ./deadlock1_marmot
#/usr/voltaire/mpi.lowmemory.icc.rsh/bin/mpirun.vapi.args: line 112: syntax error: unexpected end of file
#/usr/voltaire/mpi.lowmemory.icc.rsh/bin/mpirun.vapi.args: line 112: syntax error: unexpected end of file
#Warning: No xauth data; using fake authentication data for X11 forwarding.
#Warning: No xauth data; using fake authentication data for X11 forwarding.
#Warning: No xauth data; using fake authentication data for X11 forwarding.
#/usr/X11R6/bin/xauth:  error in locking authority file /cacau/HLRS/hlrs/hpchimml/.Xauthority
#/usr/X11R6/bin/xauth:  error in locking authority file /cacau/HLRS/hlrs/hpchimml/.Xauthority
# I am rank 0 of 2 PEs
#  I am rank 1 of 2 PEs
#  WARNING: all clients are pending! (Details see the LogFile)
#  Killed by signal 2.
#  Signal 2 received.
#  Cleaning up all processes ...
#  Killed by signal 2.
#  Killed by signal 2.
#  done.
#
# OK
#
###############################################################################

